/*****************************************************************************
*  Copyright Statement:
*  --------------------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of MediaTek Inc. (C) 2010
*
*  BY OPENING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
*  THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
*  RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON
*  AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES,
*  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
*  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.
*  NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH RESPECT TO THE
*  SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, INCORPORATED IN, OR
*  SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK ONLY TO SUCH
*  THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
*  NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S
*  SPECIFICATION OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
*
*  BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE
*  LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE,
*  AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE,
*  OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE CHARGE PAID BY BUYER TO
*  MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
*
*  THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE
*  WITH THE LAWS OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF
*  LAWS PRINCIPLES.  ANY DISPUTES, CONTROVERSIES OR CLAIMS ARISING THEREOF AND
*  RELATED THERETO SHALL BE SETTLED BY ARBITRATION IN SAN FRANCISCO, CA, UNDER
*  THE RULES OF THE INTERNATIONAL CHAMBER OF COMMERCE (ICC).
*
*****************************************************************************/

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   image_sensor_SP0A19.c
 *
 * Project:
 * --------
 *  
 *
 * Description:
 * ------------
 *  Image sensor driver function
 *  V1.0.0
 *
 * Author:
 * -------
 * Mormo
 *
 *============================================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 *  * $Log$
 * 2011/06/13 Firstly Released By Mormo;
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/
#include "drv_comm.h"
#include "isp_comm_if.h"
#include "camera_sccb.h"
#include "image_sensor.h"
#include "cal_comm_def.h"
#include "isp_if.h"

/* #define __SP0A19_DEBUG_TRACE__ */
#define SP0A19_SET_PAGE0 CamWriteCmosSensor(0xfd , 0x00)
#define SP0A19_SET_PAGE1 CamWriteCmosSensor(0xfd , 0x01)

kal_bool SP0A19_CAM_BANDING_50HZ = KAL_FALSE;
kal_bool SP0A19_NIGHT_ENABLE = KAL_FALSE;
kal_bool SP0A19_video_mode = KAL_FALSE;
static struct
{
  kal_bool BypassAe;
  kal_bool BypassAwb;
  kal_uint8 Ctrl3A;
  kal_uint8 AecMode3;
  kal_uint16 VideoFps;
  kal_uint32 IntClk; /* internal clock = half of pclk */
  kal_uint16 FrameHeight;
  kal_uint16 LineLength;
  IMAGE_SENSOR_INDEX_ENUM SensorIdx;
  sensor_data_struct *NvramData;
} SP0A19Sensor;

/*************************************************************************
* FUNCTION
*    SP0A19HalfAdjust
*
* DESCRIPTION
*    This function dividend / divisor and use round-up.
*
* PARAMETERS
*    DividEnd
*    Divisor
*
* RETURNS
*    [DividEnd / Divisor]
*
* LOCAL AFFECTED
*
*************************************************************************/
__inline static kal_uint32 SP0A19HalfAdjust(kal_uint32 DividEnd, kal_uint32 Divisor)
{
  return (DividEnd + (Divisor >> 1)) / Divisor; /* that is [DividEnd / Divisor + 0.5]*/
}

/*************************************************************************
* FUNCTION
*    SP0A19SetMirror
*
* DESCRIPTION
*    This function set the Mirror to the CMOS sensor
*
* PARAMETERS
*    Mirror
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19SetMirror(kal_uint8 Mirror)
{SP0A19_SET_PAGE0;
  switch (Mirror)
  {
  case IMAGE_SENSOR_MIRROR_H:
    CamWriteCmosSensor(0x31, 0x20);
    break;
  case IMAGE_SENSOR_MIRROR_V:
    CamWriteCmosSensor(0x31, 0x40);
    break;
  case IMAGE_SENSOR_MIRROR_HV:
    CamWriteCmosSensor(0x31, 0x60);
    break;
  default:
    CamWriteCmosSensor(0x31, 0x00);
  }
}

/*************************************************************************
* FUNCTION
*    SP0A19SetClock
*
* DESCRIPTION
*    This function set sensor vt clock and op clock
*
* PARAMETERS
*    Clk: vt clock
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19SetClock(kal_uint32 Clk)
{
  kal_uint8 ClkDiv;
  
  if (SP0A19Sensor.IntClk == Clk)
  {
    return;
  }
  SP0A19Sensor.IntClk = Clk;
  switch (SP0A19Sensor.IntClk)
  {
  case SP0A19_MCLK / 2: ClkDiv = 0x02; break; /* pclk = mclk */
  case SP0A19_MCLK / 4: ClkDiv = 0x04; break; /* pclk = mclk / 2 */
  case SP0A19_MCLK / 8: ClkDiv = 0x06; break; /* pclk = mclk / 4 */
  default: ASSERT(0);
  }
  SP0A19_SET_PAGE0;
  CamWriteCmosSensor(0x30, ClkDiv);
}

/*************************************************************************
* FUNCTION
*    SP0A19WriteShutter
*
* DESCRIPTION
*    This function apply Shutter to sensor
*
* PARAMETERS
*    Shutter: integration time
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19WriteShutter(kal_uint32 Shutter)
{SP0A19_SET_PAGE0;
  CamWriteCmosSensor(0x03, Shutter >> 8);
  CamWriteCmosSensor(0x04, Shutter);
}

/*************************************************************************
* FUNCTION
*    SP0A19ReadShutter
*
* DESCRIPTION
*    This function get shutter from sensor
*
* PARAMETERS
*    None
*
* RETURNS
*    Shutter: integration time
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint16 SP0A19ReadShutter(void)
{SP0A19_SET_PAGE0;
  return (CamReadCmosSensor(0x03) << 8)|CamReadCmosSensor(0x04);
}

/*************************************************************************
* FUNCTION
*    SP0A19AeEnable
*
* DESCRIPTION
*    disable/enable AE
*
* PARAMETERS
*    Enable
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19AeEnable(kal_bool Enable)
{
  const kal_bool AeEnable = (SP0A19Sensor.AecMode3&0x05) ? KAL_TRUE : KAL_FALSE;
  
  if (SP0A19Sensor.BypassAe)
  {
    Enable = KAL_FALSE;
  }
  if (AeEnable == Enable)
  {
    return;
  }
  if (Enable)
  {
    SP0A19Sensor.AecMode3 |= 0x05;
  }
  else
  {
    SP0A19Sensor.AecMode3 &= 0x10;
  }
  SP0A19_SET_PAGE0;
  CamWriteCmosSensor(0x32, SP0A19Sensor.AecMode3);
}

/*************************************************************************
* FUNCTION
*    SP0A19AwbEnable
*
* DESCRIPTION
*    disable/enable awb
*
* PARAMETERS
*    Enable
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19AwbEnable(kal_bool Enable)
{
  const kal_bool AwbEnable = (SP0A19Sensor.Ctrl3A&0x10) ? KAL_TRUE : KAL_FALSE;
  
  if (SP0A19Sensor.BypassAwb)
  {
    Enable = KAL_FALSE;
  }
  if (AwbEnable == Enable)
  {
    return;
  }
  if (Enable)
  {
    SP0A19Sensor.Ctrl3A |= 0x10;
  }
  else
  {
    SP0A19Sensor.Ctrl3A &= 0x05;
  }
    SP0A19_SET_PAGE0;
  CamWriteCmosSensor(0x32, SP0A19Sensor.Ctrl3A);
}

/*************************************************************************
* FUNCTION
*    SP0A19SetBaseShutter
*
* DESCRIPTION
*    This function set base shutter to the CMOS sensor
*
* PARAMETERS
*    Banding: 50Hz or 60Hz
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19SetBaseShutter(kal_uint8 Banding)
{
}

/*************************************************************************
* FUNCTION
*    SP0A19SetDummy
*
* DESCRIPTION
*    This function add DummyPixel and DummyLine.
*
* PARAMETERS
*    DummyPixel
*    DummyLine
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19SetDummy(kal_uint16 DummyPixel, kal_uint16 DummyLine)
{
}

/*************************************************************************
* FUNCTION
*    SP0A19SetVideoFps
*
* DESCRIPTION
*    This function calculate & fix frame rate in video mode
*
* PARAMETERS
*    Fps: video frame rate, base on SP0A19_FPS(x)
*
* RETURNS
*    Fps: real frame rate set by sensor
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint16 SP0A19SetVideoFps(kal_uint16 Fps)
{
}

/*************************************************************************
* FUNCTION
*    SP0A19SetMinFps
*
* DESCRIPTION
*    This function calculate & set min frame rate in camera preview mode
*
* PARAMETERS
*    Fps: min frame rate, base on SP0A19_FPS(x)
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19SetMinFps(kal_uint16 Fps)
{
}

/*************************************************************************
* FUNCTION
*    SP0A19Ev
*
* DESCRIPTION
*    Exposure setting.
*
* PARAMETERS
*    In, Out
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19Ev(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Exposure = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    Exposure->IsSupport = KAL_TRUE;
    Exposure->ItemCount = 10;
    Exposure->SupportItem[0] = CAM_EV_NEG_4_3;
    Exposure->SupportItem[1] = CAM_EV_NEG_3_3;
    Exposure->SupportItem[2] = CAM_EV_NEG_2_3;
    Exposure->SupportItem[3] = CAM_EV_NEG_1_3;
    Exposure->SupportItem[4] = CAM_EV_ZERO;
    Exposure->SupportItem[5] = CAM_EV_POS_1_3;
    Exposure->SupportItem[6] = CAM_EV_POS_2_3;
    Exposure->SupportItem[7] = CAM_EV_POS_3_3;
    Exposure->SupportItem[8] = CAM_EV_POS_4_3;
    Exposure->SupportItem[9] = CAM_EV_NIGHT_SHOT;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    static const kal_uint8 Data[9][1]=
    {
      {0xc0}, /* EV -4 */
      {0xd0}, /* EV -3 */
      {0xe0}, /* EV -2 */
      {0xf0}, /* EV -1 */
      {0x00}, /* EV 0 */
      {0x10}, /* EV +1 */
      {0x20}, /* EV +2 */
      {0x30}, /* EV +3 */
      {0x40}, /* EV +4 */
    };
    kal_uint8 i;
    
    switch (In->FeatureSetValue)
    {
    case CAM_EV_NEG_4_3: i = 0; break;
    case CAM_EV_NEG_3_3: i = 1; break;
    case CAM_EV_NEG_2_3: i = 2; break;
    case CAM_EV_NEG_1_3: i = 3; break;
    case CAM_EV_ZERO:    i = 4; break;
    case CAM_EV_POS_1_3: i = 5; break;
    case CAM_EV_POS_2_3: i = 6; break;
    case CAM_EV_POS_3_3: i = 7; break;
    case CAM_EV_POS_4_3: i = 8; break;
    default:
      return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    }
    SP0A19_SET_PAGE0;
    CamWriteCmosSensor(0xdc, Data[i][0]); /* YBright */
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19Wb
*
* DESCRIPTION
*    Wb setting.
*
* PARAMETERS
*    In, Out
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19Wb(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Wb = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    Wb->IsSupport = KAL_TRUE;
    Wb->ItemCount = 6;
    Wb->SupportItem[0] = CAM_WB_AUTO;
    Wb->SupportItem[1] = CAM_WB_CLOUD;
    Wb->SupportItem[2] = CAM_WB_DAYLIGHT;
    Wb->SupportItem[3] = CAM_WB_INCANDESCENCE;
    Wb->SupportItem[4] = CAM_WB_FLUORESCENT;
    Wb->SupportItem[5] = CAM_WB_TUNGSTEN;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    static const kal_uint8 AwbGain[5][2]=
    { /* R gain, B gain. base: 2.6bits, 0x40=1x,0x60=1.5x */
     {0xbf,0x89}, /* cloud */
     {0xbc,0x5d}, /* daylight */
     {0x89,0xb8}, /* INCANDESCENCE *///ȹ 
     {0xb5,0xa5}, /* FLUORESCENT */ //ӫƺչѶдĲͬ4200-5000
     {0xaf,0x99}, /* TUNGSTEN */   //˿//2800K
    };
    static kal_uint8 BakRGain, BakBGain;
    kal_uint8 RGain, BGain;
    kal_int8 i = sizeof(AwbGain) / sizeof(AwbGain[0]);
    SP0A19_SET_PAGE1;
    RGain = CamReadCmosSensor(0x28);
    BGain = CamReadCmosSensor(0x29);
    while (--i >= 0)
    {
      if (RGain == AwbGain[i][0] && BGain == AwbGain[i][1])
      {
        break;
      }
    }
    if (i < 0)
    {
      BakRGain = RGain;
      BakBGain = BGain;
    }
    switch (In->FeatureSetValue)
    {
    case CAM_WB_AUTO:
      SP0A19_SET_PAGE1;	
      CamWriteCmosSensor(0x28, BakRGain); /* for AWB can adjust back */
      CamWriteCmosSensor(0x29, BakBGain);
      SP0A19_SET_PAGE0;
      SP0A19AwbEnable(KAL_TRUE);
      return MM_ERROR_NONE;
    case CAM_WB_CLOUD:         i = 0; break;
    case CAM_WB_DAYLIGHT:      i = 1; break;
    case CAM_WB_INCANDESCENCE: i = 2; break;
    case CAM_WB_FLUORESCENT:   i = 3; break;
    case CAM_WB_TUNGSTEN:      i = 4; break;
    default:
      return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    }
    SP0A19AwbEnable(KAL_FALSE);
    SP0A19_SET_PAGE1;	
    CamWriteCmosSensor(0x28, AwbGain[i][0]); /* awb R gain */
    CamWriteCmosSensor(0x29, AwbGain[i][1]); /* awb B gain */
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19Effect
*
* DESCRIPTION
*    Effect setting.
*
* PARAMETERS
*    In, Out
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19Effect(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Effect = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    Effect->IsSupport = KAL_TRUE;
    Effect->ItemCount = 7;
    Effect->SupportItem[0] = CAM_EFFECT_ENC_NORMAL;
    Effect->SupportItem[1] = CAM_EFFECT_ENC_GRAYSCALE;
    Effect->SupportItem[2] = CAM_EFFECT_ENC_SEPIA;
    Effect->SupportItem[3] = CAM_EFFECT_ENC_SEPIAGREEN;
    Effect->SupportItem[4] = CAM_EFFECT_ENC_SEPIABLUE;
    Effect->SupportItem[5] = CAM_EFFECT_ENC_COLORINV;
    Effect->SupportItem[6] = CAM_EFFECT_ENC_GRAYINV;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    static const kal_uint8 Data[7][3]=
    {
      {0x00,0x80,0x80}, /* NORMAL */
      {0x20,0x80,0x80}, /* GRAYSCALE */
      {0x10,0xc0,0x20}, /* SEPIA */
      {0x10,0x20,0x20}, /* SEPIAGREEN */
      {0x10,0x20,0xf0}, /* SEPIABLUE */
      {0x04,0x80,0x80}, /* COLORINV */
      {0x08,0x80,0x80}, /* GRAYINV */
    };
    kal_uint8 i;
    
    switch (In->FeatureSetValue)
    {
    case CAM_EFFECT_ENC_NORMAL:     i = 0; break;
    case CAM_EFFECT_ENC_GRAYSCALE:  i = 1; break;
    case CAM_EFFECT_ENC_SEPIA:      i = 2; break;
    case CAM_EFFECT_ENC_SEPIAGREEN: i = 3; break;
    case CAM_EFFECT_ENC_SEPIABLUE:  i = 4; break;
    case CAM_EFFECT_ENC_COLORINV:   i = 5; break;
    case CAM_EFFECT_ENC_GRAYINV:    i = 6; break;
    default:
      return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    }
    SP0A19_SET_PAGE0;
    CamWriteCmosSensor(0x62, Data[i][0]); /* special effect */
    CamWriteCmosSensor(0x63, Data[i][1]); /* fixed Cb */
    CamWriteCmosSensor(0x64, Data[i][2]); /* fixed Cr */
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19StillCaptureSize
*
* DESCRIPTION
*    This function get capture size
*
* PARAMETERS
*    In, Out
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
MM_ERROR_CODE_ENUM SP0A19StillCaptureSize(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  const P_CAL_FEATURE_TYPE_ENUM_STRUCT CapSize = &Out->FeatureInfo.FeatureEnum;
  
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_SUPPORT;
    CapSize->IsSupport = KAL_TRUE;
    CapSize->ItemCount = 2;
    CapSize->SupportItem[0] = CAM_IMAGE_SIZE_WALLPAPER;
    CapSize->SupportItem[1] = CAM_IMAGE_SIZE_VGA;
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19Banding
*
* DESCRIPTION
*    This function get banding setting
*
* PARAMETERS
*    In, Out
*
* RETURNS
*    error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19Banding(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  const P_CAL_FEATURE_TYPE_ENUM_STRUCT Banding = &Out->FeatureInfo.FeatureEnum;
  
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    Banding->IsSupport = KAL_TRUE;
    Banding->ItemCount = 2;
    Banding->SupportItem[0] = CAM_BANDING_50HZ;
    Banding->SupportItem[1] = CAM_BANDING_60HZ;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
	switch (In->FeatureSetValue)
	{
		case CAM_BANDING_50HZ:
		SP0A19_CAM_BANDING_50HZ = KAL_TRUE;
		break;
		
	case CAM_BANDING_60HZ:
		SP0A19_CAM_BANDING_50HZ = KAL_FALSE;
		break;

	default:
       	return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
			
	}

	if(SP0A19_video_mode == KAL_FALSE) //CAM 
	{
		if(SP0A19_NIGHT_ENABLE == KAL_FALSE)  //DALLIGHT
		{
			CamWriteCmosSensor(0xfd,0x0 );
			CamWriteCmosSensor(0xb2,0x20);
			CamWriteCmosSensor(0xb3,0x1f);

			if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
			{

			//capture preview daylight 24M 50hz 20-8FPS maxgain:  
			  CamWriteCmosSensor(0xfd,0x00);
			  CamWriteCmosSensor(0x03,0x01);
			  CamWriteCmosSensor(0x04,0x32);
			  CamWriteCmosSensor(0x06,0x00);
			  CamWriteCmosSensor(0x09,0x01);
			  CamWriteCmosSensor(0x0a,0x46);
			  CamWriteCmosSensor(0xf0,0x66);
			  CamWriteCmosSensor(0xf1,0x00);
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0x90,0x0c);
			  CamWriteCmosSensor(0x92,0x01);
			  CamWriteCmosSensor(0x98,0x66);
			  CamWriteCmosSensor(0x99,0x00);
			  CamWriteCmosSensor(0x9a,0x01);
			  CamWriteCmosSensor(0x9b,0x00);
			//Status  
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0xce,0xc8);
			  CamWriteCmosSensor(0xcf,0x04);
			  CamWriteCmosSensor(0xd0,0xc8);
			  CamWriteCmosSensor(0xd1,0x04);
			  CamWriteCmosSensor(0xfd,0x00);	
					dbg_print("SP0A19_CAM_DAY_50HZ  \r\n");
					}
			else  //60HZ
			{
			SCI_TRACE_LOW("normal mode 60hz\r\n");
			//capture preview daylight 24M 60hz 20-8FPS maxgain: 
			  CamWriteCmosSensor(0xfd,0x00);
			  CamWriteCmosSensor(0x03,0x00);
			  CamWriteCmosSensor(0x04,0xff);
			  CamWriteCmosSensor(0x06,0x00);
			  CamWriteCmosSensor(0x09,0x01);
			  CamWriteCmosSensor(0x0a,0x46);
			  CamWriteCmosSensor(0xf0,0x55);
			  CamWriteCmosSensor(0xf1,0x00);
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0x90,0x0f);
			  CamWriteCmosSensor(0x92,0x01);
			  CamWriteCmosSensor(0x98,0x55);
			  CamWriteCmosSensor(0x99,0x00);
			  CamWriteCmosSensor(0x9a,0x01);
			  CamWriteCmosSensor(0x9b,0x00);
			  //Status  
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0xce,0xfb);
			  CamWriteCmosSensor(0xcf,0x04);
			  CamWriteCmosSensor(0xd0,0xfb);
			  CamWriteCmosSensor(0xd1,0x04);
			  CamWriteCmosSensor(0xfd,0x00);							
					dbg_print("SP0A19_CAM_DAY_60HZ  \r\n");

			}
		}
		else                        // night
		{

			CamWriteCmosSensor(0xfd,0x0 );
			CamWriteCmosSensor(0xb2,0x25);
			CamWriteCmosSensor(0xb3,0x1f);

		
			if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
			{

			//capture preview night 24M 50hz 20-6FPS maxgain:
			  CamWriteCmosSensor(0xfd,0x00);
			  CamWriteCmosSensor(0x03,0x01);
			  CamWriteCmosSensor(0x04,0x32);
			  CamWriteCmosSensor(0x06,0x00);
			  CamWriteCmosSensor(0x09,0x01);
			  CamWriteCmosSensor(0x0a,0x46);
			  CamWriteCmosSensor(0xf0,0x66);
			  CamWriteCmosSensor(0xf1,0x00);
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0x90,0x10);
			  CamWriteCmosSensor(0x92,0x01);
			  CamWriteCmosSensor(0x98,0x66);
			  CamWriteCmosSensor(0x99,0x00);
			  CamWriteCmosSensor(0x9a,0x01);
			  CamWriteCmosSensor(0x9b,0x00);
			  //Status  
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0xce,0x60);
			  CamWriteCmosSensor(0xcf,0x06);
			  CamWriteCmosSensor(0xd0,0x60);
			  CamWriteCmosSensor(0xd1,0x06);
			  CamWriteCmosSensor(0xfd,0x00);
			  dbg_print("SP0A19_CAM_NIGHT_50HZ  \r\n");
			}
			else  //60HZ
			{
			//capture preview night 24M 60hz 20-6FPS maxgain:
			  CamWriteCmosSensor(0xfd,0x00);
			  CamWriteCmosSensor(0x03,0x00);
			  CamWriteCmosSensor(0x04,0xff);
			  CamWriteCmosSensor(0x06,0x00);
			  CamWriteCmosSensor(0x09,0x01);
			  CamWriteCmosSensor(0x0a,0x46);
			  CamWriteCmosSensor(0xf0,0x55);
			  CamWriteCmosSensor(0xf1,0x00);
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0x90,0x14);
			  CamWriteCmosSensor(0x92,0x01);
			  CamWriteCmosSensor(0x98,0x55);
			  CamWriteCmosSensor(0x99,0x00);
			  CamWriteCmosSensor(0x9a,0x01);
			  CamWriteCmosSensor(0x9b,0x00);
			  //Status  
			  CamWriteCmosSensor(0xfd,0x01);
			  CamWriteCmosSensor(0xce,0xa4);
			  CamWriteCmosSensor(0xcf,0x06);
			  CamWriteCmosSensor(0xd0,0xa4);
			  CamWriteCmosSensor(0xd1,0x06);
			  CamWriteCmosSensor(0xfd,0x00);
			  dbg_print("SP0A19_CAM_NIGHT_60HZ  \r\n");
			}
		}
	}
	else          //video 
	{
		if(SP0A19_NIGHT_ENABLE == KAL_FALSE)  //DALLIGHT
		{

			CamWriteCmosSensor(0xfd,0x0 );
			CamWriteCmosSensor(0xb2,0x20);
			CamWriteCmosSensor(0xb3,0x1f);

		
			if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
			{
						//Video record daylight 24M 50hz 14-14FPS maxgain
						CamWriteCmosSensor(0xfd,0x00);
						CamWriteCmosSensor(0x03,0x00);
						CamWriteCmosSensor(0x04,0xd8);
						CamWriteCmosSensor(0x06,0x00);
						CamWriteCmosSensor(0x09,0x03);
						CamWriteCmosSensor(0x0a,0x31);
						CamWriteCmosSensor(0xf0,0x48);
						CamWriteCmosSensor(0xf1,0x00);
						CamWriteCmosSensor(0xfd,0x01);
						CamWriteCmosSensor(0x90,0x07);
						CamWriteCmosSensor(0x92,0x01);
						CamWriteCmosSensor(0x98,0x48);
						CamWriteCmosSensor(0x99,0x00);
						CamWriteCmosSensor(0x9a,0x01);
						CamWriteCmosSensor(0x9b,0x00);
						//Status  
						CamWriteCmosSensor(0xfd,0x01);
						CamWriteCmosSensor(0xce,0xf8);
						CamWriteCmosSensor(0xcf,0x01);
						CamWriteCmosSensor(0xd0,0xf8);
						CamWriteCmosSensor(0xd1,0x01);
						CamWriteCmosSensor(0xfd,0x00);             
					dbg_print("SP0A19_VID_DAY_50HZ  \r\n");
				}
				else  //60HZ
				{
					//Video record daylight 24M 60Hz 14-14FPS maxgain:
					  CamWriteCmosSensor(0xfd,0x00);
					  CamWriteCmosSensor(0x03,0x00);
					  CamWriteCmosSensor(0x04,0xb4);
					  CamWriteCmosSensor(0x06,0x00);
					  CamWriteCmosSensor(0x09,0x03);
					  CamWriteCmosSensor(0x0a,0x31);
					  CamWriteCmosSensor(0xf0,0x3c);
					  CamWriteCmosSensor(0xf1,0x00);
					  CamWriteCmosSensor(0xfd,0x01);
					  CamWriteCmosSensor(0x90,0x08);
					  CamWriteCmosSensor(0x92,0x01);
					  CamWriteCmosSensor(0x98,0x3c);
					  CamWriteCmosSensor(0x99,0x00);
					  CamWriteCmosSensor(0x9a,0x01);
					  CamWriteCmosSensor(0x9b,0x00);
					  //Status  
					  CamWriteCmosSensor(0xfd,0x01);
					  CamWriteCmosSensor(0xce,0xe0);
					  CamWriteCmosSensor(0xcf,0x01);
					  CamWriteCmosSensor(0xd0,0xe0);
					  CamWriteCmosSensor(0xd1,0x01);
					  CamWriteCmosSensor(0xfd,0x00);	
					  dbg_print("SP0A19_VID_DAY_60HZ  \r\n");
					}

		}
		else                        // night
		{

			CamWriteCmosSensor(0xfd,0x0 );
			CamWriteCmosSensor(0xb2,0x25);
			CamWriteCmosSensor(0xb3,0x1f);

			if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
			{
				//SI13_SP0A19 24M 1Ƶ 50Hz 8.0392-8fps AE_Parameters_ maxgain
					dbg_print("SP0A19_VID_NIGTHT_50HZ  \r\n");
					CamWriteCmosSensor(0xfd,0x00);
					CamWriteCmosSensor(0x03,0x00);
					CamWriteCmosSensor(0x04,0x7b);
					CamWriteCmosSensor(0x06,0x00);
					CamWriteCmosSensor(0x09,0x08);
					CamWriteCmosSensor(0x0a,0x1d);
					CamWriteCmosSensor(0xf0,0x29);
					CamWriteCmosSensor(0xf1,0x00);
					CamWriteCmosSensor(0xfd,0x01);
					CamWriteCmosSensor(0x90,0x0c);
					CamWriteCmosSensor(0x92,0x01);
					CamWriteCmosSensor(0x98,0x29);
					CamWriteCmosSensor(0x99,0x00);
					CamWriteCmosSensor(0x9a,0x01);
					CamWriteCmosSensor(0x9b,0x00);
					//Status  
					CamWriteCmosSensor(0xfd,0x01);
					CamWriteCmosSensor(0xce,0xec);
					CamWriteCmosSensor(0xcf,0x01);
					CamWriteCmosSensor(0xd0,0xec);
					CamWriteCmosSensor(0xd1,0x01);
					CamWriteCmosSensor(0xfd,0x00);
					
			}
			else  //60HZ
			{
				//SI13_SP0A19 24M 1Ƶ 60Hz 8.2353-8fps AE_Parameters_ maxgain
				dbg_print("SP0A19_VID_NIGTHT_60HZ  \r\n");
				CamWriteCmosSensor(0xfd,0x00);
				CamWriteCmosSensor(0x03,0x00);
				CamWriteCmosSensor(0x04,0x69);
				CamWriteCmosSensor(0x06,0x00);
				CamWriteCmosSensor(0x09,0x07);
				CamWriteCmosSensor(0x0a,0xd7);
				CamWriteCmosSensor(0xf0,0x23);
				CamWriteCmosSensor(0xf1,0x00);
				CamWriteCmosSensor(0xfd,0x01);
				CamWriteCmosSensor(0x90,0x0f);
				CamWriteCmosSensor(0x92,0x01);
				CamWriteCmosSensor(0x98,0x23);
				CamWriteCmosSensor(0x99,0x00);
				CamWriteCmosSensor(0x9a,0x01);
				CamWriteCmosSensor(0x9b,0x00);
				//Status  
				CamWriteCmosSensor(0xfd,0x01);
				CamWriteCmosSensor(0xce,0x0d);
				CamWriteCmosSensor(0xcf,0x02);
				CamWriteCmosSensor(0xd0,0x0d);
				CamWriteCmosSensor(0xd1,0x02);
				CamWriteCmosSensor(0xfd,0x00);
			}
		}
	}
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19SceneMode
*
* DESCRIPTION
*    This function get/set scene mode of sensor.
*
* PARAMETERS
*    IsCam: camera scene mode or not
*    In, Out
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19SceneMode(kal_bool IsCam, P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT SceneMode = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_SUPPORT;
    SceneMode->IsSupport = KAL_TRUE;
    SceneMode->ItemCount = 2;
    if (IsCam)
    {
      SceneMode->SupportItem[0] = CAM_AUTO_DSC;
      SceneMode->SupportItem[1] = CAM_NIGHTSCENE;
    }
    else
    {
      SceneMode->SupportItem[0] = CAM_VIDEO_AUTO;
      SceneMode->SupportItem[1] = CAM_VIDEO_NIGHT;
    }
  }
 else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
 
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19NightMode
*
* DESCRIPTION
*    This function set night mode to sensor.
*
* PARAMETERS
*    Enable
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19NightMode(kal_uint32 banding_para, kal_bool night_mode)
{
	if(banding_para == 1)
		SP0A19_CAM_BANDING_50HZ = 0;
	else
		SP0A19_CAM_BANDING_50HZ=1;
	if(night_mode == 1)
		SP0A19_NIGHT_ENABLE = 1;
	else
		SP0A19_NIGHT_ENABLE = 0 ;
}

/*************************************************************************
* FUNCTION
*  SP0A19GammaSelect
*
* DESCRIPTION
*  This function config the gamma curve fitting for  
*  different LCD performances.
*
* PARAMETERS
*  level: different gamma curves.
*         Typical level value is from 1 to 5 which 
*         means small level gamma to high level gamma.
*
* RETURNS
*  NONE
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19GammaSelect(kal_uint8 level)
{
}

/*************************************************************************
* FUNCTION
*    SP0A19InitialSetting
*
* DESCRIPTION
*    This function initialize the registers of CMOS sensor
*
* PARAMETERS
*    None
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19InitialSetting(void)
{

  CamWriteCmosSensor(0x1C,0x28);
  CamWriteCmosSensor(0x32,0x00);
  CamWriteCmosSensor(0x0f,0x2f);
  CamWriteCmosSensor(0x10,0x2e);
  CamWriteCmosSensor(0x11,0x00);
  CamWriteCmosSensor(0x12,0x18);
  CamWriteCmosSensor(0x13,0x2f);
  CamWriteCmosSensor(0x14,0x00);
  CamWriteCmosSensor(0x15,0x3f);
  CamWriteCmosSensor(0x16,0x00);
  CamWriteCmosSensor(0x17,0x18);
  CamWriteCmosSensor(0x25,0x40);
  CamWriteCmosSensor(0x1a,0x0b);
  CamWriteCmosSensor(0x1b,0xc );
  CamWriteCmosSensor(0x1e,0xb );
  CamWriteCmosSensor(0x20,0x3f);
  CamWriteCmosSensor(0x21,0x13);
  CamWriteCmosSensor(0x22,0x19);
  CamWriteCmosSensor(0x26,0x1a);
  CamWriteCmosSensor(0x27,0xab);
  CamWriteCmosSensor(0x28,0xfd);
  CamWriteCmosSensor(0x30,0x00);
  CamWriteCmosSensor(0x31,0x10);
  CamWriteCmosSensor(0xfb,0x31);
  CamWriteCmosSensor(0x1f,0x08);  
  
//Blacklevel
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0x65,0x00);//06
  CamWriteCmosSensor(0x66,0x00);//06
  CamWriteCmosSensor(0x67,0x00);//06
  CamWriteCmosSensor(0x68,0x00);//06
  CamWriteCmosSensor(0x45,0x00);
  CamWriteCmosSensor(0x46,0x0f);
  
  //ae setting
	CamWriteCmosSensor(0xfd,0x00);
	CamWriteCmosSensor(0x03,0x01);
	CamWriteCmosSensor(0x04,0x32);
	CamWriteCmosSensor(0x06,0x00);
	CamWriteCmosSensor(0x09,0x01);
	CamWriteCmosSensor(0x0a,0x46);
	CamWriteCmosSensor(0xf0,0x66);
	CamWriteCmosSensor(0xf1,0x00);
	CamWriteCmosSensor(0xfd,0x01);
	CamWriteCmosSensor(0x90,0x0c);
	CamWriteCmosSensor(0x92,0x01);
	CamWriteCmosSensor(0x98,0x66);
	CamWriteCmosSensor(0x99,0x00);
	CamWriteCmosSensor(0x9a,0x01);
	CamWriteCmosSensor(0x9b,0x00);
  
//Status
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0xce,0xc8);
  CamWriteCmosSensor(0xcf,0x04);
  CamWriteCmosSensor(0xd0,0xc8);
  CamWriteCmosSensor(0xd1,0x04); 
   
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0xc4,0x56);
  CamWriteCmosSensor(0xc5,0x8f);
  CamWriteCmosSensor(0xca,0x30);
  CamWriteCmosSensor(0xcb,0x45);
  CamWriteCmosSensor(0xcc,0x70);
  CamWriteCmosSensor(0xcd,0x48);
  CamWriteCmosSensor(0xfd,0x00);
  
 //lsc  for st
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0x35,0x15);
  CamWriteCmosSensor(0x36,0x15);
  CamWriteCmosSensor(0x37,0x15);
  CamWriteCmosSensor(0x38,0x15);
  CamWriteCmosSensor(0x39,0x15);
  CamWriteCmosSensor(0x3a,0x15);
  CamWriteCmosSensor(0x3b,0x13);
  CamWriteCmosSensor(0x3c,0x15);
  CamWriteCmosSensor(0x3d,0x15);
  CamWriteCmosSensor(0x3e,0x15);
  CamWriteCmosSensor(0x3f,0x15);
  CamWriteCmosSensor(0x40,0x18);
  CamWriteCmosSensor(0x41,0x00);
  CamWriteCmosSensor(0x42,0x04);
  CamWriteCmosSensor(0x43,0x04);
  CamWriteCmosSensor(0x44,0x00);
  CamWriteCmosSensor(0x45,0x00);
  CamWriteCmosSensor(0x46,0x00);
  CamWriteCmosSensor(0x47,0x00);
  CamWriteCmosSensor(0x48,0x00);
  CamWriteCmosSensor(0x49,0xfd);
  CamWriteCmosSensor(0x4a,0x00);
  CamWriteCmosSensor(0x4b,0x00);
  CamWriteCmosSensor(0x4c,0xfd);
  CamWriteCmosSensor(0xfd,0x00);  
//awb 1
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0x28,0xc5);
  CamWriteCmosSensor(0x29,0x9b);
  CamWriteCmosSensor(0x2e,0x02);	
  CamWriteCmosSensor(0x2f,0x16);
  CamWriteCmosSensor(0x17,0x17);
  CamWriteCmosSensor(0x18,0x19);
  CamWriteCmosSensor(0x19,0x45);
  CamWriteCmosSensor(0x2a,0xef);
  CamWriteCmosSensor(0x2b,0x15);  
  //awb2
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0x73,0x80);
  CamWriteCmosSensor(0x1a,0x80);
  CamWriteCmosSensor(0x1b,0x80); 
//d65
  CamWriteCmosSensor(0x65,0xd5);
  CamWriteCmosSensor(0x66,0xfa);
  CamWriteCmosSensor(0x67,0x72);
  CamWriteCmosSensor(0x68,0x8a);
//indoor
  CamWriteCmosSensor(0x69,0xc6);
  CamWriteCmosSensor(0x6a,0xee);
  CamWriteCmosSensor(0x6b,0x94);
  CamWriteCmosSensor(0x6c,0xab);
//f
  CamWriteCmosSensor(0x61,0x7a);
  CamWriteCmosSensor(0x62,0x98);
  CamWriteCmosSensor(0x63,0xc5);
  CamWriteCmosSensor(0x64,0xe6);
//cwf
 CamWriteCmosSensor(0x6d,0xb9);
 CamWriteCmosSensor(0x6e,0xde);
 CamWriteCmosSensor(0x6f,0xb2);
 CamWriteCmosSensor(0x70,0xd5);
 
//////////////////skin detect//////
 CamWriteCmosSensor(0xfd,0x01);
 CamWriteCmosSensor(0x08,0x15);
 CamWriteCmosSensor(0x09,0x04);
 CamWriteCmosSensor(0x0a,0x20);
 CamWriteCmosSensor(0x0b,0x12);
 CamWriteCmosSensor(0x0c,0x27);
 CamWriteCmosSensor(0x0d,0x06);
 CamWriteCmosSensor(0x0f,0x63);  
//BPC_grad
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0x79,0xf0);
  CamWriteCmosSensor(0x7a,0x80);  //f0
  CamWriteCmosSensor(0x7b,0x80);  //f0
  CamWriteCmosSensor(0x7c,0x20);  //f0  
  
//smooth
  CamWriteCmosSensor(0xfd,0x00);
	//ͨƽֵ	
  CamWriteCmosSensor(0x57,0x06);	//raw_dif_thr_outdoor
  CamWriteCmosSensor(0x58,0x0d); //raw_dif_thr_normal
  CamWriteCmosSensor(0x56,0x10); //raw_dif_thr_dummy
  CamWriteCmosSensor(0x59,0x10); //raw_dif_thr_lowlight
	//GrGbƽֵ
  CamWriteCmosSensor(0x89,0x06);	//raw_grgb_thr_outdoor 
  CamWriteCmosSensor(0x8a,0x0d); //raw_grgb_thr_normal  
  CamWriteCmosSensor(0x9c,0x10); //raw_grgb_thr_dummy   
  CamWriteCmosSensor(0x9d,0x10); //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  CamWriteCmosSensor(0x81,0xe0);    //raw_gflt_fac_outdoor
  CamWriteCmosSensor(0x82,0xe0); //80//raw_gflt_fac_normal
  CamWriteCmosSensor(0x83,0x80);    //raw_gflt_fac_dummy
  CamWriteCmosSensor(0x84,0x40);    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  CamWriteCmosSensor(0x85,0xe0); //raw_gf_fac_outdoor  
  CamWriteCmosSensor(0x86,0xc0); //raw_gf_fac_normal  
  CamWriteCmosSensor(0x87,0x80); //raw_gf_fac_dummy   
  CamWriteCmosSensor(0x88,0x40); //raw_gf_fac_lowlight
	//RBƽǿ  
  CamWriteCmosSensor(0x5a,0xff);		 //raw_rb_fac_outdoor
  CamWriteCmosSensor(0x5b,0xe0); //40//raw_rb_fac_normal
  CamWriteCmosSensor(0x5c,0x80); 	 //raw_rb_fac_dummy
  CamWriteCmosSensor(0x5d,0x00); 	 //raw_rb_fac_lowlight
  
//sharpen 
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0xe2,0x30);	//sharpen_y_base
  CamWriteCmosSensor(0xe4,0xa0);	//sharpen_y_max
  
  CamWriteCmosSensor(0xe5,0x04); //rangek_neg_outdoor
  CamWriteCmosSensor(0xd3,0x04); //rangek_pos_outdoor   
  CamWriteCmosSensor(0xd7,0x04); //range_base_outdoor   
  
  CamWriteCmosSensor(0xe6,0x04); //rangek_neg_normal
  CamWriteCmosSensor(0xd4,0x04); //rangek_pos_normal 
  CamWriteCmosSensor(0xd8,0x04); //range_base_normal  
    
  CamWriteCmosSensor(0xe7,0x08); //rangek_neg_dummy
  CamWriteCmosSensor(0xd5,0x08); //rangek_pos_dummy
  CamWriteCmosSensor(0xd9,0x08); //range_base_dummy  	    
  CamWriteCmosSensor(0xd2,0x10); //rangek_neg_lowlight
  CamWriteCmosSensor(0xd6,0x10); //rangek_pos_lowlight
  CamWriteCmosSensor(0xda,0x10); //range_base_lowlight
  CamWriteCmosSensor(0xe8,0x20);//sharp_fac_pos_outdoor
  CamWriteCmosSensor(0xec,0x35);//sharp_fac_neg_outdoor
  CamWriteCmosSensor(0xe9,0x20);//sharp_fac_pos_nr
  CamWriteCmosSensor(0xed,0x35);//sharp_fac_neg_nr
  CamWriteCmosSensor(0xea,0x20);//sharp_fac_pos_dummy
  CamWriteCmosSensor(0xef,0x30);//sharp_fac_neg_dummy
  CamWriteCmosSensor(0xeb,0x10);//sharp_fac_pos_low
  CamWriteCmosSensor(0xf0,0x20);//sharp_fac_neg_low 
  
//CCM
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0xa0,0x80);//80(ɫӽɫ)
  CamWriteCmosSensor(0xa1,0x0 );//0 
  CamWriteCmosSensor(0xa2,0x0 );//0 
  CamWriteCmosSensor(0xa3,0xf3);//f0
  CamWriteCmosSensor(0xa4,0x8e);//a6
  CamWriteCmosSensor(0xa5,0x0 );//ea
  CamWriteCmosSensor(0xa6,0x0 );//0 
  CamWriteCmosSensor(0xa7,0xe6);//e6
  CamWriteCmosSensor(0xa8,0x9a);//9a
  CamWriteCmosSensor(0xa9,0x0 );//0 
  CamWriteCmosSensor(0xaa,0x3 );//33
  CamWriteCmosSensor(0xab,0xc );//c 
  CamWriteCmosSensor(0xfd,0x00);
  
//gamma  
  
 CamWriteCmosSensor(0xfd,0x00);
 CamWriteCmosSensor(0x8b,0x0 );//0 
 CamWriteCmosSensor(0x8c,0xC );//11
 CamWriteCmosSensor(0x8d,0x19);//19 
 CamWriteCmosSensor(0x8e,0x2C);//28 
 CamWriteCmosSensor(0x8f,0x49);//46 
 CamWriteCmosSensor(0x90,0x61);//61 
 CamWriteCmosSensor(0x91,0x77);//78 
 CamWriteCmosSensor(0x92,0x8A);//8A 
 CamWriteCmosSensor(0x93,0x9B);//9B 
 CamWriteCmosSensor(0x94,0xA9);//A9 
 CamWriteCmosSensor(0x95,0xB5);//B5 
 CamWriteCmosSensor(0x96,0xC0);//C0 
 CamWriteCmosSensor(0x97,0xCA);//CA 
 CamWriteCmosSensor(0x98,0xD4);//D4 
 CamWriteCmosSensor(0x99,0xDD);//DD 
 CamWriteCmosSensor(0x9a,0xE6);//E6 
 CamWriteCmosSensor(0x9b,0xEF);//EF 
 CamWriteCmosSensor(0xfd,0x01);//01 
 CamWriteCmosSensor(0x8d,0xF7);//F7 
 CamWriteCmosSensor(0x8e,0xFF);//FF 
             
//rpc
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0xe0,0x4c);
  CamWriteCmosSensor(0xe1,0x3c);
  CamWriteCmosSensor(0xe2,0x34);
  CamWriteCmosSensor(0xe3,0x2e);
  CamWriteCmosSensor(0xe4,0x2e);
  CamWriteCmosSensor(0xe5,0x2c);
  CamWriteCmosSensor(0xe6,0x2c);
  CamWriteCmosSensor(0xe8,0x2a);
  CamWriteCmosSensor(0xe9,0x2a);
  CamWriteCmosSensor(0xea,0x2a);
  CamWriteCmosSensor(0xeb,0x28);
  CamWriteCmosSensor(0xf5,0x28);
  CamWriteCmosSensor(0xf6,0x28);
//ae min gain  
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0x94,0xa0);//rpc_max_indr
  CamWriteCmosSensor(0x95,0x28);//1e//rpc_min_indr 
  CamWriteCmosSensor(0x9c,0xa0);//rpc_max_outdr
  CamWriteCmosSensor(0x9d,0x28);//rpc_min_outdr    

//ae target
  CamWriteCmosSensor(0xfd,0x00); 
  CamWriteCmosSensor(0xed,0x8c);//80 
  CamWriteCmosSensor(0xf7,0x88);//7c 
  CamWriteCmosSensor(0xf8,0x80);//70 
  CamWriteCmosSensor(0xec,0x7c);//6c  
  CamWriteCmosSensor(0xef,0x74);//99
  CamWriteCmosSensor(0xf9,0x70);//90
  CamWriteCmosSensor(0xfa,0x68);//80
  CamWriteCmosSensor(0xee,0x64);//78    
//gray detect
  CamWriteCmosSensor(0xfd,0x01);
  CamWriteCmosSensor(0x30,0x40);
  CamWriteCmosSensor(0x31,0x70);//
  CamWriteCmosSensor(0x32,0x40);//80
  CamWriteCmosSensor(0x33,0xef);//
  CamWriteCmosSensor(0x34,0x05);//04
  CamWriteCmosSensor(0x4d,0x2f);//40
  CamWriteCmosSensor(0x4e,0x20);//
  CamWriteCmosSensor(0x4f,0x16);//13      
  //lowlight lum
  CamWriteCmosSensor(0xfd,0x00); 
  CamWriteCmosSensor(0xb2,0x20);//lum_limit
  CamWriteCmosSensor(0xb3,0x1f);//lum_set
  CamWriteCmosSensor(0xb4,0x30);//black_vt
  CamWriteCmosSensor(0xb5,0x45);//white_vt
//saturation
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0xbe,0xff);
  CamWriteCmosSensor(0xbf,0x01); 
  CamWriteCmosSensor(0xc0,0xff);
  CamWriteCmosSensor(0xc1,0xd8);
  CamWriteCmosSensor(0xd3,0x88);//0x78
  CamWriteCmosSensor(0xd4,0x88);//0x78
  CamWriteCmosSensor(0xd6,0x80);//0x78      
  CamWriteCmosSensor(0xd7,0x60);//0x78
//HEQ
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0xdc,0x00);
  CamWriteCmosSensor(0xdd,0x80);
  CamWriteCmosSensor(0xde,0xa8);//80
  CamWriteCmosSensor(0xdf,0x80);    
//func enable
  CamWriteCmosSensor(0xfd,0x00);
  CamWriteCmosSensor(0x31,0x00);//10
  CamWriteCmosSensor(0x32,0x15);//d
  CamWriteCmosSensor(0x34,0x76);//16
  CamWriteCmosSensor(0x33,0xef);
  CamWriteCmosSensor(0x5f,0x51);
}

/*************************************************************************
* FUNCTION
*	SP0A19_Write_More_Registers
*
* DESCRIPTION
*	This function is served for FAE to modify the necessary Init Regs. Do not modify the regs
*     in init_SP0A19() directly.
*
* PARAMETERS
*	None
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void SP0A19MoreSetting(void)
{
	//  TODO: FAE Modify the Init Regs here!!! 
}
/*************************************************************************
* FUNCTION
*    SP0A19GetSensorInfo
*
* DESCRIPTION
*    This function set sensor infomation
*
* PARAMETERS
*    Info
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19GetSensorInfo(P_IMAGE_SENSOR_GET_SENSOR_INFO_STRUCT Info)
{
  Info->SensorId = SP0A19_SENSOR_ID;
  Info->SensorIf = IMAGE_SENSOR_IF_PARALLEL;
  Info->PreviewNormalDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_UYVY;
  Info->PreviewHMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_UYVY;
  Info->PreviewVMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_UYVY;
  Info->PreviewHVMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_UYVY;
  Info->CaptureDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_UYVY;
  Info->PixelClkInv = KAL_FALSE;
  Info->PixelClkPolarity = POLARITY_LOW;
  Info->HsyncPolarity = POLARITY_LOW;
  Info->VsyncPolarity = POLARITY_HIGH;
  Info->PreviewMclkFreq = SP0A19_MCLK;
  Info->CaptureMclkFreq = SP0A19_MCLK;
  Info->VideoMclkFreq = SP0A19_MCLK;
  Info->PreviewWidth = SP0A19_IMAGE_SENSOR_WIDTH_DRV;
  Info->PreviewHeight = SP0A19_IMAGE_SENSOR_HEIGHT_DRV;
  Info->FullWidth = SP0A19_IMAGE_SENSOR_WIDTH_DRV;
  Info->FullHeight = SP0A19_IMAGE_SENSOR_HEIGHT_DRV;
  Info->SensorAfSupport = KAL_FALSE;
  Info->SensorFlashSupport = KAL_FALSE;
}

/*************************************************************************
* FUNCTION
*    SP0A19PowerOn
*
* DESCRIPTION
*    This function reset sensor and read id
*
* PARAMETERS
*    None
*
* RETURNS
*    sensor id
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint16 SP0A19PowerOn(void)
{
  kal_uint16 SensorId;
  
  CisModulePowerOn(SP0A19Sensor.SensorIdx, KAL_TRUE);
  kal_sleep_task(2);  /* delay for power stable */
  CameraSccbOpen(CAMERA_SCCB_SENSOR, SP0A19_WRITE_ID, CAMERA_SCCB_8BIT, CAMERA_SCCB_8BIT, 100);
  CamPdnPinCtrl(SP0A19Sensor.SensorIdx, 0);
  
  CamRstPinCtrl(SP0A19Sensor.SensorIdx, 1);
  kal_sleep_task(1);
  CamRstPinCtrl(SP0A19Sensor.SensorIdx, 0);
  kal_sleep_task(1);
  CamRstPinCtrl(SP0A19Sensor.SensorIdx, 1);
  kal_sleep_task(3);
  
  SensorId = CamReadCmosSensor(0x02);
  if (SensorId != SP0A19_SENSOR_ID) /* READ SENSOR ID */
  {
    //dbg_print("SP0A19PowerOn!the SensorId = %x , SP0A19Sensor.SensorIdx = %d\r\n",SensorId,SP0A19Sensor.SensorIdx);
#if defined(__SP0A19_DEBUG_TRACE__)
    kal_print_string_trace(MOD_ENG, TRACE_ERROR, "SENSOR ID: %x", SensorId);
#endif
  }
  return SensorId;
}

/*************************************************************************
* FUNCTION
*    SP0A19DetectSensorId
*
* DESCRIPTION
*    This function detect sensor id
*
* PARAMETERS
*    None
*
* RETURNS
*    sensor id
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint32 SP0A19DetectSensorId(void)
{
  MM_ERROR_CODE_ENUM SP0A19SensorClose(void);
  kal_uint8 SensorId;
  IMAGE_SENSOR_INDEX_ENUM InvIdx;
  kal_int8 i, j;
  
  switch (SP0A19Sensor.SensorIdx)
  {
  case IMAGE_SENSOR_MAIN:
  case IMAGE_SENSOR_BAK1:
    InvIdx = IMAGE_SENSOR_SUB;
    break;
  default:
    InvIdx = IMAGE_SENSOR_MAIN;
    break;
  }
  for (i = 1; i >= 0; i--)
  {
    for (j = 1; j >= 0; j--)
    {
      CamRstPinCtrl(InvIdx, i);
      CamPdnPinCtrl(InvIdx, j);
      SensorId = SP0A19PowerOn();
      SP0A19SensorClose();
      //dbg_print("SP0A19DetectSensorId!SP0A19_SENSOR_ID = %x,i=%d,j=%d\r\n",SP0A19_SENSOR_ID,i,j);
      if (SP0A19_SENSOR_ID == SensorId)
      {
        return SP0A19_SENSOR_ID;
      }
    }
  }
  return NULL_SENSOR_ID;
}

/*************************************************************************
* FUNCTION
*    SP0A19InitOperationPara
*
* DESCRIPTION
*    This function set sensor operation para
*
* PARAMETERS
*    Para
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19InitOperationPara(P_IMAGE_SENSOR_GET_INIT_OPERATION_PARA_STRUCT Para)
{
  Para->CaptureDelayFrame = 2;
  Para->PreviewDelayFrame = 4;
  Para->PreviewDisplayWaitFrame = 8;
}

/*************************************************************************
* FUNCTION
*    SP0A19Preview
*
* DESCRIPTION
*    This function set to preview mode
*
* PARAMETERS
*    Id, In, Out
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19Preview(CAL_SCENARIO_ENUM Id, P_IMAGE_SENSOR_SCENARIO_PARA_IN_STRUCT In, P_IMAGE_SENSOR_SCENARIO_PARA_OUT_STRUCT Out)
{
  kal_uint16 DummyPixel;
  
  SP0A19SetMirror(In->ImageMirror);
  //SP0A19SetClock(SP0A19_PREVIEW_CLK);
    switch (Id)
  {
    case CAL_SCENARIO_VIDEO:
        //GC0309SetVideoFps(In->MaxVideoFrameRate);
        SP0A19_video_mode = KAL_TRUE;
	 SP0A19NightMode(In->BandingFreq, In->NightMode);
	 break;
    default:
        SP0A19_video_mode = KAL_FALSE;
	 SP0A19NightMode(In->BandingFreq, In->NightMode);
        /* roll back shutter&gain from capture state */
	 break;
    }
  //SP0A19SetBaseShutter(CAM_BANDING_50HZ == In->BandingFreq ? SP0A19_NUM_50HZ : SP0A19_NUM_60HZ);
  //SP0A19NightMode(In->NightMode);
  SP0A19AeEnable(KAL_TRUE);
  SP0A19AwbEnable(KAL_TRUE);
  
  Out->WaitStableFrameNum = 0;
  Out->GrabStartX = SP0A19_X_START;
  Out->GrabStartY = SP0A19_Y_START;
  Out->ExposureWindowWidth = SP0A19_IMAGE_SENSOR_WIDTH;
  Out->ExposureWindowHeight = SP0A19_IMAGE_SENSOR_HEIGHT;
}

/*************************************************************************
* FUNCTION
*    SP0A19Capture
*
* DESCRIPTION
*    This function set to capture mode
*
* PARAMETERS
*    Id, In, Out
*
* RETURNS
*    None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0A19Capture(CAL_SCENARIO_ENUM Id, P_IMAGE_SENSOR_SCENARIO_PARA_IN_STRUCT In, P_IMAGE_SENSOR_SCENARIO_PARA_OUT_STRUCT Out)
{
  Out->WaitStableFrameNum = 0;
  Out->GrabStartX = SP0A19_X_START;
  Out->GrabStartY = SP0A19_Y_START;
  Out->ExposureWindowWidth = SP0A19_IMAGE_SENSOR_WIDTH;
  Out->ExposureWindowHeight = SP0A19_IMAGE_SENSOR_HEIGHT;
}

/*************************************************************************
* FUNCTION
*    SP0A19SensorOpen
*
* DESCRIPTION
*    This function read sensor id and init sensor
*
* PARAMETERS
*    None
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19SensorOpen(void)
{
#if 0
  if (SP0A19PowerOn() != SP0A19_SENSOR_ID)
  {
    return MM_ERROR_SENSOR_READ_ID_FAIL;
  }
#else
    MM_ERROR_CODE_ENUM SP0A19SensorClose(void);
    kal_uint8 SensorId;
    IMAGE_SENSOR_INDEX_ENUM InvIdx;
    kal_int8 i, j;

    switch (SP0A19Sensor.SensorIdx)
    {
        case IMAGE_SENSOR_MAIN:
        case IMAGE_SENSOR_BAK1:
            InvIdx = IMAGE_SENSOR_SUB;
            break;
        default:
            InvIdx = IMAGE_SENSOR_MAIN;
            break;
    }
    CamRstPinCtrl(InvIdx, 1);
    CamPdnPinCtrl(InvIdx,1);
    SensorId = SP0A19PowerOn();
    if (SP0A19_SENSOR_ID != SensorId)
    {
        //dbg_print("SP0A19SensorOpen!the line = %d\r\n",__LINE__);
        kal_prompt_trace(MOD_MED,"Telacom SP0A19SensorOpen!the line = %d\r\n",__LINE__);
        SP0A19SensorClose();
        return MM_ERROR_SENSOR_READ_ID_FAIL;
    }
#endif

  SP0A19InitialSetting();
 // SP0A19MoreSetting();
  /* default setting */
  SP0A19Sensor.BypassAe = KAL_FALSE;
  SP0A19Sensor.BypassAwb = KAL_FALSE;
  SP0A19Sensor.Ctrl3A = CamReadCmosSensor(0x32);
  SP0A19Sensor.AecMode3 = CamReadCmosSensor(0x32);
  SP0A19Sensor.VideoFps = 0;
  SP0A19Sensor.IntClk = 0;
  SP0A19Sensor.FrameHeight = 0;
  SP0A19Sensor.LineLength = 0;
  
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19SensorFeatureCtrl
*
* DESCRIPTION
*    This function set sensor feature mode
*
* PARAMETERS
*    Id: scenario id
*    In: input parameter
*    Out: output parameter
*    OutLen: output parameter length
*    RealOutLen: real output parameter length
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19FlashMode(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
    if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
    {
        const P_CAL_FEATURE_TYPE_ENUM_STRUCT FlashMode = &Out->FeatureInfo.FeatureEnum;
 
        Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
        Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
        Out->FeatureModeSupport = CAL_FEATURE_CAMERA_SUPPORT;
        // TODO: add supported flash mode enum here
#ifndef LED_FLASHLIGHT_SUPPORT
        FlashMode->IsSupport = KAL_FALSE;
#else
        FlashMode->IsSupport = KAL_TRUE;
        FlashMode->ItemCount = 2;
        FlashMode->SupportItem[0] = CAM_FLASH_OFF;
        FlashMode->SupportItem[1] = CAM_FLASH_ON;
#endif
    }
    return MM_ERROR_NONE;
}
static MM_ERROR_CODE_ENUM SP0A19SensorFeatureCtrl(kal_uint32 Id, void *In, void *Out, kal_uint32 OutLen, kal_uint32 *RealOutLen)
{
  MM_ERROR_CODE_ENUM ErrCode = MM_ERROR_NONE;
  
  if (Id >= CAL_FEATURE_WEBCAM_BEGIN && Id <= CAL_FEATURE_WEBCAM_END)
  {
    return SP0A19WebcamFeatureCtrl(Id, In, Out, OutLen, RealOutLen);
  }
  switch (Id)
  {
  /* query and set series */
  case CAL_FEATURE_CAMERA_BRIGHTNESS:
  case CAL_FEATURE_CAMERA_EV_VALUE:
    ErrCode = SP0A19Ev(In, Out);
    break;
  case CAL_FEATURE_CAMERA_CONTRAST:
  case CAL_FEATURE_CAMERA_HUE:
  case CAL_FEATURE_CAMERA_SATURATION:
  case CAL_FEATURE_CAMERA_SHARPNESS:
  case CAL_FEATURE_CAMERA_GAMMA:
    ErrCode = MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    break;
  case CAL_FEATURE_CAMERA_WB:
    ErrCode = SP0A19Wb(In, Out);
    break;
  case CAL_FEATURE_CAMERA_IMAGE_EFFECT:
    ErrCode = SP0A19Effect(In, Out);
    break;
  case CAL_FEATURE_CAMERA_STILL_CAPTURE_SIZE:
    ErrCode = SP0A19StillCaptureSize(In, Out);
    break;
  case CAL_FEATURE_CAMERA_BANDING_FREQ:
    ErrCode = SP0A19Banding(In, Out);
    break;
  case CAL_FEATURE_CAMERA_SCENE_MODE:
    ErrCode = SP0A19SceneMode(KAL_TRUE, In, Out);
    break;
  case CAL_FEATURE_CAMERA_VIDEO_SCENE_MODE:
    ErrCode = SP0A19SceneMode(KAL_FALSE, In, Out);
    break;
    case CAL_FEATURE_CAMERA_FLASH_MODE:
      ErrCode = SP0A19FlashMode(In, Out);
      break;
    case IMAGE_SENSOR_FEATURE_QUERY_YUV_CTRL_FLASHLIGHT_TRIGGER_MODE:
        ((P_IMAGE_SENSOR_COMM_DATA_STRUCT)Out)->FeatureValue = FLASHLIGHT_CONTINUOUS_MODE;
        break;
    case IMAGE_SENSOR_FEATURE_CTRL_FLASHLIGHT:
        FlashlightPowerOn(SP0A19Sensor.SensorIdx, ((P_IMAGE_SENSOR_COMM_CTRL_STRUCT)In)->FeatureEnable);
        break;
  
  /* get info series */
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_INFO:
    SP0A19GetSensorInfo(Out);
    break;
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_ID:
    ((P_IMAGE_SENSOR_COMM_DATA_STRUCT)Out)->FeatureValue = SP0A19DetectSensorId();
    break;
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_FRAME_RATE:
    ((P_IMAGE_SENSOR_COMM_DATA_STRUCT)Out)->FeatureValue = SP0A19Sensor.VideoFps;
    break;
  case IMAGE_SENSOR_FEATURE_GET_INIT_OPERATION_PARA:
    SP0A19InitOperationPara(Out);
    break;
  case IMAGE_SENSOR_FEATURE_GET_HW_DEPENDENT_SETTING:
    CamGetHWInfo(SP0A19Sensor.SensorIdx, Out);
    break;
  
  /* set para series */
  case IMAGE_SENSOR_FEATURE_SET_SENSOR_POWER_DOWN:
    CamPdnPinCtrl(SP0A19Sensor.SensorIdx, 1);
    CamRstPinCtrl(SP0A19Sensor.SensorIdx, 0);
    break;
  case IMAGE_SENSOR_FEATURE_SET_NVRAM_SENSOR_INFO:
    SP0A19Sensor.NvramData = &(((nvram_camera_para_struct *)In)->SENSOR);
    break;
  case IMAGE_SENSOR_FEATURE_SET_CAMERA_SOURCE:
    SP0A19Sensor.SensorIdx = ((P_IMAGE_SENSOR_SET_CAMERA_SOURCE_STRUCT)In)->ImageSensorIdx;
    break;
  case IMAGE_SENSOR_FEATURE_CTRL_NIGHTMODE: /* move to preview */
    break;
  case IMAGE_SENSOR_FEATURE_SET_AE_BYPASS:
    SP0A19Sensor.BypassAe = *(kal_bool *)In;
    SP0A19AeEnable(SP0A19Sensor.BypassAe ? KAL_FALSE : KAL_TRUE);
    break;
  case IMAGE_SENSOR_FEATURE_SET_AWB_BYPASS:
    SP0A19Sensor.BypassAwb = *(kal_bool *)In;
    SP0A19AwbEnable(SP0A19Sensor.BypassAwb ? KAL_FALSE : KAL_TRUE);
    break;
  default:
    ErrCode = MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    break;
  }
  return ErrCode;
}

/*************************************************************************
* FUNCTION
*    SP0A19SensorCtrl
*
* DESCRIPTION
*    This function set sensor scenario mode
*
* PARAMETERS
*    Id: scenario id
*    In/Out: parameter
*    OutLen: Out parameter length
*    RealOutLen: Real out parameter length
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19SensorCtrl(CAL_SCENARIO_ENUM Id, void *In, void *Out, kal_uint32 OutLen, kal_uint32 *RealOutLen)
{
  switch (Id)
  {
  case CAL_SCENARIO_CAMERA_PREVIEW:
  case CAL_SCENARIO_VIDEO:
  case CAL_SCENARIO_WEBCAM_PREVIEW:
  case CAL_SCENARIO_WEBCAM_CAPTURE:
    SP0A19Preview(Id, In, Out);
    break;
  case CAL_SCENARIO_CAMERA_STILL_CAPTURE:
    SP0A19Capture(Id, In, Out);
    break;
  default:
    break;
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19SensorClose
*
* DESCRIPTION
*    This function is to turn off sensor module power.
*
* PARAMETERS
*    None
*
* RETURNS
*    Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0A19SensorClose(void)
{
    kal_prompt_trace(MOD_MED,"Telacom SP0A19SensorClose!");
  CamPdnPinCtrl(SP0A19Sensor.SensorIdx, 1);
  CamRstPinCtrl(SP0A19Sensor.SensorIdx, 0);
  CisModulePowerOn(SP0A19Sensor.SensorIdx, KAL_FALSE);
  CameraSccbClose(CAMERA_SCCB_SENSOR);
  
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*    SP0A19SensorFunc
*
* DESCRIPTION
*    This function get sensor function pointer
*
* PARAMETERS
*    sensor function pointer
*
* RETURNS
*    Error code
*
* GLOBALS AFFECTED
*
*************************************************************************/
MM_ERROR_CODE_ENUM SP0A19SensorFunc(P_IMAGE_SENSOR_FUNCTION_STRUCT *pfSensorFunc)
{
  static IMAGE_SENSOR_FUNCTION_STRUCT ImageSensorFuncSP0A19 =
  {
    SP0A19SensorOpen,
    SP0A19SensorFeatureCtrl,
    SP0A19SensorCtrl,
    SP0A19SensorClose,
  };
  *pfSensorFunc = &ImageSensorFuncSP0A19;
  
  return MM_ERROR_NONE;
}

