/*****************************************************************************
*  Copyright Statement:
*  --------------------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of GalaxyCore Inc. (C) 2003
*
*****************************************************************************/

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   image_sensor.c
 *
 * Version:
 * --------
 *   SP0A19 MTK 6225 DRV V1.0.0 zch
 *
 * Release Date:
 * ------------
 *   2012-09-12
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *
 * Author:
 * -------
 * -------
 *
 *============================================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/
#include "drv_comm.h"
#include "IntrCtrl.h"
#include "reg_base.h"
#include "gpio_sw.h"
#include "sccb.h"
#include "isp_if.h"
#include "image_sensor_SP0A19.h"
#include "camera_para.h"
#include "upll_ctrl.h"
#include "med_api.h"



/* Global Valuable */

static kal_bool   MPEG4_encode_mode=KAL_FALSE, g_bMJPEG_mode = KAL_FALSE;
/* Image Sensor ID */
static kal_uint16 sensor_id=0;
static kal_bool sensor_cap_state=KAL_FALSE;
static kal_bool g_banding_is_60Hz = KAL_FALSE;    

#define VDOREC_14_14FPS        0//video record daylight
#define VDOREC_14_14FPS_MAXGAIN                   0x80 
#define VDOREC_12_12FPS        1//video record daylight
#define VDOREC_12_12FPS_MAXGAIN                   0X7c
#define VDOREC_12_12FPS_NIGHT        0//video record nightmode
#define VDOREC_12_12FPS_MAXGAIN_NIGHT      0xa0
#define VDOREC_10_10FPS_NIGHT        1//video record nightmode
#define VDOREC_10_10FPS_MAXGAIN_NIGHT      0x98
#define CAPPRE_20_10FPS         0//capture preview daylight
#define CAPPRE_20_10FPS_MAXGAIN                  0x78  
#define CAPPRE_20_09FPS         0//capture preview daylight
#define CAPPRE_20_09FPS_MAXGAIN                 0x74 
#define CAPPRE_20_08FPS         1//capture preview daylight
#define CAPPRE_20_08FPS_MAXGAIN                 0x70
#define CAPPRE_20_07FPS        0//capture preview daylight
#define CAPPRE_20_07FPS_MAXGAIN                 0x6c
#define CAPPRE_20_06FPS        0//capture preview daylight
#define CAPPRE_20_06FPS_MAXGAIN                 0x68 
#define CAPPRE_20_06FPS_NIGHT         1//capture preview nightmode
#define CAPPRE_20_06FPS_MAXGAIN_NIGHT     0Xa0 

#define NORMAL_Y0ffset  0x08
#define LOWLIGHT_Y0ffset  0x25

/* Function Declaration*/
kal_uint32 set_SP0A19_param_banding(kal_uint32 para);
void SP0A19_config_window(kal_uint16 Rewidth, kal_uint16 Reheight);
void SP0A19_init_settings(void);

#ifndef HW_SCCB
static void SCCB_send_byte(kal_uint8 send_byte)
{
	volatile signed char i;
	volatile kal_uint32 j;
  	for (j = 0; j < SENSOR_I2C_DELAY; j++); 
	for (i=7;i>=0;i--)
	{	/* data bit 7~0 */
		if (send_byte & (1<<i))
		{
			SET_SCCB_DATA_HIGH;
		}
		else
		{
			SET_SCCB_DATA_LOW;
		}
		for (j = 0; j < SENSOR_I2C_DELAY; j++); 
		SET_SCCB_CLK_HIGH;
		for (j = 0; j < SENSOR_I2C_DELAY; j++); 
		SET_SCCB_CLK_LOW;
		for (j = 0; j < SENSOR_I2C_DELAY; j++); 
	}
	/* don't care bit, 9th bit */
     
	SET_SCCB_DATA_INPUT;
  	for (j = 0; j < SENSOR_I2C_DELAY; j++); 
	SET_SCCB_CLK_HIGH;
  	for (j = 0; j < SENSOR_I2C_DELAY; j++); 
	SET_SCCB_CLK_LOW;
	SET_SCCB_DATA_OUTPUT;
	SET_SCCB_DATA_LOW;

}	/* SCCB_send_byte() */

static kal_uint8 SCCB_get_byte(void)
{
	volatile signed char i;
	volatile kal_uint32 j;
	kal_uint8 get_byte=0;

	SET_SCCB_DATA_INPUT;

	for (i=7;i>=0;i--)
	{	/* data bit 7~0 */
		SET_SCCB_CLK_HIGH;
		if (GET_SCCB_DATA_BIT)
			get_byte |= (1<<i);
		SET_SCCB_CLK_LOW;
		for(j=0;j<SENSOR_I2C_DELAY;j++);
		for(j=0;j<SENSOR_I2C_DELAY;j++);
	}
	/* don't care bit, 9th bit */
	SET_SCCB_DATA_OUTPUT;

	//send NACK to slave
	SET_SCCB_DATA_HIGH;          //SET_SCCB_DATA_HIGH
	for(j=0;j<SENSOR_I2C_DELAY;j++);
	SET_SCCB_CLK_HIGH;
	for(j=0;j<SENSOR_I2C_DELAY;j++);
	SET_SCCB_CLK_LOW;

  	for(j=0;j<SENSOR_I2C_DELAY;j++);
	return get_byte;
}	/* SCCB_get_byte() */


static void SP0A19_write_cmos_sensor(kal_uint32 addr, kal_uint32 para)
{
		volatile kal_uint32 i,j;
    #ifdef HW_SCCB
		SET_SCCB_DATA_LENGTH(3);
		ENABLE_SCCB;
		REG_SCCB_DATA = SP0A19_WRITE_ID | SCCB_DATA_REG_ID_ADDRESS;
		REG_SCCB_DATA = addr;
		REG_SCCB_DATA = para;
		while (SCCB_IS_WRITTING);
    #else
		for(j=0;j<SENSOR_I2C_DELAY;j++);
		I2C_STOP_TRANSMISSION;	
		for(j=0;j<SENSOR_I2C_DELAY;j++);
		I2C_START_TRANSMISSION;
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(SP0A19_WRITE_ID);  
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(addr);
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(para);
		I2C_STOP_TRANSMISSION;

    #endif /* HW_SCCB */
}	/* SP0A19_write_cmos_sensor() */

static kal_uint32 SP0A19_read_cmos_sensor(kal_uint32 addr)
{
	volatile kal_uint32 j;
    kal_uint8 get_byte = 0;

    #ifdef HW_SCCB
		SET_SCCB_DATA_LENGTH(2);
		ENABLE_SCCB;
		REG_SCCB_DATA = SP0A19_WRITE_ID | SCCB_DATA_REG_ID_ADDRESS;
		REG_SCCB_DATA = addr;
		while (SCCB_IS_WRITTING);
		ENABLE_SCCB;
		REG_SCCB_DATA = SP0A19_READ_ID | SCCB_DATA_REG_ID_ADDRESS;
		REG_SCCB_DATA = 0;
		while (SCCB_IS_READING);
		get_byte = REG_SCCB_READ_DATA & 0xFF;
    #else
		I2C_STOP_TRANSMISSION; 
		for(j=0;j<SENSOR_I2C_DELAY;j++);
		I2C_START_TRANSMISSION;
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(SP0A19_WRITE_ID);
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(addr);
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		I2C_START_TRANSMISSION;
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		SCCB_send_byte(SP0A19_READ_ID);
		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		get_byte = SCCB_get_byte();

		for (j = 0; j < SENSOR_I2C_DELAY; j++);
		I2C_STOP_TRANSMISSION;

    #endif
    return get_byte;
}


void SP0A19_write_shutter(kal_uint16 shutter)
{
}	/* write_SP0A19_shutter */

kal_uint32 SP0A19_read_shutter(void)
{
}	/* read_SP0A19_shutter */

void SP0A19_Set_dummy(kal_uint16 pixels, kal_uint16 lines)
{
}	/* set_SP0A19_dummy */

/*************************************************************************
* FUNCTION
*  config_SP0A19_window
*
* DESCRIPTION
*  This function config the hardware window of SP0A19 for getting specified
*  data of that window.
*
* PARAMETERS
*  start_x : start column of the interested window
*  start_y : start row of the interested window
*  width  : column widht of the itnerested window
*  height : row depth of the itnerested window
*
* RETURNS
*  the data that read from SP0A19
*
* GLOBALS AFFECTED
*
*************************************************************************/
void config_SP0A19_window(kal_uint16 startx,kal_uint16 starty,kal_uint16 width, kal_uint16 height)
{
  	
}	/* set_SP0A19_dummy */
image_sensor_func_struct image_sensor_func_SP0A19;


/*************************************************************************
* FUNCTION
*	init_SP0A19
*
* DESCRIPTION
*	This function initialize the registers of CMOS sensor and ISP control register.
*
* PARAMETERS
*	None
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_int8 init_SP0A19(void)
{
	cis_module_power_on(KAL_TRUE);      // Power On CIS Power 
	kal_sleep_task(2);
// SCCB high, output
	GPIO_ModeSetup(SCCB_SERIAL_CLK_PIN,0);
	GPIO_ModeSetup(SCCB_SERIAL_DATA_PIN,0);		
	GPIO_WriteIO(1, SCCB_SERIAL_CLK_PIN);		
	GPIO_WriteIO(1, SCCB_SERIAL_DATA_PIN);		      
	GPIO_InitIO(OUTPUT, SCCB_SERIAL_CLK_PIN);		
	GPIO_InitIO(OUTPUT, SCCB_SERIAL_DATA_PIN);	
	kal_sleep_task(2);				    // To wait for Stable Power
	REG_ISP_CMOS_SENSOR_MODE_CONFIG |= REG_CMOS_SENSOR_RESET_BIT;
	kal_sleep_task(2);	
	REG_ISP_CMOS_SENSOR_MODE_CONFIG &= ~REG_CMOS_SENSOR_RESET_BIT;//low 		
	kal_sleep_task(2);
	REG_ISP_CMOS_SENSOR_MODE_CONFIG |= REG_CMOS_SENSOR_RESET_BIT;
			
	SET_CMOS_CLOCK_POLARITY_LOW; 
	SET_VSYNC_POLARITY_HIGH;   
	//SET_VSYNC_POLARITY_LOW;
	SET_HSYNC_POLARITY_LOW;  // Hsync is high actived when set to LOW here  
	
	ENABLE_CAMERA_INDATA_FORMAT;
	SET_CAMERA_INPUT_TYPE(INPUT_YUV422);
	SET_CAMERA_INPUT_ORDER(INPUT_ORDER_CbYCrY1);
	
	ENABLE_CAMERA_TG_CLK_48M;
	UPLL_Enable(UPLL_OWNER_ISP); 
	
	set_isp_driving_current(ISP_DRIVING_8MA);

	kal_sleep_task(10);
	SP0A19_write_cmos_sensor(0xfd,0x00);
	sensor_id = SP0A19_read_cmos_sensor(0x02);
	dbg_print("SP0A19_sensor_id = [%x]\r\n",sensor_id);
	kal_prompt_trace(MOD_MMI,"read_00, sensor_id=%d " ,sensor_id);
	if(sensor_id != SP0A19_SENSOR_ID)
		return -1;  
     SP0A19_init_settings();
	return 1;
}


void SP0A19_init_settings(void)
{  
//SP0A19 ini
	  SP0A19_write_cmos_sensor(0xfd,0x00);
	  SP0A19_write_cmos_sensor(0x1C,0x28);
	  SP0A19_write_cmos_sensor(0x32,0x00);
	  SP0A19_write_cmos_sensor(0x0f,0x2f);
	  SP0A19_write_cmos_sensor(0x10,0x2e);
	  SP0A19_write_cmos_sensor(0x11,0x00);
	  SP0A19_write_cmos_sensor(0x12,0x18);
	  SP0A19_write_cmos_sensor(0x13,0x2f);
	  SP0A19_write_cmos_sensor(0x14,0x00);
	  SP0A19_write_cmos_sensor(0x15,0x3f);
	  SP0A19_write_cmos_sensor(0x16,0x00);
	  SP0A19_write_cmos_sensor(0x17,0x18);
	  SP0A19_write_cmos_sensor(0x25,0x40);
	  SP0A19_write_cmos_sensor(0x1a,0x0b);
	  SP0A19_write_cmos_sensor(0x1b,0xc );
	  SP0A19_write_cmos_sensor(0x1e,0xb );
	  SP0A19_write_cmos_sensor(0x20,0x3f); // add
	  SP0A19_write_cmos_sensor(0x21,0x13); // 0x0c 24
	  SP0A19_write_cmos_sensor(0x22,0x19);
	  SP0A19_write_cmos_sensor(0x26,0x1a);
	  SP0A19_write_cmos_sensor(0x27,0xab);
	  SP0A19_write_cmos_sensor(0x28,0xfd);
	  SP0A19_write_cmos_sensor(0x30,0x00);
	  SP0A19_write_cmos_sensor(0x31,0x00);//0x10
	  SP0A19_write_cmos_sensor(0xfb,0x30); // 0x33
	  SP0A19_write_cmos_sensor(0x1f,0x10);
	  
	  
	//Blacklevel
	  SP0A19_write_cmos_sensor(0xfd,0x00);
	  SP0A19_write_cmos_sensor(0x65,0x06);//blue_suboffset
	  SP0A19_write_cmos_sensor(0x66,0x06);//red_suboffset
	  SP0A19_write_cmos_sensor(0x67,0x06);//gr_suboffset
	  SP0A19_write_cmos_sensor(0x68,0x06);//gb_suboffset
	  SP0A19_write_cmos_sensor(0x45,0x00);
	  SP0A19_write_cmos_sensor(0x46,0x0f);
	//ae setting
	  SP0A19_write_cmos_sensor(0xfd,0x00);
	  SP0A19_write_cmos_sensor(0x03,0x01);
	  SP0A19_write_cmos_sensor(0x04,0x32);
	  SP0A19_write_cmos_sensor(0x06,0x00);
	  SP0A19_write_cmos_sensor(0x09,0x01);
	  SP0A19_write_cmos_sensor(0x0a,0x46);
	  SP0A19_write_cmos_sensor(0xf0,0x66);
	  SP0A19_write_cmos_sensor(0xf1,0x00);
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x90,0x0c);
	  SP0A19_write_cmos_sensor(0x92,0x01);
	  SP0A19_write_cmos_sensor(0x98,0x66);
	  SP0A19_write_cmos_sensor(0x99,0x00);
	  SP0A19_write_cmos_sensor(0x9a,0x01);
	  SP0A19_write_cmos_sensor(0x9b,0x00);
	  
	//Status
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0xce,0x98);
	  SP0A19_write_cmos_sensor(0xcf,0x04);
	  SP0A19_write_cmos_sensor(0xd0,0x98);
	  SP0A19_write_cmos_sensor(0xd1,0x04); 
	  
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0xc4,0x56);//70
	  SP0A19_write_cmos_sensor(0xc5,0x8f);//74
	  SP0A19_write_cmos_sensor(0xca,0x30);
	  SP0A19_write_cmos_sensor(0xcb,0x45);
	  SP0A19_write_cmos_sensor(0xcc,0x70);//rpc_heq_low
	  SP0A19_write_cmos_sensor(0xcd,0x48);//rpc_heq_dummy
	  SP0A19_write_cmos_sensor(0xfd,0x00);

	  //lsc  for st 
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x35,0x15);
	  SP0A19_write_cmos_sensor(0x36,0x15); //20
	  SP0A19_write_cmos_sensor(0x37,0x15);
	  SP0A19_write_cmos_sensor(0x38,0x15);
	  SP0A19_write_cmos_sensor(0x39,0x15);
	  SP0A19_write_cmos_sensor(0x3a,0x15); //15
	  SP0A19_write_cmos_sensor(0x3b,0x13);
	  SP0A19_write_cmos_sensor(0x3c,0x15);
	  SP0A19_write_cmos_sensor(0x3d,0x15);
	  SP0A19_write_cmos_sensor(0x3e,0x15); //12
	  SP0A19_write_cmos_sensor(0x3f,0x15);
	  SP0A19_write_cmos_sensor(0x40,0x18);
	  SP0A19_write_cmos_sensor(0x41,0x00);
	  SP0A19_write_cmos_sensor(0x42,0x04);
	  SP0A19_write_cmos_sensor(0x43,0x04);
	  SP0A19_write_cmos_sensor(0x44,0x00);
	  SP0A19_write_cmos_sensor(0x45,0x00);
	  SP0A19_write_cmos_sensor(0x46,0x00);
	  SP0A19_write_cmos_sensor(0x47,0x00);
	  SP0A19_write_cmos_sensor(0x48,0x00);
	  SP0A19_write_cmos_sensor(0x49,0xfd);
	  SP0A19_write_cmos_sensor(0x4a,0x00);
	  SP0A19_write_cmos_sensor(0x4b,0x00);
	  SP0A19_write_cmos_sensor(0x4c,0xfd);
	  SP0A19_write_cmos_sensor(0xfd,0x00);

	//awb 1
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x28,0xc5);
	  SP0A19_write_cmos_sensor(0x29,0x9b);
	//SP0A19_write_cmos_sensor(0x10,0x08);
	//SP0A19_write_cmos_sensor(0x11,0x14);	
	//SP0A19_write_cmos_sensor(0x12,0x14);
	  SP0A19_write_cmos_sensor(0x2e,0x02);	
	  SP0A19_write_cmos_sensor(0x2f,0x16);
	  SP0A19_write_cmos_sensor(0x17,0x17);
	  SP0A19_write_cmos_sensor(0x18,0x19);	//0x29	 0813
	  SP0A19_write_cmos_sensor(0x19,0x45);	

	//SP0A19_write_cmos_sensor(0x1a,0x9e);//a1;a5   
	//SP0A19_write_cmos_sensor(0x1b,0xae);//b0;9a
	//SP0A19_write_cmos_sensor(0x33,0xef);
	  SP0A19_write_cmos_sensor(0x2a,0xef);
	  SP0A19_write_cmos_sensor(0x2b,0x15);

	  //awb2
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x73,0x80);
	  SP0A19_write_cmos_sensor(0x1a,0x80);
	  SP0A19_write_cmos_sensor(0x1b,0x80); 
	//d65
	  SP0A19_write_cmos_sensor(0x65,0xd5); //d6
	  SP0A19_write_cmos_sensor(0x66,0xfa); //f0
	  SP0A19_write_cmos_sensor(0x67,0x72); //7a
	  SP0A19_write_cmos_sensor(0x68,0x8a); //9a
	//indoor
	  SP0A19_write_cmos_sensor(0x69,0xc6); //ab
	  SP0A19_write_cmos_sensor(0x6a,0xee); //ca
	  SP0A19_write_cmos_sensor(0x6b,0x94); //a3
	  SP0A19_write_cmos_sensor(0x6c,0xab); //c1
	//f 
	  SP0A19_write_cmos_sensor(0x61,0x7a); //82
	  SP0A19_write_cmos_sensor(0x62,0x98); //a5
	  SP0A19_write_cmos_sensor(0x63,0xc5); //d6
	  SP0A19_write_cmos_sensor(0x64,0xe6); //ec
	  //cwf
	  SP0A19_write_cmos_sensor(0x6d,0xb9); //a5
	  SP0A19_write_cmos_sensor(0x6e,0xde); //c2
	  SP0A19_write_cmos_sensor(0x6f,0xb2); //a7
	  SP0A19_write_cmos_sensor(0x70,0xd5); //c5
	 
	//skin detect
	 SP0A19_write_cmos_sensor(0xfd,0x01);
	 SP0A19_write_cmos_sensor(0x08,0x15);
	 SP0A19_write_cmos_sensor(0x09,0x04);
	 SP0A19_write_cmos_sensor(0x0a,0x20);
	 SP0A19_write_cmos_sensor(0x0b,0x12);
	 SP0A19_write_cmos_sensor(0x0c,0x27);
	 SP0A19_write_cmos_sensor(0x0d,0x06);
	 SP0A19_write_cmos_sensor(0x0f,0x63);//0x5f	0813

	   //BPC_grad
	  SP0A19_write_cmos_sensor(0xfd,0x00);
	  SP0A19_write_cmos_sensor(0x79,0xf0);
	  SP0A19_write_cmos_sensor(0x7a,0x80);  //f0
	  SP0A19_write_cmos_sensor(0x7b,0x80); //f0 
	  SP0A19_write_cmos_sensor(0x7c,0x20);//f0	
	  
	//smooth
	  SP0A19_write_cmos_sensor(0xfd,0x00);

	  //
	  SP0A19_write_cmos_sensor(0x57,0x06); //raw_dif_thr_outdoor
	  SP0A19_write_cmos_sensor(0x58,0x0d); //raw_dif_thr_normal
	  SP0A19_write_cmos_sensor(0x56,0x10); //raw_dif_thr_dummy
	  SP0A19_write_cmos_sensor(0x59,0x10); //raw_dif_thr_lowlight
		//GrGb
	  SP0A19_write_cmos_sensor(0x89,0x06); //raw_grgb_thr_outdoor 
	  SP0A19_write_cmos_sensor(0x8a,0x0d); //raw_grgb_thr_normal	
	  SP0A19_write_cmos_sensor(0x9c,0x10); //raw_grgb_thr_dummy	
	  SP0A19_write_cmos_sensor(0x9d,0x10); //raw_grgb_thr_lowlight
		//Gr\Gb
	  SP0A19_write_cmos_sensor(0x81,0xe0); //raw_gflt_fac_outdoor
	  SP0A19_write_cmos_sensor(0x82,0xe0); //raw_gflt_fac_normal
	  SP0A19_write_cmos_sensor(0x83,0x80); //raw_gflt_fac_dummy
	  SP0A19_write_cmos_sensor(0x84,0x40); //raw_gflt_fac_lowlight
		//GrGb
	  SP0A19_write_cmos_sensor(0x85,0xe0); //raw_gf_fac_outdoor  
	  SP0A19_write_cmos_sensor(0x86,0xc0); //raw_gf_fac_normal  
	  SP0A19_write_cmos_sensor(0x87,0x80); //raw_gf_fac_dummy   
	  SP0A19_write_cmos_sensor(0x88,0x40); //raw_gf_fac_lowlight
		//
	  SP0A19_write_cmos_sensor(0x5a,0xff);  //raw_rb_fac_outdoor
	  SP0A19_write_cmos_sensor(0x5b,0xe0);  //raw_rb_fac_normal
	  SP0A19_write_cmos_sensor(0x5c,0x80);  //raw_rb_fac_dummy
	  SP0A19_write_cmos_sensor(0x5d,0x00);  //raw_rb_fac_lowlight
	  
	//sharpen 
	  SP0A19_write_cmos_sensor(0xfd,0x01); 
	  SP0A19_write_cmos_sensor(0xe2,0x30); //sharpen_y_base
	  SP0A19_write_cmos_sensor(0xe4,0xa0); //sharpen_y_max

	  SP0A19_write_cmos_sensor(0xe5,0x04); //rangek_neg_outdoor  //0x08
	  SP0A19_write_cmos_sensor(0xd3,0x04); //rangek_pos_outdoor	//0x08
	  SP0A19_write_cmos_sensor(0xd7,0x04); //range_base_outdoor	//0x08
	 
	  SP0A19_write_cmos_sensor(0xe6,0x04); //rangek_neg_normal   // 0x08
	  SP0A19_write_cmos_sensor(0xd4,0x04); //rangek_pos_normal   // 0x08
	  SP0A19_write_cmos_sensor(0xd8,0x04); //range_base_normal   // 0x08
	  
	  SP0A19_write_cmos_sensor(0xe7,0x08); //rangek_neg_dummy   // 0x10
	  SP0A19_write_cmos_sensor(0xd5,0x08); //rangek_pos_dummy   // 0x10
	  SP0A19_write_cmos_sensor(0xd9,0x08); //range_base_dummy    // 0x10
		
	  SP0A19_write_cmos_sensor(0xd2,0x10); //rangek_neg_lowlight
	  SP0A19_write_cmos_sensor(0xd6,0x10); //rangek_pos_lowlight
	  SP0A19_write_cmos_sensor(0xda,0x10); //range_base_lowlight
	 
	  SP0A19_write_cmos_sensor(0xe8,0x20);  //sharp_fac_pos_outdoor  // 0x35
	  SP0A19_write_cmos_sensor(0xec,0x35);  //sharp_fac_neg_outdoor
	  SP0A19_write_cmos_sensor(0xe9,0x20);  //sharp_fac_pos_nr	  // 0x35
	  SP0A19_write_cmos_sensor(0xed,0x35);  //sharp_fac_neg_nr
	  SP0A19_write_cmos_sensor(0xea,0x20);  //sharp_fac_pos_dummy   // 0x30
	  SP0A19_write_cmos_sensor(0xef,0x30);  //sharp_fac_neg_dummy   // 0x20
	  SP0A19_write_cmos_sensor(0xeb,0x10);  //sharp_fac_pos_low
	  SP0A19_write_cmos_sensor(0xf0,0x20);  //sharp_fac_neg_low 
	  
	//CCM
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0xa0,0x80);
	  SP0A19_write_cmos_sensor(0xa1,0x00);
	  SP0A19_write_cmos_sensor(0xa2,0x00);
	  SP0A19_write_cmos_sensor(0xa3,0xf3);  // 0xf6
	  SP0A19_write_cmos_sensor(0xa4,0x8e);  // 0x99
	  SP0A19_write_cmos_sensor(0xa5,0x00);  // 0xf2
	  SP0A19_write_cmos_sensor(0xa6,0x00);  // 0x0d
	  SP0A19_write_cmos_sensor(0xa7,0xe6);   // 0xda
	  SP0A19_write_cmos_sensor(0xa8,0x9a);//0xa0 0813  // 0x98
	  SP0A19_write_cmos_sensor(0xa9,0x00);
	  SP0A19_write_cmos_sensor(0xaa,0x03);  // 0x33
	  SP0A19_write_cmos_sensor(0xab,0x0c);
	  SP0A19_write_cmos_sensor(0xfd,0x00);

			//gamma  
	  SP0A19_write_cmos_sensor(0xfd,0x00);
	  SP0A19_write_cmos_sensor(0x8b,0x0 );  // 00;0 ;0 
	  SP0A19_write_cmos_sensor(0x8c,0xC );  // 0f;C ;11
	  SP0A19_write_cmos_sensor(0x8d,0x19);  // 1e;19;19
	  SP0A19_write_cmos_sensor(0x8e,0x2C);  // 3d;2C;28
	  SP0A19_write_cmos_sensor(0x8f,0x49);  // 6c;49;46
	  SP0A19_write_cmos_sensor(0x90,0x61);  // 92;61;61
	  SP0A19_write_cmos_sensor(0x91,0x77);  // aa;77;78
	  SP0A19_write_cmos_sensor(0x92,0x8A);  // b9;8A;8A
	  SP0A19_write_cmos_sensor(0x93,0x9B);  // c4;9B;9B
	  SP0A19_write_cmos_sensor(0x94,0xA9);  // cf;A9;A9
	  SP0A19_write_cmos_sensor(0x95,0xB5);  // d4;B5;B5
	  SP0A19_write_cmos_sensor(0x96,0xC0);  // da;C0;C0
	  SP0A19_write_cmos_sensor(0x97,0xCA);  // e0;CA;CA
	  SP0A19_write_cmos_sensor(0x98,0xD4);  // e4;D4;D4
	  SP0A19_write_cmos_sensor(0x99,0xDD);  // e8;DD;DD
	  SP0A19_write_cmos_sensor(0x9a,0xE6);  // ec;E6;E6
	  SP0A19_write_cmos_sensor(0x9b,0xEF);  // f1;EF;EF
	  SP0A19_write_cmos_sensor(0xfd,0x01);  // 01;01;01
	  SP0A19_write_cmos_sensor(0x8d,0xF7);  // f7;F7;F7
	  SP0A19_write_cmos_sensor(0x8e,0xFF);  // ff;FF;FF		 
	  SP0A19_write_cmos_sensor(0xfd,0x00);  //

	   //rpc
	  SP0A19_write_cmos_sensor(0xfd,0x00); 
	  SP0A19_write_cmos_sensor(0xe0,0x4c); //  4c;44;4c;3e;3c;3a;38;rpc_1base_max
	  SP0A19_write_cmos_sensor(0xe1,0x3c); //  3c;36;3c;30;2e;2c;2a;rpc_2base_max
	  SP0A19_write_cmos_sensor(0xe2,0x34); //  34;2e;34;2a;28;26;26;rpc_3base_max
	  SP0A19_write_cmos_sensor(0xe3,0x2e); //  2e;2a;2e;26;24;22;rpc_4base_max
	  SP0A19_write_cmos_sensor(0xe4,0x2e); //  2e;2a;2e;26;24;22;rpc_5base_max
	  SP0A19_write_cmos_sensor(0xe5,0x2c); //  2c;28;2c;24;22;20;rpc_6base_max
	  SP0A19_write_cmos_sensor(0xe6,0x2c); //  2c;28;2c;24;22;20;rpc_7base_max
	  SP0A19_write_cmos_sensor(0xe8,0x2a); //  2a;26;2a;22;20;20;1e;rpc_8base_max
	  SP0A19_write_cmos_sensor(0xe9,0x2a); //  2a;26;2a;22;20;20;1e;rpc_9base_max 
	  SP0A19_write_cmos_sensor(0xea,0x2a); //  2a;26;2a;22;20;20;1e;rpc_10base_max
	  SP0A19_write_cmos_sensor(0xeb,0x28); //  28;24;28;20;1f;1e;1d;rpc_11base_max
	  SP0A19_write_cmos_sensor(0xf5,0x28); //  28;24;28;20;1f;1e;1d;rpc_12base_max
	  SP0A19_write_cmos_sensor(0xf6,0x28); //  28;24;28;20;1f;1e;1d;rpc_13base_max	

	//ae min gain  
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x94,0xa0);  //rpc_max_indr
	  SP0A19_write_cmos_sensor(0x95,0x28);   // 1e rpc_min_indr 
	  SP0A19_write_cmos_sensor(0x9c,0xa0);  //rpc_max_outdr
	  SP0A19_write_cmos_sensor(0x9d,0x28);  //rpc_min_outdr	 
	//ae target
	  SP0A19_write_cmos_sensor(0xfd,0x00); 
	  SP0A19_write_cmos_sensor(0xed,0x8c); //80 
	  SP0A19_write_cmos_sensor(0xf7,0x88); //7c 
	  SP0A19_write_cmos_sensor(0xf8,0x80); //70 
	  SP0A19_write_cmos_sensor(0xec,0x7c); //6c  
	  
	  SP0A19_write_cmos_sensor(0xef,0x74); //99
	  SP0A19_write_cmos_sensor(0xf9,0x70); //90
	  SP0A19_write_cmos_sensor(0xfa,0x68); //80
	  SP0A19_write_cmos_sensor(0xee,0x64); //78

		
	//gray detect
	  SP0A19_write_cmos_sensor(0xfd,0x01);
	  SP0A19_write_cmos_sensor(0x30,0x40);
	  //add 0813 
	  SP0A19_write_cmos_sensor(0x31,0x70);
	  SP0A19_write_cmos_sensor(0x32,0x40);
	  SP0A19_write_cmos_sensor(0x33,0xef);
	  SP0A19_write_cmos_sensor(0x34,0x05);
	  SP0A19_write_cmos_sensor(0x4d,0x2f);
	  SP0A19_write_cmos_sensor(0x4e,0x20);
	  SP0A19_write_cmos_sensor(0x4f,0x16);

	//lowlight lum
	  SP0A19_write_cmos_sensor(0xfd,0x00); //
	  SP0A19_write_cmos_sensor(0xb2,0x20); //lum_limit  // 0x10
	  SP0A19_write_cmos_sensor(0xb3,0x1f); //lum_set
	  SP0A19_write_cmos_sensor(0xb4,0x30); //black_vt  // 0x20
	  SP0A19_write_cmos_sensor(0xb5,0x45); //white_vt

	//saturation
	  SP0A19_write_cmos_sensor(0xfd,0x00); 
	  SP0A19_write_cmos_sensor(0xbe,0xff); 
	  SP0A19_write_cmos_sensor(0xbf,0x01); 
	  SP0A19_write_cmos_sensor(0xc0,0xff); 
	  SP0A19_write_cmos_sensor(0xc1,0xd8); 
	  SP0A19_write_cmos_sensor(0xd3,0x88); //0x78
	  SP0A19_write_cmos_sensor(0xd4,0x88); //0x78
	  SP0A19_write_cmos_sensor(0xd6,0x70); //0x78 	   (0xd7,0x60); //0x78
	  SP0A19_write_cmos_sensor(0xd7,0x60);

	//HEQ
	  SP0A19_write_cmos_sensor(0xfd,0x00); 
	  SP0A19_write_cmos_sensor(0xdc,0x00); 
	  SP0A19_write_cmos_sensor(0xdd,0x80); //0x80 0813  // 0x78
	  SP0A19_write_cmos_sensor(0xde,0xa8); //80  0x88  0813
	  SP0A19_write_cmos_sensor(0xdf,0x80); 
	   
	//func enable
	  SP0A19_write_cmos_sensor(0xfd,0x00);  
	  SP0A19_write_cmos_sensor(0x32,0x15);  //0x0d
	  SP0A19_write_cmos_sensor(0x34,0x76);  //16
	  SP0A19_write_cmos_sensor(0x35,0x00);  
	  SP0A19_write_cmos_sensor(0x33,0xef);  
	  SP0A19_write_cmos_sensor(0x5f,0x51); 
}


/*************************************************************************
* FUNCTION
*	power_off_SP0A19
*
* DESCRIPTION
*	This function is to turn off sensor module power.
*
* PARAMETERS
*	None
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/

void power_off_SP0A19(void)
{
   
    //POWER_OFF_MODULE;
    cis_module_power_on(KAL_FALSE);
  #ifndef HW_SCCB
    SET_SCCB_CLK_LOW;
    SET_SCCB_DATA_LOW;
  #endif
  
	/* Set Low to MCLK */
	SET_CMOS_FALLING_EDGE(0);
	DISABLE_CAMERA_TG_PHASE_COUNTER;
	DISABLE_CAMERA_CLOCK_OUTPUT_TO_CMOS;  

}	/* power_off_SP0A19 */

/*************************************************************************
* FUNCTION
*	get_SP0A19_id
*
* DESCRIPTION
*	This function return the sensor read/write id of SCCB interface.
*
* PARAMETERS
*	*sensor_write_id : address pointer of sensor write id
*  *sensor_read_id  : address pointer of sensor read id
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void get_SP0A19_id(kal_uint8 *sensor_write_id, kal_uint8 *sensor_read_id)
{
	*sensor_write_id=SP0A19_WRITE_ID;
	*sensor_read_id=SP0A19_READ_ID;
}	/* get_SP0A19_id */

/*************************************************************************
* FUNCTION
*	get_SP0A19_size
*
* DESCRIPTION
*	This function return the image width and height of image sensor.
*
* PARAMETERS
*	*sensor_width : address pointer of horizontal effect pixels of image sensor
*  *sensor_height : address pointer of vertical effect pixels of image sensor
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void get_SP0A19_size(kal_uint16 *sensor_width, kal_uint16 *sensor_height)
{
	dbg_print("get_SP0A19_size\r\n");
	*sensor_width=IMAGE_SENSOR_VGA_WIDTH;//IMAGE_SENSOR_VGA_WIDTH;		/* pixel numbers actually used in one frame */
	*sensor_height=IMAGE_SENSOR_VGA_HEIGHT;//IMAGE_SENSOR_VGA_HEIGHT;	      /* line numbers actually used in one frame */
}	/* get_SP0A19_size */

/*************************************************************************
* FUNCTION
*	get_SP0A19_period
*
* DESCRIPTION
*	This function return the image width and height of image sensor.
*
* PARAMETERS
*	*pixel_number : address pointer of pixel numbers in one period of HSYNC
*  *line_number : address pointer of line numbers in one period of VSYNC
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void get_SP0A19_period(kal_uint16 *pixel_number, kal_uint16 *line_number)
{
	dbg_print("get_SP0A19_period\r\n");
}	/* get_SP0A19_period */


/*************************************************************************
* FUNCTION
*	config_SP0A19_window
*
* DESCRIPTION
*	This function config the hardware window of SP0A19 for getting specified
*  data of that window.
*
* PARAMETERS
*	start_x : start column of the interested window
*  start_y : start row of the interested window
*  width  : column widht of the itnerested window
*  height : row depth of the itnerested window
*
* RETURNS
*	the data that read from SP0A19
*
* GLOBALS AFFECTED
*
*************************************************************************/
void SP0A19_config_window(kal_uint16 Rewidth, kal_uint16 Reheight)
{
	dbg_print("xg:SP0A19_config_window %d,%d\r\n",Rewidth,Reheight);
}	/* config_SP0A19_window */

void SP0A19_preview(image_sensor_exposure_window_struct *image_window, image_sensor_config_struct *sensor_config_data)
{
	sensor_cap_state=KAL_FALSE;
	g_bMJPEG_mode = KAL_FALSE;

	if((sensor_config_data->isp_op_mode==ISP_MJPEG_PREVIEW_MODE)||(sensor_config_data->isp_op_mode==ISP_MJPEG_ENCODE_MODE))  //// MPEG4 Encode Mode
	{
			MPEG4_encode_mode=KAL_FALSE;
			g_bMJPEG_mode = KAL_TRUE;
			/* config TG of ISP to match the setting of image sensor*/
			SET_TG_OUTPUT_CLK_DIVIDER(1);	
			SET_CMOS_RISING_EDGE(0);
			SET_CMOS_FALLING_EDGE(1);  
			ENABLE_CAMERA_PIXEL_CLKIN_ENABLE;
			SET_TG_PIXEL_CLK_DIVIDER(1);
			SET_CMOS_DATA_LATCH(1);
	}
	else 
	{
		if(sensor_config_data->frame_rate==0x0F)		
		{
			MPEG4_encode_mode=KAL_TRUE;
			g_bMJPEG_mode = KAL_TRUE;
			/* config TG of ISP to match the setting of image sensor*/
			SET_TG_OUTPUT_CLK_DIVIDER(3);
			SET_CMOS_RISING_EDGE(0);
			SET_CMOS_FALLING_EDGE(2);
			ENABLE_CAMERA_PIXEL_CLKIN_ENABLE;
			SET_TG_PIXEL_CLK_DIVIDER(3);
			SET_CMOS_DATA_LATCH(2);
		}
    		else                                                                  // normal preview MCLK@24MHz
   		{
			MPEG4_encode_mode=KAL_FALSE;
			/* config TG of ISP to match the setting of image sensor*/		
			SET_TG_OUTPUT_CLK_DIVIDER(1);	
			SET_CMOS_RISING_EDGE(0);
			SET_CMOS_FALLING_EDGE(1);  
			ENABLE_CAMERA_PIXEL_CLKIN_ENABLE;
			SET_TG_PIXEL_CLK_DIVIDER(1);
			SET_CMOS_DATA_LATCH(1);
		}
	}
		image_window->grab_start_x=0;
		image_window->grab_start_y=1;
		image_window->exposure_window_width=IMAGE_SENSOR_VGA_WIDTH;
		image_window->exposure_window_height=IMAGE_SENSOR_VGA_HEIGHT- 1;
        kal_sleep_task(10);		
}


void SP0A19_capture(image_sensor_exposure_window_struct *image_window, image_sensor_config_struct *sensor_config_data)
{
		kal_uint8 iStartX = 0, iStartY = 1;
        sensor_cap_state=KAL_TRUE;		

		SET_TG_OUTPUT_CLK_DIVIDER(1);	
		SET_CMOS_RISING_EDGE(0);
		SET_CMOS_FALLING_EDGE(1);
		ENABLE_CAMERA_PIXEL_CLKIN_ENABLE;
		SET_TG_PIXEL_CLK_DIVIDER(1);
		SET_CMOS_DATA_LATCH(1);
					
		image_window->grab_start_x = iStartX;
		image_window->grab_start_y = iStartY;
		image_window->exposure_window_width=IMAGE_SENSOR_VGA_WIDTH - iStartX;
		image_window->exposure_window_height=IMAGE_SENSOR_VGA_HEIGHT- iStartY;
		kal_sleep_task(10);	
	}	
}	/* SP0A19_capture() */

/*************************************************************************
* FUNCTION
*	write_SP0A19_reg
*
* DESCRIPTION
*	This function set the register of SP0A19.
*
* PARAMETERS
*	addr : the register index of SP0A19
*  para : setting parameter of the specified register of SP0A19
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void write_SP0A19_reg(kal_uint32 addr, kal_uint32 para)
{
	//SP0A19_write_cmos_sensor(addr,para);
}	/* write_SP0A19_reg() */

/*************************************************************************
* FUNCTION
*	SP0A19_read_cmos_sensor
*
* DESCRIPTION
*	This function read parameter of specified register from SP0A19.
*
* PARAMETERS
*	addr : the register index of SP0A19
*
* RETURNS
*	the data that read from SP0A19
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 read_SP0A19_reg(kal_uint32 addr)
{
	return (SP0A19_read_cmos_sensor(addr));
}	/* read_SP0A19_reg() */


/*************************************************************************
* FUNCTION
*	set_SP0A19_shutter
*
* DESCRIPTION
*	This function set e-shutter of SP0A19 to change exposure time.
*
* PARAMETERS
*	shutter : exposured lines
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void SP0A19_set_shutter(kal_uint16 shutter)
{
  //  MY_CAM_TRACE_HEAD_1(SP0A19_set_shutter, shutter);
	exposure_lines=shutter;
	SP0A19_write_shutter(shutter);
}	/* set_SP0A19_shutter */
/*************************************************************************
* FUNCTION
*	set_SP0A19_gain
*
* DESCRIPTION
*	This function is to set global gain to sensor.
*
* PARAMETERS
*	gain : sensor global gain(base: 0x40)
*
* RETURNS
*	the actually gain set to sensor.
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint16 set_SP0A19_gain(kal_uint16 gain)
{
}

/*************************************************************************
* FUNCTION
*	SP0A19_night_mode
*
* DESCRIPTION
*	This function night mode of SP0A19.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void SP0A19_night_mode(kal_bool enable)
{
#if 1	
	if (sensor_cap_state)
	{
		return ;
	}
	else
	{ 
		if (!enable) //daylight
		{
			if(g_bMJPEG_mode)	
			{
				SP0A19_write_cmos_sensor(0xfd,0x0 );
				SP0A19_write_cmos_sensor(0xb2,0x08);
				SP0A19_write_cmos_sensor(0xb3,0x1f);
			
				if(g_banding_is_60Hz == KAL_FALSE)  //50HZ
				{
					//Video record daylight 24M 50hz 14-14FPS maxgain:0x80
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0xd8);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x03);
					SP0A19_write_cmos_sensor(0x0a,0x31);
					SP0A19_write_cmos_sensor(0xf0,0x48);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x07);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x48);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xf8);
					SP0A19_write_cmos_sensor(0xcf,0x01);
					SP0A19_write_cmos_sensor(0xd0,0xf8);
					SP0A19_write_cmos_sensor(0xd1,0x01);
					SP0A19_write_cmos_sensor(0xfd,0x00);             
					dbg_print("SP0A19_VID_DAY_50HZ  \r\n");
				}
				else  //60HZ
				{
					//Video record daylight 24M 60Hz 14-14FPS maxgain:0x80
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0xb4);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x03);
					SP0A19_write_cmos_sensor(0x0a,0x31);
					SP0A19_write_cmos_sensor(0xf0,0x3c);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x08);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x3c);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xe0);
					SP0A19_write_cmos_sensor(0xcf,0x01);
					SP0A19_write_cmos_sensor(0xd0,0xe0);
					SP0A19_write_cmos_sensor(0xd1,0x01);
					SP0A19_write_cmos_sensor(0xfd,0x00);	
					dbg_print("SP0A19_VID_DAY_60HZ  \r\n");
				}
			}
			else
			{
				SP0A19_write_cmos_sensor(0xfd,0x0 );
				SP0A19_write_cmos_sensor(0xb2,0x08);
				SP0A19_write_cmos_sensor(0xb3,0x1f);

				if(g_banding_is_60Hz == KAL_FALSE)  //50HZ
				{
					//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x01);
					SP0A19_write_cmos_sensor(0x04,0x32);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x01);
					SP0A19_write_cmos_sensor(0x0a,0x46);
					SP0A19_write_cmos_sensor(0xf0,0x66);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x0c);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x66);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xc8);
					SP0A19_write_cmos_sensor(0xcf,0x04);
					SP0A19_write_cmos_sensor(0xd0,0xc8);
					SP0A19_write_cmos_sensor(0xd1,0x04);
					SP0A19_write_cmos_sensor(0xfd,0x00);	
					dbg_print("SP0A19_CAM_DAY_50HZ  \r\n");
				}
				else  //60HZ
				{
					SCI_TRACE_LOW("normal mode 60hz\r\n");
					//capture preview daylight 24M 60hz 20-8FPS maxgain:0x70   
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0xff);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x01);
					SP0A19_write_cmos_sensor(0x0a,0x46);
					SP0A19_write_cmos_sensor(0xf0,0x55);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x0f);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x55);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xfb);
					SP0A19_write_cmos_sensor(0xcf,0x04);
					SP0A19_write_cmos_sensor(0xd0,0xfb);
					SP0A19_write_cmos_sensor(0xd1,0x04);
					SP0A19_write_cmos_sensor(0xfd,0x00);							
					dbg_print("SP0A19_CAM_DAY_60HZ  \r\n");
				}
			}	
		}
		else //night mode
		{
			if(g_bMJPEG_mode)	
			{
				SP0A19_write_cmos_sensor(0xfd,0x0 );
				SP0A19_write_cmos_sensor(0xb2,0x25);
				SP0A19_write_cmos_sensor(0xb3,0x1f);
			
				if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
				{
					dbg_print("SP0A19_VID_NIGTHT_50HZ  \r\n");
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0x7b);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x08);
					SP0A19_write_cmos_sensor(0x0a,0x1d);
					SP0A19_write_cmos_sensor(0xf0,0x29);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x0c);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x29);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xec);
					SP0A19_write_cmos_sensor(0xcf,0x01);
					SP0A19_write_cmos_sensor(0xd0,0xec);
					SP0A19_write_cmos_sensor(0xd1,0x01);
					SP0A19_write_cmos_sensor(0xfd,0x00);
				}
				else  //60HZ
				{
					//SI13_SP0A19 24M 1Ƶ 50Hz 8.0392-8fps AE_Parameters_ maxgain 80
					dbg_print("SP0A19_VID_NIGTHT_60HZ  \r\n");
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0x69);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x07);
					SP0A19_write_cmos_sensor(0x0a,0xd7);
					SP0A19_write_cmos_sensor(0xf0,0x23);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x0f);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x23);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0x0d);
					SP0A19_write_cmos_sensor(0xcf,0x02);
					SP0A19_write_cmos_sensor(0xd0,0x0d);
					SP0A19_write_cmos_sensor(0xd1,0x02);
					SP0A19_write_cmos_sensor(0xfd,0x00);
				}
			}
			else
			{
				SP0A19_write_cmos_sensor(0xfd,0x0 );
				SP0A19_write_cmos_sensor(0xb2,0x25);
				SP0A19_write_cmos_sensor(0xb3,0x1f);

				if(SP0A19_CAM_BANDING_50HZ == KAL_TRUE)  //50HZ
				{
					//capture preview night 24M 50hz 20-6FPS maxgain:0x78	 
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x01);
					SP0A19_write_cmos_sensor(0x04,0x32);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x01);
					SP0A19_write_cmos_sensor(0x0a,0x46);
					SP0A19_write_cmos_sensor(0xf0,0x66);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x10);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x66);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0x60);
					SP0A19_write_cmos_sensor(0xcf,0x06);
					SP0A19_write_cmos_sensor(0xd0,0x60);
					SP0A19_write_cmos_sensor(0xd1,0x06);
					SP0A19_write_cmos_sensor(0xfd,0x00);
					dbg_print("SP0A19_CAM_NIGHT_50HZ  \r\n");
				}
				else  //60HZ
				{
					//capture preview night 24M 60hz 20-6FPS maxgain:0x78
					SP0A19_write_cmos_sensor(0xfd,0x00);
					SP0A19_write_cmos_sensor(0x03,0x00);
					SP0A19_write_cmos_sensor(0x04,0xff);
					SP0A19_write_cmos_sensor(0x06,0x00);
					SP0A19_write_cmos_sensor(0x09,0x01);
					SP0A19_write_cmos_sensor(0x0a,0x46);
					SP0A19_write_cmos_sensor(0xf0,0x55);
					SP0A19_write_cmos_sensor(0xf1,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0x90,0x14);
					SP0A19_write_cmos_sensor(0x92,0x01);
					SP0A19_write_cmos_sensor(0x98,0x55);
					SP0A19_write_cmos_sensor(0x99,0x00);
					SP0A19_write_cmos_sensor(0x9a,0x01);
					SP0A19_write_cmos_sensor(0x9b,0x00);
					SP0A19_write_cmos_sensor(0xfd,0x01);
					SP0A19_write_cmos_sensor(0xce,0xa4);
					SP0A19_write_cmos_sensor(0xcf,0x06);
					SP0A19_write_cmos_sensor(0xd0,0xa4);
					SP0A19_write_cmos_sensor(0xd1,0x06);
					SP0A19_write_cmos_sensor(0xfd,0x00);
					dbg_print("SP0A19_CAM_NIGHT_60HZ  \r\n");
				}
			}	
		}//end night mode
	}//end settings
#endif
}//end func

/*************************************************************************
* FUNCTION
*	set_SP0A19_flashlight
*
* DESCRIPTION
*	turn on/off SP0A19 flashlight .
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
void set_SP0A19_flashlight(kal_bool enable)
{
	// Todo
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_zoom
*
* DESCRIPTION
*	SP0A19 zoom setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_zoom(kal_uint32 para)
{
	return KAL_FALSE;
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_contrast
*
* DESCRIPTION
*	SP0A19 contrast setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_contrast(kal_uint32 para)
{
	// Not Support
	return KAL_FALSE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_brightness
*
* DESCRIPTION
*	SP0A19 brightness setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_brightness(kal_uint32 para)
{
	// Not Support
	return KAL_FALSE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_hue
*
* DESCRIPTION
*	SP0A19 hue setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_hue(kal_uint32 para)
{
	// Not Support
	return KAL_FALSE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_gamma
*
* DESCRIPTION
*	SP0A19 gamma setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_gamma(kal_uint32 para)
{
	return KAL_FALSE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_wb
*
* DESCRIPTION
*	SP0A19 wb setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_wb(kal_uint32 para)
{ 
kal_uint16 P1_28,P1_29;
#if 1
	if (SP0A19_sensor_bCaptureMode == KAL_TRUE)
	{
		return KAL_TRUE;
	}

	switch (para)
	{
		case CAM_WB_AUTO:   // enable AWB 
			//SP0A19_reg_WB_auto         Զ
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0xc4);		                                                       
			SP0A19_write_cmos_sensor(0x29,0x9e);
			SP0A19_write_cmos_sensor(0xfd,0x00);  // AUTO 3000K~7000K   	  
			SP0A19_write_cmos_sensor(0x32,0x0d);       		
		break;

		case CAM_WB_CLOUD: 
			// SP0A19_reg_WB_auto   
			SP0A19_write_cmos_sensor(0xfd,0x00);   //7000K                                     
			SP0A19_write_cmos_sensor(0x32,0x05);                                                          
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0xbf);		                                                       
			SP0A19_write_cmos_sensor(0x29,0x89);		                                                       
			SP0A19_write_cmos_sensor(0xfd,0x00);                                             
		break;

		case CAM_WB_DAYLIGHT: 
			// SP0A19_reg_WB_auto   
			SP0A19_write_cmos_sensor(0xfd,0x00);  //6500K                                     
			SP0A19_write_cmos_sensor(0x32,0x05);                                                          
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0xbc);		                                                       
			SP0A19_write_cmos_sensor(0x29,0x5d);		                                                       
			SP0A19_write_cmos_sensor(0xfd,0x00);                                                         
		break;

		case CAM_WB_INCANDESCENCE: 
			// SP0A19_reg_WB_auto ׳ 
			SP0A19_write_cmos_sensor(0xfd,0x00);  //2800K~3000K                                     
			SP0A19_write_cmos_sensor(0x32,0x05);                                                          
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0x89);		                                                       
			SP0A19_write_cmos_sensor(0x29,0xb8);		                                                       
			SP0A19_write_cmos_sensor(0xfd,0x00);                                                          
		break;

		case CAM_WB_FLUORESCENT: 
			//SP0A19_reg_WB_auto  ӫ 
			SP0A19_write_cmos_sensor(0xfd,0x00);  //4200K~5000K                                     
			SP0A19_write_cmos_sensor(0x32,0x05);                                                          
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0xb5);		                                                       
			SP0A19_write_cmos_sensor(0x29,0xa5);		                                                       
			SP0A19_write_cmos_sensor(0xfd,0x00);                                                         
		break;

		case CAM_WB_TUNGSTEN: 
			// SP0A19_reg_WB_auto ȹ
			SP0A19_write_cmos_sensor(0xfd,0x00);  //4000K                                   
			SP0A19_write_cmos_sensor(0x32,0x05);                                                          
			SP0A19_write_cmos_sensor(0xfd,0x01);                                                          
			SP0A19_write_cmos_sensor(0x28,0xaf);		                                                       
			SP0A19_write_cmos_sensor(0x29,0x99);		                                                       
			SP0A19_write_cmos_sensor(0xfd,0x00);                                                         
		break;

		case CAM_WB_MANUAL:
		break;

		default:
		return KAL_FALSE;
	}

	return KAL_TRUE;
#endif
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_exposure
*
* DESCRIPTION
*	SP0A19 exposure setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_exposure(kal_uint32 para)
{
#if 1
	switch (para)
	{
		case CAM_EV_NEG_4_3:    // -4 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0xc0);
		break;

		case CAM_EV_NEG_3_3:    // -3 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0xd0);
		break;

		case CAM_EV_NEG_2_3:    // -2 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0xe0);
		break;

		case CAM_EV_NEG_1_3:    // -1 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0xf0);
		break;

		case CAM_EV_ZERO:   // +0 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0x00);//0xfa before
		break;

		case CAM_EV_POS_1_3:    // +1 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0x10);
		break;

		case CAM_EV_POS_2_3:    // +2 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0x20);
		break;

		case CAM_EV_POS_3_3:    // +3 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0x30);
		break;

		case CAM_EV_POS_4_3:    // +4 EV
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0xdc, 0x40);
		break;

		default:
		return KAL_FALSE;    
	}
	
	return KAL_TRUE;	
#endif
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_effect
*
* DESCRIPTION
*	SP0A19 effect setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_effect(kal_uint32 para)
{
	kal_uint32  ret = KAL_TRUE;	
	dbg_print("set_SP0A19_param_effect\r\n");
#if 1
	switch (para)
	{
		case CAM_EFFECT_ENC_NORMAL:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x00);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);
		break;

		case CAM_EFFECT_ENC_GRAYSCALE:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x20);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);

		break;

		case CAM_EFFECT_ENC_SEPIA:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x10);
			SP0A19_write_cmos_sensor(0x63, 0xc0);
			SP0A19_write_cmos_sensor(0x64, 0x20);

		break;

		case CAM_EFFECT_ENC_COLORINV:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x04);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);
		break;

		case CAM_EFFECT_ENC_SEPIAGREEN:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x10);
			SP0A19_write_cmos_sensor(0x63, 0x20);
			SP0A19_write_cmos_sensor(0x64, 0x20);
		break;

		case CAM_EFFECT_ENC_SEPIABLUE:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x10);
			SP0A19_write_cmos_sensor(0x63, 0x20);
			SP0A19_write_cmos_sensor(0x64, 0xf0);

		break;

		case CAM_EFFECT_ENC_GRAYINV:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x08);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);

		break;

		case CAM_EFFECT_ENC_EMBOSSMENT:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x01);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);

		break;

		case CAM_EFFECT_ENC_SKETCH:
			SP0A19_write_cmos_sensor(0xfd, 0x00);
			SP0A19_write_cmos_sensor(0x62, 0x02);
			SP0A19_write_cmos_sensor(0x63, 0x80);
			SP0A19_write_cmos_sensor(0x64, 0x80);

		break;
		case CAM_EFFECT_ENC_WHITEBOARD:
		case CAM_EFFECT_ENC_COPPERCARVING:        
		case CAM_EFFECT_ENC_BLUECARVING:
		case CAM_EFFECT_ENC_CONTRAST:
		case CAM_EFFECT_ENC_JEAN:       
		case CAM_EFFECT_ENC_OIL:
		default:
		return 
			ret = KAL_FALSE;
	}
	return ret;
#endif
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_banding
*
* DESCRIPTION
*	SP0A19 banding setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_banding(kal_uint32 para)
{
	dbg_print("set_SP0A19_param_banding\r\n");
	switch (para)
	{
		case CAM_BANDING_50HZ:
			g_banding_is_60Hz = KAL_FALSE;
			break;                        
		case CAM_BANDING_60HZ:
			g_banding_is_60Hz = KAL_TRUE;
			break;
		default:
			return KAL_FALSE;
	}
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_saturation
*
* DESCRIPTION
*	SP0A19 SATURATION setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_saturation(kal_uint32 para)
{
    // Not Support
	return KAL_FALSE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_nightmode
*
* DESCRIPTION
*	SP0A19 night mode setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_nightmode(kal_uint32 para)
{
	SP0A19_night_mode((kal_bool)para);
	return KAL_TRUE;	
}

/*************************************************************************
* FUNCTION
*	set_SP0A19_param_ev
*
* DESCRIPTION
*	SP0A19 ev setting.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 set_SP0A19_param_ev(kal_uint32 para)
{
	return set_SP0A19_param_exposure(para);
}

/*************************************************************************
* FUNCTION
*	SP0A19_yuv_sensor_setting
*
* DESCRIPTION
*	This function send command and parameter to yuv sensor module OV0306
*  to configure it
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
kal_uint32 SP0A19_yuv_sensor_setting(kal_uint32 cmd, kal_uint32 para)
{
	kal_uint32 ret = KAL_TRUE;
	dbg_print("SP0A19_yuv_sensor_setting\r\n");
	switch (cmd)
	{
		case CAM_PARAM_ZOOM_FACTOR:
			ret = set_SP0A19_param_zoom(para);		
			break;			
		case CAM_PARAM_CONTRAST:
			ret = set_SP0A19_param_contrast(para);		
			break;					
		case CAM_PARAM_BRIGHTNESS:
			ret = set_SP0A19_param_brightness(para);		
			break;							
		case CAM_PARAM_HUE:
			ret = set_SP0A19_param_hue(para);		
			break;							
		case CAM_PARAM_GAMMA:
			ret = set_SP0A19_param_gamma(para);		
			break;							
		case CAM_PARAM_WB:
			ret = set_SP0A19_param_wb(para);		
			break;
		case CAM_PARAM_EXPOSURE:
			ret = set_SP0A19_param_exposure(para);		
			break;							
		case CAM_PARAM_EFFECT:
			ret = set_SP0A19_param_effect(para);
			break;
		case CAM_PARAM_BANDING:
			ret = set_SP0A19_param_banding(para);		
			break;		
		case CAM_PARAM_SATURATION:
			ret = set_SP0A19_param_saturation(para);				
			break;						
		case CAM_PARAM_NIGHT_MODE:
			ret = set_SP0A19_param_nightmode(para);				
			break;						
		case CAM_PARAM_EV_VALUE:
			ret = set_SP0A19_param_ev(para);				
			break;				
		default:
			ret = KAL_FALSE;
	}
	return ret;
}

/*************************************************************************
* FUNCTION
*	image_sensor_func_SP0A19
*
* DESCRIPTION
*	SP0A19 Image Sensor functions struct.
*
* PARAMETERS
*	none
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
image_sensor_func_struct image_sensor_func_SP0A19=
{
	init_SP0A19,
	get_SP0A19_id,
	get_SP0A19_size,
	get_SP0A19_period,
	SP0A19_preview,
	SP0A19_capture,
	write_SP0A19_reg,
	read_SP0A19_reg,
	SP0A19_set_shutter,
	SP0A19_night_mode,
	power_off_SP0A19,
	set_SP0A19_gain,
	set_SP0A19_flashlight,
	SP0A19_yuv_sensor_setting
};	/* image_sensor_func_SP0A19 */

/*************************************************************************
* FUNCTION
*	cam_module_func_config
*
* DESCRIPTION
*	This function maps the external camera module function API structure.
*
* PARAMETERS
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/

void SP0A19_image_sensor_func_config(void)
{
	image_sensor_func = &image_sensor_func_SP0A19;
}	/* cam_module_func_config() */

#endif
