/*****************************************************************************
*  Copyright Statement:
*  --------------------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of GalaxyCore Inc. (C) 2003
*
*****************************************************************************/

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   image_sensor.c
 *
 * Version:
 * --------
 *   SP0A19 MTK 6225 DRV V1.0.0 by zch
 *
 * Release Date:
 * ------------
 *   2012-09-12
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *
 * Author:
 * -------
 * -------
 *
 *============================================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/
#ifndef _IMAGE_SENSOR_H
#define _IMAGE_SENSOR_H

	#include "isp_if.h"
	
	//------------------------Engineer mode---------------------------------
	
	#define FACTORY_START_ADDR 70
	
	typedef enum group_enum {
	   AWB_GAIN=0,
	   PRE_GAIN,
	   SENSOR_DBLC,
	   GAMMA_ENABLE,
	   CMMCLK_CURRENT,
	   FRAME_RATE_LIMITATION,
	   REGISTER_EDITOR,
	   GROUP_TOTAL_NUMS
	} FACTORY_CCT_GROUP_ENUM;
	
	typedef enum register_index {
	   AWB_GAIN_R_INDEX=FACTORY_START_ADDR,
	   AWB_GAIN_B_INDEX,
	   SENSOR_DBLC_INDEX,
	   GAMMA_ENABLE_INDEX,
	   CMMCLK_CURRENT_INDEX,	   
	   FACTORY_END_ADDR
	} FACTORY_REGISTER_INDEX;
	
	typedef enum cct_register_index {
	   GLOBAL_GAIN_INDEX=0,
	   PRE_GAIN_R_INDEX,
	   PRE_GAIN_B_INDEX,
	   CCT_END_ADDR
	} CCT_REGISTER_INDEX;
	
	typedef struct
	{
	   kal_uint8   item_name_ptr[50];         // item name
	   kal_int32	item_value;                // item value
	   kal_bool    is_true_false;             // is this item for enable/disable functions
	   kal_bool	   is_read_only;              // is this item read only
	   kal_bool	   is_need_restart;           // after set this item need restart
	   kal_int32	min;                       // min value of item value	
	   kal_int32	max;                       // max value of item value	
	} ENG_sensor_info;
	
	// API FOR ENGINEER FACTORY MODE
	void  get_sensor_group_count(kal_int32* sensor_count_ptr);
	void  get_sensor_group_info(kal_uint16 group_idx, kal_int8* group_name_ptr, kal_int32* item_count_ptr);
	void  get_sensor_item_info(kal_uint16 group_idx,kal_uint16 item_idx, ENG_sensor_info* info_ptr);
	kal_bool set_sensor_item_info(kal_uint16 group_idx, kal_uint16 item_idx, kal_int32 item_value);
	
	//------------------------Engineer mode---------------------------------
	
	typedef struct {
		kal_uint32	addr;
		kal_uint32	para;
	} sensor_reg_struct;
	
	typedef struct {
		sensor_reg_struct	reg[FACTORY_END_ADDR];
		sensor_reg_struct	cct[CCT_END_ADDR];
	} sensor_data_struct;

	// write camera_para to sensor register 
	void camera_para_to_sensor(void);
	// update camera_para from sensor register 
	void sensor_to_camera_para(void);
	// config sensor callback function 
	void image_sensor_func_config(void);
	// Compact Image Sensor Module Power ON/OFF
	void cis_module_power_on(kal_bool on);
	
	/* HW PRODUCE I2C SIGNAL TO CONTROL SENSOR REGISTER */
	//#define HW_SCCB
	
	/* OUTPUT DEBUG INFO. BY UART */
	//#define OUTPUT_DEBUG_INFO

	typedef enum _SENSOR_TYPE {
		CMOS_SENSOR=0,
		CCD_SENSOR
	} SENSOR_TYPE;

	typedef struct {
		kal_uint16		id;
		SENSOR_TYPE		type;
	} SensorInfo;

	/* SENSOR PIXEL/LINE NUMBERS IN ONE PERIOD */
	#define VGA_PERIOD_PIXEL_NUMS						804//784
	#define VGA_PERIOD_LINE_NUMS						506//510

	
	/* SENSOR VGA SIZE */
	#define IMAGE_SENSOR_VGA_WIDTH					640
	#define IMAGE_SENSOR_VGA_HEIGHT					480


	/* SETUP TIME NEED TO BE INSERTED */
	#define IMAGE_SENSOR_VGA_INSERTED_PIXELS		72
	#define IMAGE_SENSOR_VGA_INSERTED_LINES		    34

	/* SENSOR READ/WRITE ID */
	#define SP0A19_WRITE_ID						       	(0x42) 
	#define SP0A19_READ_ID								(0x43)

	/* SENSOR CHIP VERSION */
	#define SP0A19_SENSOR_ID							 (0xa6)
	

#ifdef MCU_104M
	#define SENSOR_I2C_DELAY							0x40
#else
	#define SENSOR_I2C_DELAY							0x40
#endif

#define I2C_START_TRANSMISSION\
{ \
	volatile kal_uint32 j; \
	SET_SCCB_DATA_OUTPUT; \
	SET_SCCB_CLK_OUTPUT; \
	SET_SCCB_DATA_HIGH; \
	for(j=0;j<SENSOR_I2C_DELAY;j++);\
	SET_SCCB_CLK_HIGH; \
	for(j=0;j<SENSOR_I2C_DELAY;j++);\
	SET_SCCB_DATA_LOW; \
	for(j=0;j<SENSOR_I2C_DELAY;j++);\
	SET_SCCB_CLK_LOW; \
}

#define I2C_STOP_TRANSMISSION\
{ \
	volatile kal_uint32 j; \
	SET_SCCB_CLK_OUTPUT; \
	SET_SCCB_DATA_OUTPUT; \
	SET_SCCB_CLK_LOW; \
	SET_SCCB_DATA_LOW; \
	for(j=0;j<SENSOR_I2C_DELAY;j++);\
	SET_SCCB_CLK_HIGH; \
	for(j=0;j<SENSOR_I2C_DELAY;j++);\
	SET_SCCB_DATA_HIGH; \
}

#endif /* _IMAGE_SENSOR_H */
