#include "nucleus.h"
#include "sysdefs.h"
#include "hwddefs.h"
#include "exeapi.h"
#include "mmdapi.h"
#include "hwdapi.h"
#include "hwddma.h"
#include "hwdlcd.h"
#include "hwdcameraapi.h"
#include "hwdsensor.h"
#include "hwdcameracust.h"
#include "monapi.h"



#ifdef SYS_OPTION_CAMERA_ENABLE

#ifdef D_VGA_SP0A19

#define __DEBUG_SENSOR__SP0A19__

#ifdef __DEBUG_SENSOR__SP0A19__
#define DEBUG_SENSOR_SP0A19 MonPrintf
#else
#define DEBUG_SENSOR_SP0A19   
#endif

/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0A19_I2C_ADDR_W    			0x42
#define SP0A19_I2C_ADDR_R    			0x43
#define SP0A19_I2C_ACK				       0x0
uint32			s_sensor_mclk		 = 0;
uint32			Antiflicker		 = HWD_CMR_FLICK_FREQ_50HZ;
uint32			nightmode		 = HWD_CMR_NIGHT_MODE_OFF;      

    
 //Saturation
#define  SP0A19_P0_0xd4  0x88
#define  SP0A19_P0_0xde  0xa8

const uint16 SP0A19_SENSOR_INITIAL_SETTING_TABLE[] = 
{
	//SP0A19 ini
   0xfd,0x00,
   0x1C,0x28,
   0x32,0x00,
   0x0f,0x2f,
   0x10,0x2e,
   0x11,0x00,
   0x12,0x18,
   0x13,0x2f,
   0x14,0x00,
   0x15,0x3f,
   0x16,0x00,
   0x17,0x18,
   0x25,0x40,
   0x1a,0x0b,
   0x1b,0xc ,
   0x1e,0xb ,
   0x20,0x3f,
   0x21,0x13,
   0x22,0x19,
   0x26,0x1a,
   0x27,0xab,
   0x28,0xfd,
   0x30,0x00,
   0xfb,0x31,
   0x1f,0x08,
   
   ///*==Blacklevel===;;*/
   0xfd,0x00,
   0x65,0x00,
   0x66,0x00,
   0x67,0x00,
   0x68,0x00,
   0x45,0x00,
   0x46,0x0f,

   ///*==ae setting===;;*///24M 50HZ 15.098-8fps
   0xfd,0x00, 
   0x03,0x00, 
   0x04,0xe7, 
   0x06,0x00, 
   0x09,0x02, 
   0x0a,0xc4, 
   0xf0,0x4d, 
   0xf1,0x00, 
   0xfd,0x01, 
   0x90,0x0c, 
   0x92,0x01, 
   0x98,0x4d, 
   0x99,0x00, 
   0x9a,0x01, 
   0x9b,0x00, 


  ///*==status==;;*/
    0xfd,0x01,  
    0xce,0x9c,  
    0xcf,0x03,  
    0xd0,0x9c,  
    0xd1,0x03,       
    0xfd,0x01,  
    0xc4,0x56,  
    0xc5,0x8f,  
    0xca,0x30,  
    0xcb,0x45,  
    0xcc,0x70,  
    0xcd,0x48,  
    0xfd,0x00,

  /*==lsc  for st==*/
    0xfd,0x01,
    0x35,0x15,
    0x36,0x15,
    0x37,0x15,
    0x38,0x15,
    0x39,0x15,
    0x3a,0x15,
    0x3b,0x13,
    0x3c,0x15,
    0x3d,0x15,
    0x3e,0x15,
    0x3f,0x15,
    0x40,0x18,
    0x41,0x00,
    0x42,0x04,
    0x43,0x04,
    0x44,0x00,
    0x45,0x00,
    0x46,0x00,
    0x47,0x00,
    0x48,0x00,
    0x49,0xfd,
    0x4a,0x00,
    0x4b,0x00,
    0x4c,0xfd,
    0xfd,0x00,
    
    /*==AWB 1==*/    
    0xfd,0x01,          
    0x28,0xc5,          
    0x29,0x9b,                                         
    0x2e,0x02,         
    0x2f,0x16,          
    0x17,0x17,          
    0x18,0x19,	         
    0x19,0x45,                       
    0x2a,0xef,          
    0x2b,0x15,   
   
    /*==AWB 2==*/      
    0xfd,0x01,
    0x73,0x80,
    0x1a,0x80,
    0x1b,0x80,
   //===d65
    0x65,0xd5,//d6
    0x66,0xfa,//f0
    0x67,0x72,//7a
    0x68,0x8a,//9a
  //===indoor
    0x69,0xc6,//ab
    0x6a,0xee,//ca
    0x6b,0x94,//a3
    0x6c,0xab,//c1
  //===f
    0x61,0x7a,//82
    0x62,0x98,//a5
    0x63,0xc5,//d6
    0x64,0xe6,//ec
  //===cwf
   0x6d,0xb9,//a5
   0x6e,0xde,//c2
   0x6f,0xb2,//a7
   0x70,0xd5,//c5
   
  //=======skin detect;;;
   0xfd,0x01,
   0x08,0x15,
   0x09,0x04,
   0x0a,0x20,
   0x0b,0x12,
   0x0c,0x27,
   0x0d,0x06,
   0x0f,0x63,
  
  
    
  //====BPC_grad
    0xfd,0x00,
    0x79,0xf0,
    0x7a,0x80,  //f0
    0x7b,0x80,  //f0
    0x7c,0x20,  //f0  
    
  //smooth
    0xfd,0x00,
  	////====ͨƽֵ	
    0x57,0x06,//raw_dif_thr_outdoor
    0x58,0x0d,//raw_dif_thr_normal
    0x56,0x10,//raw_dif_thr_dummy
    0x59,0x10,//raw_dif_thr_lowlight
  	//GrGbƽֵ
    0x89,0x06,//raw_grgb_thr_outdoor 
    0x8a,0x0d,//raw_grgb_thr_normal  
    0x9c,0x10,//raw_grgb_thr_dummy   
    0x9d,0x10,//raw_grgb_thr_lowlight
  
  	//====Gr\Gb֮ƽǿ
    0x81,0xe0,//   ;raw_gflt_fac_outdoor
    0x82,0xe0,//;80;raw_gflt_fac_normal
    0x83,0x80,//   ;raw_gflt_fac_dummy
    0x84,0x40,//   ;raw_gflt_fac_lowlight
  	//====GrGbͨƽǿ  
    0x85,0xe0,//;raw_gf_fac_outdoor  
    0x86,0xc0,//;raw_gf_fac_normal  
    0x87,0x80,//;raw_gf_fac_dummy   
    0x88,0x40,//;raw_gf_fac_lowlight
  	//=====RBƽǿ  
    0x5a,0xff,//		 ;raw_rb_fac_outdoor
    0x5b,0xe0,// ;40;raw_rb_fac_normal
    0x5c,0x80,// 	 ;raw_rb_fac_dummy
    0x5d,0x00,// 	 ;raw_rb_fac_lowlight
    
  //===sharpen 
    0xfd,0x01,//
    0xe2,0x30,//	;sharpen_y_base
    0xe4,0xa0,//	;sharpen_y_max
    0xe5,0x04,// ;rangek_neg_outdoor
    0xd3,0x04,// ;rangek_pos_outdoor   
    0xd7,0x04,// ;range_base_outdoor   
    0xe6,0x04,// ;rangek_neg_normal
    0xd4,0x04,// ;rangek_pos_normal 
    0xd8,0x04,// ;range_base_normal  
    0xe7,0x08,// ;rangek_neg_dummy
    0xd5,0x08,// ;rangek_pos_dummy
    0xd9,0x08,// ;range_base_dummy 
    0xd2,0x10,// ;rangek_neg_lowlight
    0xd6,0x10,// ;rangek_pos_lowlight
    0xda,0x10,// ;range_base_lowlight
    0xe8,0x20,//;sharp_fac_pos_outdoor
    0xec,0x35,//;sharp_fac_neg_outdoor
    0xe9,0x20,//;sharp_fac_pos_nr
    0xed,0x35,//;sharp_fac_neg_nr
    0xea,0x20,//;sharp_fac_pos_dummy
    0xef,0x30,//;sharp_fac_neg_dummy
    0xeb,0x10,//;sharp_fac_pos_low
    0xf0,0x20,//;sharp_fac_neg_low 
    
  //===CCM
    0xfd,0x01,//
    0xa0,0x80,//;80(ɫӽɫ)
    0xa1,0x0 ,//;0 
    0xa2,0x0 ,//;0 
    0xa3,0xf3,//;f0
    0xa4,0x8e,//;a6
    0xa5,0x0 ,//;ea
    0xa6,0x0 ,//;0 
    0xa7,0xe6,//;e6
    0xa8,0x9a,//;9a
    0xa9,0x0 ,//;0 
    0xaa,0x3 ,//;33
    0xab,0xc ,//;c   
  //===gamma    
   0xfd,0x00,//
   0x8b,0x0 ,//;0 
   0x8c,0xC ,//;11
   0x8d,0x19,//;19 
   0x8e,0x2C,//;28 
   0x8f,0x49,//;46 
   0x90,0x61,//;61 
   0x91,0x77,//;78 
   0x92,0x8A,//;8A 
   0x93,0x9B,//;9B 
   0x94,0xA9,//;A9 
   0x95,0xB5,//;B5 
   0x96,0xC0,//;C0 
   0x97,0xCA,//;CA 
   0x98,0xD4,//;D4 
   0x99,0xDD,//;DD 
   0x9a,0xE6,//;E6 
   0x9b,0xEF,//;EF 
   0xfd,0x01,//;01 
   0x8d,0xF7,//;F7 
   0x8e,0xFF,//;FF 
/*
 0xfd,0x00
 0x8b,0x00,
 0x8c,0x14,
 0x8d,0x24,
 0x8e,0x3A,
 0x8f,0x59,
 0x90,0x70,
 0x91,0x85,
 0x92,0x96,
 0x93,0xA6,
 0x94,0xB3,
 0x95,0xBE,
 0x96,0xC9,
 0x97,0xD2,
 0x98,0xDB,
 0x99,0xE3,
 0x9a,0xEB,
 0x9b,0xF2,
 0xfd,0x01,
 0x8d,0xF9,
 0x8e,0xFF,
               */
  //===rpc
    0xfd,0x00,//
    0xe0,0x4c,//;48;4c;44;4c;3e;3c;3a;38;rpc_1base_max
    0xe1,0x3c,//;38;3c;36;3c;30;2e;2c;2a;rpc_2base_max
    0xe2,0x34,//;30;34;2e;34;2a;28;26;26;rpc_3base_max
    0xe3,0x2e,//;2c;2e;2a;2e;26;24;22;rpc_4base_max
    0xe4,0x2e,//;2c;2e;2a;2e;26;24;22;rpc_5base_max
    0xe5,0x2c,//;2a;2c;28;2c;24;22;20;rpc_6base_max
    0xe6,0x2c,//;2a;2c;28;2c;24;22;20;rpc_7base_max
    0xe8,0x2a,//;28;2a;26;2a;22;20;20;1e;rpc_8base_max
    0xe9,0x2a,//;28;2a;26;2a;22;20;20;1e;rpc_9base_max
    0xea,0x2a,//;28;2a;26;2a;22;20;20;1e;rpc_10base_max
    0xeb,0x28,//;26;28;24;28;20;1f;1e;1d;rpc_11base_max
    0xf5,0x28,//;26;28;24;28;20;1f;1e;1d;rpc_12base_max
    0xf6,0x28,//;26;28;24;28;20;1f;1e;1d;rpc_13base_max
  //===ae min gain  
    0xfd,0x01,//
    0x94,0xa0,//;rpc_max_indr
    0x95,0x28,//;1e;rpc_min_indr 
    0x9c,0xa0,//;rpc_max_outdr
    0x9d,0x28,//;rpc_min_outdr    
  
  //===ae target
    0xfd,0x00,
    0xed,0x8c,//;80 
    0xf7,0x88,//;7c 
    0xf8,0x80,//;70 
    0xec,0x7c,//;6c   
    0xef,0x74,//;99
    0xf9,0x70,//;90
    0xfa,0x68,//;80
    0xee,0x64,//;78
  
      
  //gray detect
    0xfd,0x01,//
    0x30,0x40,//
    0x31,0x70,//;
    0x32,0x40,//;80
    0x33,0xef,//;
    0x34,0x05,//;04
    0x4d,0x2f,//;40
    0x4e,0x20,//;
    0x4f,0x16,//;13         
    
  //====lowlight lum
    0xfd,0x00,// 
    0xb2,0x20,//;lum_limit
    0xb3,0x1f,//;lum_set
    0xb4,0x30,//;black_vt
    0xb5,0x45,//;white_vt
  
  //====saturation
    0xfd,0x00,//
    0xbe,0xff,//
    0xbf,0x01,// 
    0xc0,0xff,//
    0xc1,0xd8,//
    0xd3,0x88,//;0x78
    0xd4,SP0A19_P0_0xd4,//;0x78
    0xd6,0x80,//;0x78      
    0xd7,0x60,//;0x78
  
  //===HEQ
    0xfd,0x00,//
    0xdc,0x00,//
    0xdd,0x80,//
    0xde,SP0A19_P0_0xde,//;80
    0xdf,0x80,// 
     
  //====func enable
    0xfd,0x00,
    0x31,0x00,//10
    0x32,0x15,
    0x34,0x76,//16
    0x33,0xef,
    0x35,0x40,
    0x5f,0x51,
};


void	SP0A19_SensorDoReset(void)
{
//   HwdSensorRegisterWrite(0x12, 0x80);
//   HwdCmrCustDelayMs(3);
/*
   No Hardware reset for SP0A19
   HwdCmrCustSensorCtrlResetPin(FALSE);
   HwdCmrCustDelayMs(1);
   HwdCmrCustSensorCtrlResetPin(TRUE);
*/   
}
/***************************************************************************************************/
/*		Function Name :	   HwdSensorWeakup (D_VGA_SP0A19)*/
/*		Description :		  control mp81X gpio weakup sensor*/
/*						This function must Modify by host*/
/*		Argument:		None*/
/*		Return:			None*/
/*		Author :		
/*		History :*/
/***************************************************************************************************/
void SP0A19_SensorWeakup(void)
{
  //HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_ANA_ID,TRUE);
     HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_ANA_ID,TRUE);
   HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_CORE_ID,TRUE);
    HwdCmrCustSensorCtrlPowerDownPin(FALSE);
   CmrDelayMs(2);
   HwdCmrCustSensorCtrlPowerDownPin(TRUE);
   CmrDelayMs(2);
    HwdCmrCustSensorCtrlPowerDownPin(FALSE);
   //HwdCmrCustMclkCtrl(HWD_CMR_CUST_MCLK_USBPLL,TRUE);
   CmrDelayMs(5);
}

/***************************************************************************************************/
/*		Function Name :	HwdSensorPowerDown (D_VGA_SP0A19)*/
/*		Description :		control mp81X gpio power down sensor*/
/*						This function must Modify by host*/
/*		Argument:		None*/
/*		Return:			None*/
/*		Author :		
/*		History :*/
/***************************************************************************************************/
void SP0A19_SensorPowerDown(void)
{
   //HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_ANA_ID,TRUE);
   HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_ANA_ID,TRUE);
   HwdCmrCustSensorCtrl_LDO(HWD_CMR_CUST_SNR_LDO_CORE_ID,TRUE);
   
   HwdCmrCustSensorCtrlPowerDownPin(TRUE);
}

int SP0A19EV_Adjust(HwdEvLevelT level)
{
   switch(level)
   {
      case HWD_CMR_BRIGHT_1:  
		HwdSensorRegisterWrite(0xfd, 0x00);
		HwdSensorRegisterWrite(0xdc, 0xe0);
		break;

		case HWD_CMR_BRIGHT_2:
		HwdSensorRegisterWrite(0xfd, 0x00);
		HwdSensorRegisterWrite(0xdc, 0xf0);
		break;

		case HWD_CMR_BRIGHT_3: 
		HwdSensorRegisterWrite(0xfd, 0x00);
		HwdSensorRegisterWrite(0xdc, 0x00);
		break;

		case HWD_CMR_BRIGHT_4: 
		HwdSensorRegisterWrite(0xfd, 0x00);
		HwdSensorRegisterWrite(0xdc, 0x10);
		break;
		case HWD_CMR_BRIGHT_5: 
		HwdSensorRegisterWrite(0xfd, 0x00);
		HwdSensorRegisterWrite(0xdc, 0x20);
		break;
      default:
      break;
   }
   DEBUG_SENSOR_SP0A19("SP0A19EV_Adjust:%d\r\n",level);
   return 0;
}

int SP0A19AWB_ModeSelect(HwdAwbModeT Mode_select)
{
   uint8 temp;
   DEBUG_SENSOR_SP0A19("SP0A19AWB_ModeSelect:%d\r\n",Mode_select);
   switch(Mode_select)
   {
      case HWD_CMR_AWB_AUTO:
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xc5);
			HwdSensorRegisterWrite(0x29,0x9b);
			HwdSensorRegisterWrite(0xfd,0x00);   // Enable AWB
			HwdSensorRegisterWrite(0x32,0x15);
			HwdSensorRegisterWrite(0xe7,0x03);
			HwdSensorRegisterWrite(0xe7,0x00);
			HwdSensorRegisterWrite(0xfd,0x00);
		break;
		case HWD_CMR_AWB_SUNNY:   
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xbc);
			HwdSensorRegisterWrite(0x29,0x5d);
			HwdSensorRegisterWrite(0xfd,0x00); 
			HwdSensorRegisterWrite(0xe7,0x03);
			HwdSensorRegisterWrite(0xe7,0x00); 
		break;
		case HWD_CMR_AWB_CLOUDY:
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xbf);
			HwdSensorRegisterWrite(0x29,0x89);
			HwdSensorRegisterWrite(0xfd,0x00); 
			HwdSensorRegisterWrite(0xe7,0x03);
			HwdSensorRegisterWrite(0xe7,0x00); 
		break;
		case HWD_CMR_AWB_FLUORESCENT://office
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xaf);
			HwdSensorRegisterWrite(0x29,0x99);
			HwdSensorRegisterWrite(0xfd,0x00);  
			HwdSensorRegisterWrite(0xe7,0x03);
			HwdSensorRegisterWrite(0xe7,0x00);
		break;
		case HWD_CMR_AWB_TUNGSTEN://home
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0x89);
			HwdSensorRegisterWrite(0x29,0xb8);
			HwdSensorRegisterWrite(0xfd,0x00);  
			HwdSensorRegisterWrite(0xe7,0x03);
			HwdSensorRegisterWrite(0xe7,0x00);
          break;
      default:
          break;
   }
   return 0;
}

int SP0A19FlickFreqSelect(HwdFlickFreqT select)
{
#if 0
  UINT16  read_data ;
  switch  (select)
  { 
    case HWD_CMR_FLICK_FREQ_50HZ: 
		
            Antiflicker= HWD_CMR_FLICK_FREQ_50HZ;
				
             if(nightmode==HWD_CMR_NIGHT_MODE_OFF)
			 	
      {   
        //SCI_TRACE_LOW("normal mode 50hz\r\n");
        //caprure preview daylight 24M 50hz 15.098-8FPS
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xe7};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc4};
       HwdSensorRegisterWrite{0xf0,0x4d};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x0c};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x4d};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0x9c};
       HwdSensorRegisterWrite{0xcf,0x03};
       HwdSensorRegisterWrite{0xd0,0x9c};
       HwdSensorRegisterWrite{0xd1,0x03};
       HwdSensorRegisterWrite{0xfd,0x00};  	
      }    
      else
      { 
      		//SCI_TRACE_LOW("night mode 50hz\r\n");      
          //caprure preview night 24M 50hz 15.098-6FPS
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xe7};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc4};
       HwdSensorRegisterWrite{0xf0,0x4d};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x10};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x4d};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0xd0};
       HwdSensorRegisterWrite{0xcf,0x04};
       HwdSensorRegisterWrite{0xd0,0xd0};
       HwdSensorRegisterWrite{0xd1,0x04};
       HwdSensorRegisterWrite{0xfd,0x00};   
      }     
         break;
   case case HWD_CMR_FLICK_FREQ_60HZ :

       Antiflicker= HWD_CMR_FLICK_FREQ_60HZ;

       if(nightmode==HWD_CMR_NIGHT_MODE_OFF)
              	{
       //SCI_TRACE_LOW("night mode 60hz\r\n");      
       //caprure preview night 24M 60hz 15.058-8FPS      	
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xc0};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc9};
       HwdSensorRegisterWrite{0xf0,0x40};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x0f};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x40};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0xc0};
       HwdSensorRegisterWrite{0xcf,0x03};
       HwdSensorRegisterWrite{0xd0,0xc0};
       HwdSensorRegisterWrite{0xd1,0x03};
       HwdSensorRegisterWrite{0xfd,0x00};        		
              	}
       else
       	{
      //SCI_TRACE_LOW("night mode 60hz\r\n");      
       //caprure preview night 24M 60hz 15.058-6FPS 
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xc0};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc9};
       HwdSensorRegisterWrite{0xf0,0x40};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x14};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x40};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0x00};
       HwdSensorRegisterWrite{0xcf,0x05};
       HwdSensorRegisterWrite{0xd0,0x00};
       HwdSensorRegisterWrite{0xd1,0x05};
       HwdSensorRegisterWrite{0xfd,0x00};   
       	}  
       	break; 
     case HWD_CMR_FLICK_FREQ_AUTO:

             Antiflicker= HWD_CMR_FLICK_FREQ_50HZ;
		
	          if(nightmode==HWD_CMR_NIGHT_MODE_OFF)
	    {
	    //SCI_TRACE_LOW("normal mode 50hz\r\n");
        //caprure preview daylight 24M 50hz 15.098-8FPS
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xe7};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc4};
       HwdSensorRegisterWrite{0xf0,0x4d};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x0c};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x4d};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0x9c};
       HwdSensorRegisterWrite{0xcf,0x03};
       HwdSensorRegisterWrite{0xd0,0x9c};
       HwdSensorRegisterWrite{0xd1,0x03};
       HwdSensorRegisterWrite{0xfd,0x00};  	
      }    
      else
      	{ 
      		//SCI_TRACE_LOW("night mode 50hz\r\n");      
          //caprure preview night 24M 50hz 15.098-6FPS
       HwdSensorRegisterWrite{0xfd,0x00};
       HwdSensorRegisterWrite{0x03,0x00};
       HwdSensorRegisterWrite{0x04,0xe7};
       HwdSensorRegisterWrite{0x06,0x00};
       HwdSensorRegisterWrite{0x09,0x02};
       HwdSensorRegisterWrite{0x0a,0xc4};
       HwdSensorRegisterWrite{0xf0,0x4d};
       HwdSensorRegisterWrite{0xf1,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0x90,0x10};
       HwdSensorRegisterWrite{0x92,0x01};
       HwdSensorRegisterWrite{0x98,0x4d};
       HwdSensorRegisterWrite{0x99,0x00};
       HwdSensorRegisterWrite{0x9a,0x01};
       HwdSensorRegisterWrite{0x9b,0x00};
       HwdSensorRegisterWrite{0xfd,0x01};
       HwdSensorRegisterWrite{0xce,0xd0};
       HwdSensorRegisterWrite{0xcf,0x04};
       HwdSensorRegisterWrite{0xd0,0xd0};
       HwdSensorRegisterWrite{0xd1,0x04};
       HwdSensorRegisterWrite{0xfd,0x00};   
       }     
         break;
	      default;
	      break;  	    	
   }
   DEBUG_SENSOR_SP0A19("SP0A19FlickFreqSelect:%d\r\n",select);
   #endif
   return 0;
}


int SP0A19NightModeSelect(HwdNightModeT select)
{
   DEBUG_SENSOR_SP0A19("SP0A19NightModeSelect:%d\r\n",select);
   switch(select)
   {
      case HWD_CMR_NIGHT_MODE_ON:
       nightmode=HWD_CMR_NIGHT_MODE_ON;
					   
			if(Antiflicker== HWD_CMR_FLICK_FREQ_50HZ)
				{
			//SCI_TRACE_LOW("night mode 50hz\r\n");
			//caprure preview night 24M 50hz 15-6FPS		
       HwdSensorRegisterWrite(0xfd,0x00);
       HwdSensorRegisterWrite(0x03,0x00);
       HwdSensorRegisterWrite(0x04,0xe7);
       HwdSensorRegisterWrite(0x06,0x00);
       HwdSensorRegisterWrite(0x09,0x02);
       HwdSensorRegisterWrite(0x0a,0xc4);
       HwdSensorRegisterWrite(0xf0,0x4d);
       HwdSensorRegisterWrite(0xf1,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0x90,0x10);
       HwdSensorRegisterWrite(0x92,0x01);
       HwdSensorRegisterWrite(0x98,0x4d);
       HwdSensorRegisterWrite(0x99,0x00);
       HwdSensorRegisterWrite(0x9a,0x01);
       HwdSensorRegisterWrite(0x9b,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0xce,0xd0);
       HwdSensorRegisterWrite(0xcf,0x04);
       HwdSensorRegisterWrite(0xd0,0xd0);
       HwdSensorRegisterWrite(0xd1,0x04);
       HwdSensorRegisterWrite(0xfd,0x00);  	
      }    
      else
      { 
      		//SCI_TRACE_LOW("night mode 60hz\r\n");      
          //caprure preview night 24M 60hz 15.098-6FPS
       HwdSensorRegisterWrite(0xfd,0x00);
       HwdSensorRegisterWrite(0x03,0x00);
       HwdSensorRegisterWrite(0x04,0xc0);
       HwdSensorRegisterWrite(0x06,0x00);
       HwdSensorRegisterWrite(0x09,0x02);
       HwdSensorRegisterWrite(0x0a,0xc9);
       HwdSensorRegisterWrite(0xf0,0x40);
       HwdSensorRegisterWrite(0xf1,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0x90,0x14);
       HwdSensorRegisterWrite(0x92,0x01);
       HwdSensorRegisterWrite(0x98,0x40);
       HwdSensorRegisterWrite(0x99,0x00);
       HwdSensorRegisterWrite(0x9a,0x01);
       HwdSensorRegisterWrite(0x9b,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0xce,0x00);
       HwdSensorRegisterWrite(0xcf,0x05);
       HwdSensorRegisterWrite(0xd0,0x00);
       HwdSensorRegisterWrite(0xd1,0x05);
       HwdSensorRegisterWrite(0xfd,0x00);  
      }     
         break;
   case HWD_CMR_NIGHT_MODE_OFF:
       nightmode=HWD_CMR_NIGHT_MODE_OFF;
					   
			if(Antiflicker== HWD_CMR_FLICK_FREQ_50HZ)
              	{
       //SCI_TRACE_LOW("normal mode 50hz\r\n");      
       //caprure preview daylight 24M 50hz 15.058-8FPS      	
       HwdSensorRegisterWrite(0xfd,0x00);
       HwdSensorRegisterWrite(0x03,0x00);
       HwdSensorRegisterWrite(0x04,0xe7);
       HwdSensorRegisterWrite(0x06,0x00);
       HwdSensorRegisterWrite(0x09,0x02);
       HwdSensorRegisterWrite(0x0a,0xc4);
       HwdSensorRegisterWrite(0xf0,0x4d);
       HwdSensorRegisterWrite(0xf1,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0x90,0x0c);
       HwdSensorRegisterWrite(0x92,0x01);
       HwdSensorRegisterWrite(0x98,0x4d);
       HwdSensorRegisterWrite(0x99,0x00);
       HwdSensorRegisterWrite(0x9a,0x01);
       HwdSensorRegisterWrite(0x9b,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0xce,0x9c);
       HwdSensorRegisterWrite(0xcf,0x03);
       HwdSensorRegisterWrite(0xd0,0x9c);
       HwdSensorRegisterWrite(0xd1,0x03);
       HwdSensorRegisterWrite(0xfd,0x00);        		
              	}
       else
       	{
      //SCI_TRACE_LOW("normal mode 60hz\r\n");      
       //caprure preview daylight 24M 60hz 15.058-8FPS 
       HwdSensorRegisterWrite(0xfd,0x00);
       HwdSensorRegisterWrite(0x03,0x00);
       HwdSensorRegisterWrite(0x04,0xc0);
       HwdSensorRegisterWrite(0x06,0x00);
       HwdSensorRegisterWrite(0x09,0x02);
       HwdSensorRegisterWrite(0x0a,0xc9);
       HwdSensorRegisterWrite(0xf0,0x40);
       HwdSensorRegisterWrite(0xf1,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0x90,0x0f);
       HwdSensorRegisterWrite(0x92,0x01);
       HwdSensorRegisterWrite(0x98,0x40);
       HwdSensorRegisterWrite(0x99,0x00);
       HwdSensorRegisterWrite(0x9a,0x01);
       HwdSensorRegisterWrite(0x9b,0x00);
       HwdSensorRegisterWrite(0xfd,0x01);
       HwdSensorRegisterWrite(0xce,0xc0);
       HwdSensorRegisterWrite(0xcf,0x03);
       HwdSensorRegisterWrite(0xd0,0xc0);
       HwdSensorRegisterWrite(0xd1,0x03);
       HwdSensorRegisterWrite(0xfd,0x00);  
				}  
				break;
				default:
				break;
}
   return 0;
}


int SP0A19SceneSelect(HwdSceneModeT select)
{
   DEBUG_SENSOR_SP0A19("SP0A19SceneSelect:%d\r\n",select);
   return 0;
   switch(select)
   {
      case HWD_CMR_SCENE_OUTDOOR: /*Sunny*/
      case HWD_CMR_SCENE_LANDSCAPE:/*Outdoor*/
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xbc);
			HwdSensorRegisterWrite(0x29,0x5d);
			HwdSensorRegisterWrite(0xfd,0x00);  
      break;
      case HWD_CMR_SCENE_SUNSET:/*Cloudy*/
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xbf);
			HwdSensorRegisterWrite(0x29,0x89);
			HwdSensorRegisterWrite(0xfd,0x00);  
      break;
      case HWD_CMR_SCENE_INDOOR:/*Office*/
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xaf);
			HwdSensorRegisterWrite(0x29,0x99);
			HwdSensorRegisterWrite(0xfd,0x00);
      break;
      case HWD_CMR_SCENE_NIGHT: /*Night*/
			HwdSensorRegisterWrite(0xfd,0x00);
			HwdSensorRegisterWrite(0x32,0x05);
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0x89);
			HwdSensorRegisterWrite(0x29,0xb8);
			HwdSensorRegisterWrite(0xfd,0x00);  
      break;
      default:
      case HWD_CMR_SCENE_AUTO:
			HwdSensorRegisterWrite(0xfd,0x01);
			HwdSensorRegisterWrite(0x28,0xc5);
			HwdSensorRegisterWrite(0x29,0x9b);
			HwdSensorRegisterWrite(0xfd,0x00);   // Enable AWB
			HwdSensorRegisterWrite(0x32,0x15);
			HwdSensorRegisterWrite(0xfd,0x00);
      break;
   }

   return 0;
}

int SP0A19BrightnessSelect(HwdBrightnessModeT select)
{
   DEBUG_SENSOR_SP0A19("SP0A19BrightnessSelect:%d\r\n",select);
   SP0A19EV_Adjust(select);
   return 0;

}

int SP0A19SaturationSelect(HwdSaturationT select)
{
   DEBUG_SENSOR_SP0A19("SP0A19SaturationSelect:%d\r\n",select);
   ///return 0;
#if 1
   switch(select)
   {
      case HWD_CMR_SATURATION_0:
         HwdSensorRegisterWrite(0xfd, 0x00);
         HwdSensorRegisterWrite(0xd4, SP0A19_P0_0xd4-0x30);
       
      break;
      case HWD_CMR_SATURATION_1:
         HwdSensorRegisterWrite(0xfd, 0x00);
         HwdSensorRegisterWrite(0xd4, SP0A19_P0_0xd4-0x20);
        
      break;
      case HWD_CMR_SATURATION_2:
         HwdSensorRegisterWrite(0xfd, 0x00);
         HwdSensorRegisterWrite(0xd4, SP0A19_P0_0xd4-0x10);
       
      break;
      case HWD_CMR_SATURATION_3:
         HwdSensorRegisterWrite(0xfd, 0x00);
         HwdSensorRegisterWrite(0xd4, SP0A19_P0_0xd4);
        
      break;
      default:
      case HWD_CMR_SATURATION_4:
         HwdSensorRegisterWrite(0xfd, 0x00);
         HwdSensorRegisterWrite(0xd4, SP0A19_P0_0xd4+0x10);
       
      break;
   }
   #endif
   return 0;

}
int SP0A19ContrastSelect(HwdContrastT select)
{
   uint8 temp;

   DEBUG_SENSOR_SP0A19("SP0A19ContrastSelect:%d\r\n",select);
   switch(select)
   {
   /*
   	case HWD_CMR_CONTRAST_0:
    HwdSensorRegisterWrite(0xfd, 0x00);
	  HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde-0x30);
      break;
   	 case HWD_CMR_CONTRAST_0:
    HwdSensorRegisterWrite(0xfd, 0x00);
	  HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde-0x20);
      break;*/
      case HWD_CMR_CONTRAST_0:
         HwdSensorRegisterWrite(0xfd, 0x00);
	  HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde-0x10);
      break;
      
      case HWD_CMR_CONTRAST_1:
         HwdSensorRegisterWrite(0xfd, 0x00);
	  HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde);
      break;
      case HWD_CMR_CONTRAST_2:
          HwdSensorRegisterWrite(0xfd, 0x00);
	   HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde+0x10);
      break;
      case HWD_CMR_CONTRAST_3:
          HwdSensorRegisterWrite(0xfd, 0x00);
	   HwdSensorRegisterWrite(0xde, SP0A19_P0_0xde+0x20);
      break;
     
   }

   return 0;

}

int SP0A19FPSSelect(SensorFPSSetTypeT type,uint8 Fps)
{
   DEBUG_SENSOR_SP0A19("SP0A19FPSSelect:%d:%d\r\n",type,Fps);
   return 0;
}
int SP0A19ModeSwitch(int Mode)
{
  
#if 1
   switch(Mode)
   {
      case SNR_MODE_PRV_640x480_15FPS:
      case SNR_MODE_CAP_640x480_15FPS:
      	#if 0 
         HwdSensorRegisterWrite(0xfd,0x00);
         HwdSensorRegisterWrite(0x03,0x00);       
         HwdSensorRegisterWrite(0x04,0xe7);
         HwdSensorRegisterWrite(0x06,0x00);
         HwdSensorRegisterWrite(0x09,0x02);          
         HwdSensorRegisterWrite(0x0a,0xc4);
         HwdSensorRegisterWrite(0xf0,0x4d);
         HwdSensorRegisterWrite(0xf1,0x00);
         HwdSensorRegisterWrite(0xfd,0x01);
         HwdSensorRegisterWrite(0x90,0x06);
         HwdSensorRegisterWrite(0x92,0x01);
         HwdSensorRegisterWrite(0x98,0x4d);
         HwdSensorRegisterWrite(0x99,0x00);
         HwdSensorRegisterWrite(0x9a,0x01);
         HwdSensorRegisterWrite(0x9b,0x00);
         HwdSensorRegisterWrite(0xfd,0x01);
         HwdSensorRegisterWrite(0xce,0xce);
         HwdSensorRegisterWrite(0xcf,0x01);
         HwdSensorRegisterWrite(0xd0,0xce);
         HwdSensorRegisterWrite(0xd1,0x01);
     #endif                            
      break;
      case SNR_MODE_CAP_640x480_7FPS:
         /*7fps auto*/
         #if 0
         HwdSensorRegisterWrite(0xfd,0x00);
         HwdSensorRegisterWrite(0x03,0x00);
         HwdSensorRegisterWrite(0x04,0x6c);
         HwdSensorRegisterWrite(0x06,0x00);
         HwdSensorRegisterWrite(0x09,0x09);
         HwdSensorRegisterWrite(0x0a,0xb3);
         HwdSensorRegisterWrite(0xf0,0x24);
         HwdSensorRegisterWrite(0xf1,0x00);
         HwdSensorRegisterWrite(0xfd,0x01);
         HwdSensorRegisterWrite(0x90,0x0e);
         HwdSensorRegisterWrite(0x92,0x01);
         HwdSensorRegisterWrite(0x98,0x24);
         HwdSensorRegisterWrite(0x99,0x00);
         HwdSensorRegisterWrite(0x9a,0x01);
         HwdSensorRegisterWrite(0x9b,0x00);
         HwdSensorRegisterWrite(0xfd,0x01);
         HwdSensorRegisterWrite(0xce,0xf8);
         HwdSensorRegisterWrite(0xcf,0x01);
         HwdSensorRegisterWrite(0xd0,0xf8);
         HwdSensorRegisterWrite(0xd1,0x01);
#endif 
      break;
      case SNR_MODE_VIDEO_640x480_15FPS:
         /*Fixed Frame Rate*/
         #if 0
          HwdSensorRegisterWrite(0xfd,0x00);
          HwdSensorRegisterWrite(0x03,0x00);
          HwdSensorRegisterWrite(0x04,0xe7);
          HwdSensorRegisterWrite(0x06,0x00);
          HwdSensorRegisterWrite(0x09,0x02);
          HwdSensorRegisterWrite(0x0a,0xc4);
          HwdSensorRegisterWrite(0xf0,0x4d);
          HwdSensorRegisterWrite(0xf1,0x00);
          HwdSensorRegisterWrite(0xfd,0x01);
          HwdSensorRegisterWrite(0x90,0x06);
          HwdSensorRegisterWrite(0x92,0x01);
          HwdSensorRegisterWrite(0x98,0x4d);
          HwdSensorRegisterWrite(0x99,0x00);
          HwdSensorRegisterWrite(0x9a,0x01);
          HwdSensorRegisterWrite(0x9b,0x00);
          HwdSensorRegisterWrite(0xfd,0x01);
          HwdSensorRegisterWrite(0xce,0xce);
          HwdSensorRegisterWrite(0xcf,0x01);
          HwdSensorRegisterWrite(0xd0,0xce);
          HwdSensorRegisterWrite(0xd1,0x01);
   #endif   
      break;
   }
#endif   
   return CMR_OK;
}


#if 1
HwdSensorInfoT gSensorSP0A19Def=
{

   "SP0A19",                                     /*Sensor Name*/
   0X42,//SP0A19_I2C_ADDR,                              /*Sensor I2C Address*/
   {0x02,0x02},                                    /*Sensor ID Address */
   0xa6a6,//SP0A19_SENSOR_ID,                             /*Sensor ID*/
   0x01,//SP0A19_SENSOR_TYPE,                          /*Sensor Type*/
   (HWD_PVI_CR_CLK_POL(CMR_SNR_DATA_RISING_LATCH)| /*Sensor Timing*/
      HWD_PVI_CR_V_POL(CMR_SNR_VALID_H_LOW)|
      HWD_PVI_CR_H_POL(CMR_SNR_VALID_H_HIGH)|
      HWD_PVI_CR_YUV_ORDER(CMR_SNR_ORDER_YUYV)
   ), 
   CMR_SENSOR_CLK_24M,                             /*Sensor Clock Frequency input in Camera Mode*/  
   CMR_SENSOR_CLK_24M,                             /*Sensor Clock Frequency input in Video Mode*/
   640,                                           /*Sensor Output Width in Pixels*/
   480,                                           /*Sensor Output Height in Pixels*/ 
   sizeof(SP0A19_SENSOR_INITIAL_SETTING_TABLE),  /*Initial Table Len*/
   SP0A19_SENSOR_INITIAL_SETTING_TABLE,          /*Initial Table Pointer*/
   SP0A19_SensorDoReset,                         /*Do Reset*/
   SP0A19_SensorWeakup,                          /*Wake up*/
   SP0A19_SensorPowerDown,                       /*Power Down*/
   SP0A19EV_Adjust,                              /*EV Adjust*/
   SP0A19AWB_ModeSelect,                         /*AWB control*/
   SP0A19FlickFreqSelect,                        /*Flicker Frequency Select*/
   SP0A19NightModeSelect,                        /*Night Mode Control*/
   SP0A19SceneSelect,                            /*Scene Select*/
   SP0A19BrightnessSelect,                       /*Brightness Select*/
   SP0A19SaturationSelect,                       /*Saturation*/
   SP0A19ContrastSelect,                         /*Contrast*/
   SP0A19FPSSelect,                              /*Frame per second(FPS) Setting*/
   SP0A19ModeSwitch                              /*Mode Switch*/ 
};
#endif
#endif
#endif

