/*============================================================================

                 Camera Interface Device Driver Source File
                   for Gaxlxycroe 0309 30W Mega Pixel sensor

   DESCRIPTION
     This file contains the definitions needed for the camera interface
     device driver.


============================================================================*/

/*============================================================================

                      EDIT HISTORY FOR FILE

 This section contains comments describing changes made to this file.
 Notice that changes are listed in reverse chronological order.

 $Header: 

 when      who          what, where, why
 --------  ---    ----------------------------------------------------------
2010-9-2      Initial release.

============================================================================*/

/*============================================================================
                        INCLUDE FILES
============================================================================*/

#include "camsensor.h"

#if defined (FEATURE_CAMERA) && defined (USE_CAMSENSOR_SUPERPIX_SP0A19)
#include "camsensor_sp0A19_ycbcr.h"
#include "camera.h"
#include "ipl.h"
#include "msg.h"
#include "i2c.h"
#include "clk.h"

#ifdef LOCAL
#undef LOCAL
#endif
#define LOCAL static

#define  SP0A19_P0_0xde  0xa8

LOCAL   boolean  blIs60Hz = FALSE;  //Judge is working 60mhz light fre


/*===========================================================================
                          MACRO DEFINITIONS
============================================================================*/

/*============================================================================
                          DUMMY DEFINITIONS TO MAKE LINT HAPPY
============================================================================*/

/*============================================================================
                          CONSTANT DEFINITIONS
============================================================================*/

#define CAMSENSOR_SP0A19_I2C_SLAVE_ID     0x42       /* Camera image sensor slave address  */
#define SP0A19_SENSOR_ID							0xA6

typedef struct register_address_value_pair_struct {
uint16 register_address;
uint16 register_value;
} register_address_value_pair;

/*============================================================================
                        STATIC VARIABLES 
============================================================================*/
const char camsensor_sp0A19_sensor_name[]  = "SP0A19 VGA";
LOCAL const char camsensor_sp0A19_sensor_suffix[]="sp0A19_ycbcr";

/*============================================================================
                          INTERNAL API PROTOTYPES
============================================================================*/

LOCAL boolean camsensor_sp0A19_sensor_init(void);
LOCAL void camsensor_sp0A19_init_camsensor_info(void);
LOCAL void camsensor_sp0A19_register (camsensor_function_table_type *camsensor_function_table_ptr);
void camsensor_sp0A19_power_down(void);
/*============================================================================
                          EXTERNAL API DEFINITIONS
============================================================================*/

boolean sp0A19_i2c_read(uint8 reg, uint8 * pData)
{
  i2c_status_type res;
  i2c_rw_cmd_type gc_i2c_cmd;
  static byte data = 0;

  gc_i2c_cmd.bus_id = I2C_BUS_HW_CTRL;
  gc_i2c_cmd.slave_addr = CAMSENSOR_SP0A19_I2C_SLAVE_ID;
  gc_i2c_cmd.options = I2C_REG_DEV;  	
  gc_i2c_cmd.addr.reg = reg;
  gc_i2c_cmd.buf_ptr = &data;
  gc_i2c_cmd.len = 1;

  res = i2c_read(&gc_i2c_cmd);

  if (res == I2C_SUCCESS)
  {
    *pData = data;
  }
  //MSG_HIGH("@sp0A19_i2c_read(), reg=0x%x, data=0x%x, res=%d", reg, data, res);
  return res == I2C_SUCCESS;
}

boolean sp0A19_i2c_write(uint8 reg, uint8 data)
{
  i2c_status_type res;
  i2c_rw_cmd_type gc_i2c_cmd;
  static uint8 my_data;
  my_data = data;
  gc_i2c_cmd.bus_id = I2C_BUS_HW_CTRL;
  gc_i2c_cmd.slave_addr = CAMSENSOR_SP0A19_I2C_SLAVE_ID;
  gc_i2c_cmd.options = I2C_REG_DEV;  	
  gc_i2c_cmd.addr.reg = reg;
  gc_i2c_cmd.buf_ptr = &my_data;
  gc_i2c_cmd.len = 1;

  res = i2c_write(&gc_i2c_cmd);
  //MSG_HIGH("@sp0A19_i2c_write(), reg=0x%x, data=0x%x, res=%d", reg, data, res);
  return res == I2C_SUCCESS;
}

static void SP0A19_Initial_Setting(void)
{

  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0x1C,0x28);
  sp0A19_i2c_write(0x32,0x00);
  sp0A19_i2c_write(0x0f,0x2f);
  sp0A19_i2c_write(0x10,0x2e);
  sp0A19_i2c_write(0x11,0x00);
  sp0A19_i2c_write(0x12,0x18);
  sp0A19_i2c_write(0x13,0x2f);
  sp0A19_i2c_write(0x14,0x00);
  sp0A19_i2c_write(0x15,0x3f);
  sp0A19_i2c_write(0x16,0x00);
  sp0A19_i2c_write(0x17,0x18);
  sp0A19_i2c_write(0x25,0x40);
  sp0A19_i2c_write(0x1a,0x0b);
  sp0A19_i2c_write(0x1b,0xc );
  sp0A19_i2c_write(0x1e,0xb );
  sp0A19_i2c_write(0x20,0x3f);
  sp0A19_i2c_write(0x21,0x13);
  sp0A19_i2c_write(0x22,0x19);
  sp0A19_i2c_write(0x26,0x1a);
  sp0A19_i2c_write(0x27,0xab);
  sp0A19_i2c_write(0x28,0xfd);
  sp0A19_i2c_write(0x30,0x00);
  sp0A19_i2c_write(0xfb,0x31);
  sp0A19_i2c_write(0x1f,0x08);     
//Blacklevel
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0x65,0x00);//06
  sp0A19_i2c_write(0x66,0x00);//06
  sp0A19_i2c_write(0x67,0x00);//06
  sp0A19_i2c_write(0x68,0x00);//06
  sp0A19_i2c_write(0x45,0x00);
  sp0A19_i2c_write(0x46,0x0f);
  
  //ae setting
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0x03,0x01);
  sp0A19_i2c_write(0x04,0x32);
  sp0A19_i2c_write(0x06,0x00);
  sp0A19_i2c_write(0x09,0x01);
  sp0A19_i2c_write(0x0a,0x46);
  sp0A19_i2c_write(0xf0,0x66);
  sp0A19_i2c_write(0xf1,0x00);
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x90,0x0c);
  sp0A19_i2c_write(0x92,0x01);
  sp0A19_i2c_write(0x98,0x66);
  sp0A19_i2c_write(0x99,0x00);
  sp0A19_i2c_write(0x9a,0x01);
  sp0A19_i2c_write(0x9b,0x00);
  
//Status
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0xce,0xc8);
  sp0A19_i2c_write(0xcf,0x04);
  sp0A19_i2c_write(0xd0,0xc8);
  sp0A19_i2c_write(0xd1,0x04); 
   
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0xc4,0x56);
  sp0A19_i2c_write(0xc5,0x8f);
  sp0A19_i2c_write(0xca,0x30);
  sp0A19_i2c_write(0xcb,0x45);
  sp0A19_i2c_write(0xcc,0x70);
  sp0A19_i2c_write(0xcd,0x48);
  sp0A19_i2c_write(0xfd,0x00);
  
 //lsc  for st
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x35,0x15);
  sp0A19_i2c_write(0x36,0x15);
  sp0A19_i2c_write(0x37,0x15);
  sp0A19_i2c_write(0x38,0x15);
  sp0A19_i2c_write(0x39,0x15);
  sp0A19_i2c_write(0x3a,0x15);
  sp0A19_i2c_write(0x3b,0x13);
  sp0A19_i2c_write(0x3c,0x15);
  sp0A19_i2c_write(0x3d,0x15);
  sp0A19_i2c_write(0x3e,0x15);
  sp0A19_i2c_write(0x3f,0x15);
  sp0A19_i2c_write(0x40,0x18);
  sp0A19_i2c_write(0x41,0x00);
  sp0A19_i2c_write(0x42,0x04);
  sp0A19_i2c_write(0x43,0x04);
  sp0A19_i2c_write(0x44,0x00);
  sp0A19_i2c_write(0x45,0x00);
  sp0A19_i2c_write(0x46,0x00);
  sp0A19_i2c_write(0x47,0x00);
  sp0A19_i2c_write(0x48,0x00);
  sp0A19_i2c_write(0x49,0xfd);
  sp0A19_i2c_write(0x4a,0x00);
  sp0A19_i2c_write(0x4b,0x00);
  sp0A19_i2c_write(0x4c,0xfd);
  sp0A19_i2c_write(0xfd,0x00);  
//awb 1
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x28,0xc5);
  sp0A19_i2c_write(0x29,0x9b);
  sp0A19_i2c_write(0x2e,0x02);	
  sp0A19_i2c_write(0x2f,0x16);
  sp0A19_i2c_write(0x17,0x17);
  sp0A19_i2c_write(0x18,0x19);
  sp0A19_i2c_write(0x19,0x45);
  sp0A19_i2c_write(0x2a,0xef);
  sp0A19_i2c_write(0x2b,0x15);  
  //awb2
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x73,0x80);
  sp0A19_i2c_write(0x1a,0x80);
  sp0A19_i2c_write(0x1b,0x80); 
//d65
  sp0A19_i2c_write(0x65,0xd5);
  sp0A19_i2c_write(0x66,0xfa);
  sp0A19_i2c_write(0x67,0x72);
  sp0A19_i2c_write(0x68,0x8a);
//indoor
  sp0A19_i2c_write(0x69,0xc6);
  sp0A19_i2c_write(0x6a,0xee);
  sp0A19_i2c_write(0x6b,0x94);
  sp0A19_i2c_write(0x6c,0xab);
//f
  sp0A19_i2c_write(0x61,0x7a);
  sp0A19_i2c_write(0x62,0x98);
  sp0A19_i2c_write(0x63,0xc5);
  sp0A19_i2c_write(0x64,0xe6);
//cwf
 sp0A19_i2c_write(0x6d,0xb9);
 sp0A19_i2c_write(0x6e,0xde);
 sp0A19_i2c_write(0x6f,0xb2);
 sp0A19_i2c_write(0x70,0xd5);
 
//////////////////skin detect//////
 sp0A19_i2c_write(0xfd,0x01);
 sp0A19_i2c_write(0x08,0x15);
 sp0A19_i2c_write(0x09,0x04);
 sp0A19_i2c_write(0x0a,0x20);
 sp0A19_i2c_write(0x0b,0x12);
 sp0A19_i2c_write(0x0c,0x27);
 sp0A19_i2c_write(0x0d,0x06);
 sp0A19_i2c_write(0x0f,0x63);  
//BPC_grad
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0x79,0xf0);
  sp0A19_i2c_write(0x7a,0x80);  //f0
  sp0A19_i2c_write(0x7b,0x80);   //f0
  sp0A19_i2c_write(0x7c,0x20);     //f0
  
//smooth
  sp0A19_i2c_write(0xfd,0x00);
	//ͨƽֵ	
  sp0A19_i2c_write(0x57,0x06);	//raw_dif_thr_outdoor
  sp0A19_i2c_write(0x58,0x0d); //raw_dif_thr_normal
  sp0A19_i2c_write(0x56,0x10); //raw_dif_thr_dummy
  sp0A19_i2c_write(0x59,0x10); //raw_dif_thr_lowlight
	//GrGbƽֵ
  sp0A19_i2c_write(0x89,0x06);	//raw_grgb_thr_outdoor 
  sp0A19_i2c_write(0x8a,0x0d); //raw_grgb_thr_normal  
  sp0A19_i2c_write(0x9c,0x10); //raw_grgb_thr_dummy   
  sp0A19_i2c_write(0x9d,0x10); //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  sp0A19_i2c_write(0x81,0xe0);    //raw_gflt_fac_outdoor
  sp0A19_i2c_write(0x82,0xe0); //80//raw_gflt_fac_normal
  sp0A19_i2c_write(0x83,0x80);    //raw_gflt_fac_dummy
  sp0A19_i2c_write(0x84,0x40);    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  sp0A19_i2c_write(0x85,0xe0); //raw_gf_fac_outdoor  
  sp0A19_i2c_write(0x86,0xc0); //raw_gf_fac_normal  
  sp0A19_i2c_write(0x87,0x80); //raw_gf_fac_dummy   
  sp0A19_i2c_write(0x88,0x40); //raw_gf_fac_lowlight
	//RBƽǿ  
  sp0A19_i2c_write(0x5a,0xff);		 //raw_rb_fac_outdoor
  sp0A19_i2c_write(0x5b,0xe0); //40//raw_rb_fac_normal
  sp0A19_i2c_write(0x5c,0x80); 	 //raw_rb_fac_dummy
  sp0A19_i2c_write(0x5d,0x00); 	 //raw_rb_fac_lowlight
  
//sharpen 
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0xe2,0x30);	//sharpen_y_base
  sp0A19_i2c_write(0xe4,0xa0);	//sharpen_y_max
  
  sp0A19_i2c_write(0xe5,0x04); //rangek_neg_outdoor
  sp0A19_i2c_write(0xd3,0x04); //rangek_pos_outdoor   
  sp0A19_i2c_write(0xd7,0x04); //range_base_outdoor   
  
  sp0A19_i2c_write(0xe6,0x04); //rangek_neg_normal
  sp0A19_i2c_write(0xd4,0x04); //rangek_pos_normal 
  sp0A19_i2c_write(0xd8,0x04); //range_base_normal  
    
  sp0A19_i2c_write(0xe7,0x08); //rangek_neg_dummy
  sp0A19_i2c_write(0xd5,0x08); //rangek_pos_dummy
  sp0A19_i2c_write(0xd9,0x08); //range_base_dummy  	    
  sp0A19_i2c_write(0xd2,0x10); //rangek_neg_lowlight
  sp0A19_i2c_write(0xd6,0x10); //rangek_pos_lowlight
  sp0A19_i2c_write(0xda,0x10); //range_base_lowlight
  sp0A19_i2c_write(0xe8,0x20);//sharp_fac_pos_outdoor
  sp0A19_i2c_write(0xec,0x35);//sharp_fac_neg_outdoor
  sp0A19_i2c_write(0xe9,0x20);//sharp_fac_pos_nr
  sp0A19_i2c_write(0xed,0x35);//sharp_fac_neg_nr
  sp0A19_i2c_write(0xea,0x20);//sharp_fac_pos_dummy
  sp0A19_i2c_write(0xef,0x30);//sharp_fac_neg_dummy
  sp0A19_i2c_write(0xeb,0x10);//sharp_fac_pos_low
  sp0A19_i2c_write(0xf0,0x20);//sharp_fac_neg_low 
  
//CCM
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0xa0,0x80);//80(ɫӽɫ)
  sp0A19_i2c_write(0xa1,0x0 );//0 
  sp0A19_i2c_write(0xa2,0x0 );//0 
  sp0A19_i2c_write(0xa3,0xf3);//f0
  sp0A19_i2c_write(0xa4,0x8e);//a6
  sp0A19_i2c_write(0xa5,0x0 );//ea
  sp0A19_i2c_write(0xa6,0x0 );//0 
  sp0A19_i2c_write(0xa7,0xe6);//e6
  sp0A19_i2c_write(0xa8,0x9a);//9a
  sp0A19_i2c_write(0xa9,0x0 );//0 
  sp0A19_i2c_write(0xaa,0x3 );//33
  sp0A19_i2c_write(0xab,0xc );//c 
  sp0A19_i2c_write(0xfd,0x00);
  
//gamma  
  
 sp0A19_i2c_write(0xfd,0x00);
 sp0A19_i2c_write(0x8b,0x0 );//0 
 sp0A19_i2c_write(0x8c,0xC );//11
 sp0A19_i2c_write(0x8d,0x19);//19 
 sp0A19_i2c_write(0x8e,0x2C);//28 
 sp0A19_i2c_write(0x8f,0x49);//46 
 sp0A19_i2c_write(0x90,0x61);//61 
 sp0A19_i2c_write(0x91,0x77);//78 
 sp0A19_i2c_write(0x92,0x8A);//8A 
 sp0A19_i2c_write(0x93,0x9B);//9B 
 sp0A19_i2c_write(0x94,0xA9);//A9 
 sp0A19_i2c_write(0x95,0xB5);//B5 
 sp0A19_i2c_write(0x96,0xC0);//C0 
 sp0A19_i2c_write(0x97,0xCA);//CA 
 sp0A19_i2c_write(0x98,0xD4);//D4 
 sp0A19_i2c_write(0x99,0xDD);//DD 
 sp0A19_i2c_write(0x9a,0xE6);//E6 
 sp0A19_i2c_write(0x9b,0xEF);//EF 
 sp0A19_i2c_write(0xfd,0x01);//01 
 sp0A19_i2c_write(0x8d,0xF7);//F7 
 sp0A19_i2c_write(0x8e,0xFF);//FF 
             
//rpc
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0xe0,0x4c);
  sp0A19_i2c_write(0xe1,0x3c);
  sp0A19_i2c_write(0xe2,0x34);
  sp0A19_i2c_write(0xe3,0x2e);
  sp0A19_i2c_write(0xe4,0x2e);
  sp0A19_i2c_write(0xe5,0x2c);
  sp0A19_i2c_write(0xe6,0x2c);
  sp0A19_i2c_write(0xe8,0x2a);
  sp0A19_i2c_write(0xe9,0x2a);
  sp0A19_i2c_write(0xea,0x2a);
  sp0A19_i2c_write(0xeb,0x28);
  sp0A19_i2c_write(0xf5,0x28);
  sp0A19_i2c_write(0xf6,0x28);
//ae min gain  
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x94,0xa0);//rpc_max_indr
  sp0A19_i2c_write(0x95,0x28);//1e//rpc_min_indr 
  sp0A19_i2c_write(0x9c,0xa0);//rpc_max_outdr
  sp0A19_i2c_write(0x9d,0x28);//rpc_min_outdr    

//ae target
  sp0A19_i2c_write(0xfd,0x00); 
  sp0A19_i2c_write(0xed,0x8c);//80 
  sp0A19_i2c_write(0xf7,0x88);//7c 
  sp0A19_i2c_write(0xf8,0x80);//70 
  sp0A19_i2c_write(0xec,0x7c);//6c  
  sp0A19_i2c_write(0xef,0x74);//99
  sp0A19_i2c_write(0xf9,0x70);//90
  sp0A19_i2c_write(0xfa,0x68);//80
  sp0A19_i2c_write(0xee,0x64);//78    
//gray detect
  sp0A19_i2c_write(0xfd,0x01);
  sp0A19_i2c_write(0x30,0x40);
  sp0A19_i2c_write(0x31,0x70);//
  sp0A19_i2c_write(0x32,0x40);//80
  sp0A19_i2c_write(0x33,0xef);//
  sp0A19_i2c_write(0x34,0x05);//04
  sp0A19_i2c_write(0x4d,0x2f);//40
  sp0A19_i2c_write(0x4e,0x20);//
  sp0A19_i2c_write(0x4f,0x16);//13      
  //lowlight lum
  sp0A19_i2c_write(0xfd,0x00); 
  sp0A19_i2c_write(0xb2,0x20);//lum_limit
  sp0A19_i2c_write(0xb3,0x1f);//lum_set
  sp0A19_i2c_write(0xb4,0x30);//black_vt
  sp0A19_i2c_write(0xb5,0x45);//white_vt
//saturation
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0xbe,0xff);
  sp0A19_i2c_write(0xbf,0x01); 
  sp0A19_i2c_write(0xc0,0xff);
  sp0A19_i2c_write(0xc1,0xd8);
  sp0A19_i2c_write(0xd3,0x88);//0x78
  sp0A19_i2c_write(0xd4,0x88);//0x78
  sp0A19_i2c_write(0xd6,0x80);//0x78      
  sp0A19_i2c_write(0xd7,0x60);//0x78
//HEQ
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0xdc,0x00);
  sp0A19_i2c_write(0xdd,0x80);
  sp0A19_i2c_write(0xde,0xa8);//80
  sp0A19_i2c_write(0xdf,0x80);    
//func enable
  sp0A19_i2c_write(0xfd,0x00);
  sp0A19_i2c_write(0x31,0x00);//10
  sp0A19_i2c_write(0x32,0x15);//d
  sp0A19_i2c_write(0x34,0x76);//16
  sp0A19_i2c_write(0x33,0xef);
  sp0A19_i2c_write(0x5f,0x51);     

}

void SP0A19_GAMMA_Select(uint32 GammaLvl)
{
	
}

void SP0A19_H_V_Switch(uint8 direction)
{
	 
}

void SP0A19_Write_More_Registers(void)
{
}

/*===========================================================================

FUNCTION      CAMSENSOR_SP0A19_INIT

DESCRIPTION
              Initialize the camera sensor;

DEPENDENCIES
  None

RETURN VALUE
  TRUE  - Successful
  FALSE - Failure

SIDE EFFECTS
  None

===========================================================================*/

boolean camsensor_sp0A19_init(camsensor_function_table_type *camsensor_function_table_ptr,
                                     camctrl_tbl_type              *camctrl_tbl_ptr)
{
  boolean blRes = TRUE;
  uint8 product_id = 0;
	/* 1. Initiate I2C: */
  camsensor_i2c_command.bus_id	   = I2C_BUS_HW_CTRL;
  camsensor_i2c_command.slave_addr = CAMSENSOR_SP0A19_I2C_SLAVE_ID;
  camsensor_i2c_command.options    = I2C_REG_DEV;

  //pull power down pin to low
  #ifdef FEATURE_CAMERA_MULTI_SENSOR
  if(camsensor_get_active_camera() == CAMERA_MAIN)
  	{
	  gpio_out(MAIN_CAM_POWER_CONTROL, 0);
	  //gpio_out(SUB_CAM_POWER_CONTROL, 0);
  	}
  else
  	{
  	 // gpio_out(MAIN_CAM_POWER_CONTROL, 1);
	  //gpio_out(SUB_CAM_POWER_CONTROL, 0);
	  camsensor_sp0A19_power_down();
	  return FALSE;
  	}
  #else
  	gpio_out(MAIN_CAM_POWER_CONTROL, 0); //active sensor
  #endif
  //-------------enable master control clk
  camsensor_camclk_po_hz = 24000000;/*24576000*/
  (void)camsensor_config_camclk_po(camsensor_camclk_po_hz);  
  camera_timed_wait(10);
 
  camsensor_preview_resolution	= CAMSENSOR_QTR_SIZE;		//CAMSENSOR_FULL_SIZE;
  camsensor_snapshot_resolution = CAMSENSOR_FULL_SIZE;
  camsensor_current_resolution	= CAMSENSOR_INVALID_SIZE;

  blRes = sp0A19_i2c_read(0x02, &product_id);
  MSG_HIGH ("[SP0A19] product_id=0x%x (%d)",product_id,blRes,0);	  

  if (product_id != SP0A19_SENSOR_ID)
  {
  	camsensor_sp0A19_power_down();
    MSG_HIGH("[SP0A19] ID is not match (0x%x)",SP0A19_SENSOR_ID,0,0);
    return FALSE;
  }

  /* Always initialize the sensor with preview resolution */
  if (camsensor_sp0A19_sensor_init() == FALSE) 
  {
	 MSG_HIGH ("[SP0A19] camsensor_sp0A19_sensor_init Failed",0,0,0);	 
	 return FALSE;
  }
  else
  {
	camsensor_current_resolution = camsensor_preview_resolution;
  }
 
  /* Initialize the sensor_function_table */
  camsensor_sp0A19_register(camsensor_function_table_ptr);
  //camsensor_name[1] = camsensor_sp0A19_sensor_name;	
  MSG_HIGH ("[SP0A19] camsensor_SP0A19_ycbcr_init END",0,0,0);

  return TRUE;
} /* camsensor_sp0A19_init */

/*===========================================================================
FUNCTION      CAMSENSOR_SP0A19_INIT_CAMSENSOR_INFO

DESCRIPTION
              Initialize the camera sensor info;

DEPENDENCIES
  msm_id must be set to a vaild id before calling this function.

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/

LOCAL void camsensor_sp0A19_init_camsensor_info()
{
   camsensor_info.pclk_invert = TRUE; 
   camsensor_info.snapshot_width        = 640; 
   camsensor_info.snapshot_height       = 480;
   camsensor_info.snapshot_dummy_pixels =  0;
   camsensor_info.snapshot_dummy_lines  =  0;

   camsensor_info.preview_width         = 640;
   camsensor_info.preview_height        = 480;
   camsensor_info.preview_dummy_pixels  = 0;
   camsensor_info.preview_dummy_lines   = 0;   
 
}
/*===========================================================================

FUNCTION      CAMSENSOR_SP0A19_START

DESCRIPTION
              Initialize the camsensor parameters.

DEPENDENCIES
  None

RETURN VALUE
  if successful
    TRUE
  else
    FALSE

SIDE EFFECTS
  None

===========================================================================*/
boolean camsensor_sp0A19_start( camsensor_static_params_type *camsensor_params)
{
	MSG_HIGH ("[SP0A19] camsensor_SP0A19_ycbcr_start Enter",0,0,0);
	
	camsensor_sp0A19_init_camsensor_info();

  /* Initialize CAMIF operation mode */
  camsensor_params->camif_config.SyncMode         = CAMIF_APS;
  camsensor_params->camif_config.HSyncEdge        = CAMIF_High;
  camsensor_params->camif_config.VSyncEdge        = CAMIF_High;//CAMIF_Low;
  camsensor_params->camif_efs_config.EFS_EOL      = 0x0000;
  camsensor_params->camif_efs_config.EFS_SOL      = 0x0000;
  camsensor_params->camif_efs_config.EFS_EOF      = 0x0000;
  camsensor_params->camif_efs_config.EFS_SOF      = 0x0000;

  /* ------------------  Sensor-specific Config -------------- */
  /* Make/model of sensor */
  camsensor_params->sensor_model = CAMSENSOR_SUPERPIX_SP0A19;

  /* CCD or CMOS */
  camsensor_params->sensor_type = CAMSENSOR_CMOS;
  
  camsensor_params->format = CAMIF_YCbCr_Cr_Y_Cb_Y;

  /* BAYER or YCbCr */
  camsensor_params->output_format = CAMSENSOR_YCBCR;
  
  /* A pointer to the sensor name for EXIF tags                */
  camsensor_params->sensor_name = camsensor_sp0A19_sensor_name;

  /* Pointer to sensor_suffix */
  camsensor_params->sensor_suffix=camsensor_sp0A19_sensor_suffix;

  camsensor_params->ignore_camif_error = TRUE;


  /* What is the maximum FPS that can be supported by this sensor
     in video mode */
  camsensor_params->max_video_fps = 15 * Q8;

  /* Application assigned FPS in video mode */
  camsensor_params->video_fps = 15 * Q8;

  /* Snapshot mode operation */
  camsensor_params->max_preview_fps = 15 * Q8;
  camsensor_params->nightshot_fps = 15 * Q8/2;

  /* May be assigned with max_preview_fps or nightshot_fps. */
  camsensor_params->preview_fps = 15 * Q8;

  /* ------------  Auto Exposure Control Config -------------- */
  camsensor_params->aec_enable = FALSE;
  
  /* ------------  Auto White Balance Config ----------------- */
  /* AWB -Auto White Balance Parameters */
  camsensor_params->awb_enable = FALSE;
  
  /* ------------  Auto Focus Config ------------------------- */
  /* AF -Auto Focus Parameters */
  camsensor_params->af_enable = FALSE;


  /* Snapshot dimensions */
  camsensor_params->full_size_width  = 640;
  camsensor_params->full_size_height = 480;
  
  /* Preview dimensions */
  camsensor_params->qtr_size_width	= 640;
  camsensor_params->qtr_size_height = 480;

  MSG_HIGH ("[SP0A19] camsensor_SP0A19_ycbcr_start camsensor_preview_resolution=%d",camsensor_preview_resolution,0,0);
  
  switch (camsensor_preview_resolution)
  {
  
  
	  case CAMSENSOR_QTR_SIZE:
  
		  camsensor_params->preview_dx_decimation = \
						camsensor_params->full_size_width * Q12 / \
					  camsensor_params->qtr_size_width;
		  camsensor_params->preview_dy_decimation = \
					  camsensor_params->full_size_height * Q12 / \
					  camsensor_params->qtr_size_height;
  
		  camsensor_params->camsensor_width  = \
					  camsensor_params->qtr_size_width;
		  camsensor_params->camsensor_height = \
					  camsensor_params->qtr_size_height;
  
		  break;
  
	  case CAMSENSOR_FULL_SIZE:
  
		  camsensor_params->preview_dx_decimation = Q12;
		  camsensor_params->preview_dy_decimation = Q12;
		  camsensor_params->camsensor_width  = \
					  camsensor_params->full_size_width;
		  camsensor_params->camsensor_height = \
					  camsensor_params->full_size_height;
  
		  break;
  
	  default:
  
		  MSG_ERROR("[SP0A19] Unrecognized preview resolution, assuming full size",0,0,0);
  
		  camsensor_params->camsensor_width  = \
					  camsensor_params->full_size_width;
		  camsensor_params->camsensor_height = \
					  camsensor_params->full_size_height;
  
		   break;
	}	

  MSG_HIGH ("[SP0A19] camsensor_SP0A19_ycbcr_start end",0,0,0);

  return TRUE;
} /* camsensor_sp0A19_start */


/*===========================================================================

FUNCTION      CAMSENSOR_SP0A19_VIDEO_CONFIG

DESCRIPTION
              Configure the camera sensor for preview 

DEPENDENCIES
  None

RETURN VALUE
  TRUE if successful
  FALSE otherwise

SIDE EFFECTS
  None

===========================================================================*/
boolean camsensor_sp0A19_video_config
(
  camsensor_static_params_type *camsensor_params /* Other config params */
)
{
	MSG_HIGH ("[SP0A19] camsensor_SP0A19_ycbcr_video_config start(%d)",camsensor_preview_resolution,0,0);
  /* Sensor output data format */
  camsensor_params->format = CAMIF_YCbCr_Cr_Y_Cb_Y;

  switch ( camsensor_preview_resolution ) {
	case CAMSENSOR_QTR_SIZE:
  
	  /* Set the current dimensions */
	  camsensor_params->camsensor_width  = \
					camsensor_params->qtr_size_width;
	  camsensor_params->camsensor_height = \
					camsensor_params->qtr_size_height;
  
	  /* CAMIF frame */
	  camsensor_params->camif_frame_config.pixelsPerLine = \
					camsensor_params->qtr_size_width*2;
  
	  camsensor_params->camif_frame_config.linesPerFrame = \
					camsensor_params->qtr_size_height;
  
	  /* CAMIF Window */
	  camsensor_params->camif_window_width_config.firstPixel = \
						 camsensor_info.preview_dummy_pixels*2;
  
	  camsensor_params->camif_window_width_config.lastPixel  = \
				  (camsensor_params->camif_window_width_config.firstPixel) \
			  +(camsensor_params->qtr_size_width*2)-1;
  
	  camsensor_params->camif_window_height_config.firstLine = camsensor_info.preview_dummy_lines;
  
	  camsensor_params->camif_window_height_config.lastLine  =camsensor_info.preview_dummy_lines \
					+ camsensor_params->qtr_size_height-1;
	  break;
  
  
	case CAMSENSOR_FULL_SIZE:
	  /* Set the current dimensions */
	  camsensor_params->camsensor_width  = \
					camsensor_params->full_size_width;
	  camsensor_params->camsensor_height = \
					camsensor_params->full_size_height;
  
	  /* CAMIF frame */
	  camsensor_params->camif_frame_config.pixelsPerLine = \
					camsensor_params->full_size_width*2;
  
	  camsensor_params->camif_frame_config.linesPerFrame = \
					camsensor_params->full_size_height;
  
	  /* CAMIF window */
	  camsensor_params->camif_window_width_config.firstPixel = 0;
  
	  camsensor_params->camif_window_width_config.lastPixel  = \
				   (camsensor_params->full_size_width*2)-1;
  
	  camsensor_params->camif_window_height_config.firstLine = 0;
  
	  camsensor_params->camif_window_height_config.lastLine  = \
				   camsensor_params->full_size_height-1;
  
	  break;
  
	case CAMSENSOR_INVALID_SIZE:
  
	   return FALSE;
  
  }
  //camsensor_dynamic_params.preview_pixelsPerLine  = 640;
  //camsensor_dynamic_params.preview_linesPerFrame  = 480;
  
  camsensor_params->epoch_line = 10000;
  
  camsensor_current_resolution = camsensor_preview_resolution;
  
  MSG_HIGH("[SP0A19] [video_config]: full_size_width %d, full_size_height %d", camsensor_params->full_size_width, camsensor_params->full_size_height, 0);
  MSG_HIGH("[SP0A19] [video_config]: camsensor_width %d, camsensor_height %d", camsensor_params->camsensor_width, camsensor_params->camsensor_height, 0);
  MSG_HIGH("[SP0A19] [video_config]: pixelsPerLine %d, linesPerFrame %d", camsensor_params->camif_frame_config.pixelsPerLine, camsensor_params->camif_frame_config.linesPerFrame, 0);
  MSG_HIGH("[SP0A19] [video_config]: w.firstPixel %d, w.lastPixel %d", camsensor_params->camif_window_width_config.firstPixel, camsensor_params->camif_window_width_config.lastPixel, 0);
  MSG_HIGH("[SP0A19] [video_config]: h.firstLine %d, h.lastLine %d", camsensor_params->camif_window_height_config.firstLine, camsensor_params->camif_window_height_config.lastLine, 0);
  MSG_HIGH("[SP0A19] camsensor_SP0A19_ycbcr_video_config end.",0,0,0);
  return TRUE;
} /* camsensor_sp0A19_video_config */

 
/*===========================================================================

FUNCTION      CAMSENSOR_SP0A19_POWER_UP

DESCRIPTION
              Power up the sensor.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/

void camsensor_sp0A19_power_up(void)
{
  // no code here
  MSG_HIGH("[SP0A19] camsensor_sp0A19_power_up",0,0,0);
} /* camsensor_sp0A19_power_up */


/*===========================================================================

FUNCTION      camsensor_sp0A19_POWER_DOWN

DESCRIPTION
              Power down the sensor, using I2C driver.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void camsensor_sp0A19_power_down(void)
{
#ifdef FEATURE_CAMERA_MULTI_SENSOR
	if(camsensor_get_active_camera() == CAMERA_MAIN)
		gpio_out(MAIN_CAM_POWER_CONTROL, 1);  
	else
	    gpio_out(SUB_CAM_POWER_CONTROL, 1);
#else
  gpio_out(MAIN_CAM_POWER_CONTROL, 1);
#endif

  MSG_HIGH("[SP0A19] camsensor_sp0A19_power_down",0,0,0);
} /* camsensor_sp0A19_power_down */


/*===========================================================================

FUNCTION      CAMSENSOR_SP0A19_WRITE_SENSOR

DESCRIPTION
    Write to sensor I2C register space.

DEPENDENCIES

RETURN VALUE
  TRUE  - Sensor I2C configuration successful.
  FALSE - Sensor I2C configuration failed.

SIDE EFFECTS
  None

===========================================================================*/
LOCAL boolean camsensor_sp0A19_sensor_init(void)
{
  MSG_HIGH ("[SP0A19] camsensor_SP0A19_init_sensor--begin(%d)",0,0,0);	

  SP0A19_Initial_Setting();
  //SP0A19_Write_More_Registers();
    
  return TRUE;
} /* camsensor_sp0A19_write_sensor */


/*===========================================================================
Function:	
  camsensor_sp0A19_snapshot_config

Describle:
  snapshot config

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

boolean camsensor_sp0A19_snapshot_config(camsensor_static_params_type *camsensor_params /* Other config params */)
{
  MSG_HIGH ("[SP0A19] camsensor_sp0A19_snapshot_config entry",0,0,0);

	/* Sensor output data format */
  camsensor_params->format = CAMIF_YCbCr_Cr_Y_Cb_Y;

  /* Discard the first frame? */
  camsensor_params->discardFirstFrame =  TRUE;

  /* 3x3 luma filter is always disabled, use ASF in encoding. */
  //camsensor_params->f3x3LumaFilter.enable = FALSE;

  camsensor_params->preview_dx_decimation = Q12;
  camsensor_params->preview_dy_decimation = Q12;

  /* CAMIF frame */
  camsensor_params->camif_frame_config.pixelsPerLine = camsensor_info.snapshot_width*2;
  camsensor_params->camif_frame_config.linesPerFrame = camsensor_info.snapshot_height;

 /* Configure the VFE_CAMIF window parameters. */
camsensor_params->camif_window_width_config.firstPixel = camsensor_info.snapshot_dummy_pixels*2;
camsensor_params->camif_window_width_config.lastPixel  = camsensor_params->camif_window_width_config.firstPixel +
 (camsensor_params->full_size_width*2)-1;

camsensor_params->camif_window_height_config.firstLine = camsensor_info.snapshot_dummy_lines;
camsensor_params->camif_window_height_config.lastLine  = camsensor_params->camif_window_height_config.firstLine +
 camsensor_params->full_size_height - 1;

  /* Set the current dimensions */
  camsensor_params->camsensor_width  = camsensor_params->full_size_width;
  camsensor_params->camsensor_height = camsensor_params->full_size_height;

#ifdef FEATURE_STROBE_FLASH
  camsensor_params->epoch_line = 1192;
#else
  /* Disable the Epoch Interrupt by setting more
   * number of lines than in a frame */
  camsensor_params->epoch_line = 10000;
#endif /* FEATURE_STROBE_FLASH */

  camsensor_current_resolution = camsensor_snapshot_resolution;

  return TRUE;
}

/*===========================================================================
Function:	
  camsensor_sp0A19_set_effect

Describle:
  set effect

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

camera_ret_code_type camsensor_sp0A19_set_effect(int8 effect)
{
	camera_ret_code_type ret_val = CAMERA_SUCCESS;
	MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_effect() effect = %d",effect,0,0);
	switch(effect)
	{
		case CAMERA_EFFECT_OFF://normal
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0x62, 0x00);
			sp0A19_i2c_write(0x63, 0x80);
			sp0A19_i2c_write(0x64, 0x80);
			break;
			
		case CAMERA_EFFECT_MONO:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0x62, 0x20);
			sp0A19_i2c_write(0x63, 0x80);
			sp0A19_i2c_write(0x64, 0x80);

			break;
			
		case CAMERA_EFFECT_NEGATIVE:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0x62, 0x04);
			sp0A19_i2c_write(0x63, 0x80);
			sp0A19_i2c_write(0x64, 0x80);
			break;
			
		case CAMERA_EFFECT_SEPIA:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0x62, 0x10);
			sp0A19_i2c_write(0x63, 0xc0);
			sp0A19_i2c_write(0x64, 0x20);
			break;

		default:
			ret_val = CAMERA_INVALID_PARM;
			break;
	}
	return ret_val;
}


/*===========================================================================
Function:	
  camsensor_sp0A19_raw_snapshot_config

Describle:
  raw snapshot config

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

boolean camsensor_sp0A19_raw_snapshot_config
(
	camsensor_static_params_type *camsensor_params /* Other config params */
)
{
	MSG_HIGH ("[SP0A19] camsensor_sp0A19_raw_snapshot_config entry",0,0,0);
	/* Sensor output data format */
  camsensor_params->format = CAMIF_YCbCr_Cr_Y_Cb_Y;

  /* Discard the first frame? */
  camsensor_params->discardFirstFrame =  FALSE;

  /* CAMIF frame */
  camsensor_params->camif_frame_config.pixelsPerLine = 640 * 2;
  camsensor_params->camif_frame_config.linesPerFrame = 480;

  /* CAMIF window */
  camsensor_params->camif_window_width_config.firstPixel = 0;
  camsensor_params->camif_window_width_config.lastPixel  = camsensor_params->camif_window_width_config.firstPixel + (camsensor_params->full_size_width * 2) - 1;
  camsensor_params->camif_window_height_config.firstLine = 0;
  camsensor_params->camif_window_height_config.lastLine = camsensor_params->camif_window_height_config.firstLine + camsensor_params->full_size_height - 1;

  /* Set the current dimensions camera_svcs_set_dimensions*/
  camsensor_params->camsensor_width  = camsensor_params->full_size_width;
  camsensor_params->camsensor_height = camsensor_params->full_size_height;

#ifdef FEATURE_STROBE_FLASH
  camsensor_params->epoch_line = 1192;
#else
  /* Disable the Epoch Interrupt by setting more
   * number of lines than in a frame */
  camsensor_params->epoch_line = 10000;
#endif /* FEATURE_STROBE_FLASH */

  camsensor_current_resolution = camsensor_snapshot_resolution;

  camsensor_dynamic_params.preview_pixelsPerLine  = 640;
  camsensor_dynamic_params.preview_linesPerFrame	= 480;


  return TRUE;
}


/*===========================================================================
Function:	
  camsensor_sp0A19_set_nightmode

Describle:
  set_nightmode

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

camera_ret_code_type  camsensor_sp0A19_set_nightmode(int8 nightmode)
{
  uint8 data = 0;
  camera_ret_code_type ret_val = CAMERA_SUCCESS;
  MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_nightmode nightmode = %d",nightmode,0,0);

  
  switch(nightmode)
  {
    case CAMERA_NIGHTSHOT_MODE_ON:
		sp0A19_i2c_write(0xfd,0x00);
		sp0A19_i2c_write(0xb2,0x25);
		sp0A19_i2c_write(0xb3,0x1f);
      break;
    case CAMERA_NIGHTSHOT_MODE_OFF:
		sp0A19_i2c_write(0xfd,0x00);
		sp0A19_i2c_write(0xb2,0x20);
		sp0A19_i2c_write(0xb3,0x1f);

      break;
    default:
      ret_val = CAMERA_INVALID_PARM;
    break;
  }
  return ret_val;
}

/*===========================================================================
Function:	
  camsensor_sp0A19_set_wb

Describle:
  wb

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

camera_ret_code_type camsensor_sp0A19_set_wb(int8 wb)
{
	uint8 data = 0;
	camera_ret_code_type ret_val = CAMERA_SUCCESS;

	MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_wb wb = %d",wb,0,0);

	
	switch(wb)
	{
		case  CAMERA_WB_AUTO://Զ Auto
			//SP0A19_reg_WB_auto         Զ
			sp0A19_i2c_write(0xfd,0x01);                                                          
			sp0A19_i2c_write(0x28,0xc5);		                                                       
			sp0A19_i2c_write(0x29,0x9b);
			sp0A19_i2c_write(0xfd,0x00);  // AUTO 3000K~7000K   	  
			sp0A19_i2c_write(0x32,0x15);      			
			break;
		case CAMERA_WB_CLOUDY_DAYLIGHT:// Cloudy
			// SP0A19_reg_WB_auto   
			sp0A19_i2c_write(0xfd,0x00);   //7000K                                     
			sp0A19_i2c_write(0x32,0x05);                                                          
			sp0A19_i2c_write(0xfd,0x01);                                                          
			sp0A19_i2c_write(0x28,0xbf);		                                                       
			sp0A19_i2c_write(0x29,0x89);		                                                       
			sp0A19_i2c_write(0xfd,0x00);                                             
			break;
		case CAMERA_WB_INCANDESCENT://׳ Home
			// SP0A19_reg_WB_auto ׳ 
			sp0A19_i2c_write(0xfd,0x00);  //2800K~3000K                                     
			sp0A19_i2c_write(0x32,0x05);                                                          
			sp0A19_i2c_write(0xfd,0x01);                                                          
			sp0A19_i2c_write(0x28,0x89);		                                                       
			sp0A19_i2c_write(0x29,0xb8);		                                                       
			sp0A19_i2c_write(0xfd,0x00);                                                          
			break;
		case CAMERA_WB_FLUORESCENT: //ӫ Office
			//SP0A19_reg_WB_auto  ӫ 
			sp0A19_i2c_write(0xfd,0x00);  //4200K~5000K                                     
			sp0A19_i2c_write(0x32,0x05);                                                          
			sp0A19_i2c_write(0xfd,0x01);                                                          
			sp0A19_i2c_write(0x28,0xaf);		                                                       
			sp0A19_i2c_write(0x29,0x99);		                                                       
			sp0A19_i2c_write(0xfd,0x00);                                                         
			break;
		case CAMERA_WB_DAYLIGHT:  //չ Sunny
			// SP0A19_reg_WB_auto   
			sp0A19_i2c_write(0xfd,0x00);  //6500K                                     
			sp0A19_i2c_write(0x32,0x05);                                                          
			sp0A19_i2c_write(0xfd,0x01);                                                          
			sp0A19_i2c_write(0x28,0xbc);		                                                       
			sp0A19_i2c_write(0x29,0x5d);		                                                       
			sp0A19_i2c_write(0xfd,0x00);                                                         
			break;
		default:
			ret_val = CAMERA_INVALID_PARM;
			break;			
	}	
	return ret_val;
}
/*===========================================================================
Function:	
  camsensor_sp0A19_set_contrast

Describle:
  contrast

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None 

===========================================================================*/

camera_ret_code_type camsensor_sp0A19_set_contrast(int8 contrast)
{
	camera_ret_code_type ret_val = CAMERA_SUCCESS;
	MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_contrast contrast = %d",contrast,0,0);
	switch(contrast)
	{
		case LEVEL_POSITIVE_4://+4
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde+0x40);
			break;
		case LEVEL_POSITIVE_3://+3
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde+0x30);
			break;
		case LEVEL_POSITIVE_2://+2
		 	sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde+0x20);
			break;
		case LEVEL_POSITIVE_1://+1
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde+0x10);
			break;
		case LEVEL_POSITIVE_0://0
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde); // 0
			break;
		case LEVEL_NEGTIVE_1://-1
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde-0x10);
			break;
		case LEVEL_NEGTIVE_2://-2
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde-0x20);
			break;
		case LEVEL_NEGTIVE_3://-3
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde-0x30);
			break;
		case LEVEL_NEGTIVE_4://-4
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xde, SP0A19_P0_0xde-0x40);
			break;
		default:
			ret_val = CAMERA_INVALID_PARM;
			break;
	}
	return ret_val;
}



/*===========================================================================
Function:	
  camsensor_sp0A19_set_antibanding

Describle:
  antibanding

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

camera_ret_code_type camsensor_sp0A19_set_antibanding(int8 antibanding)
{
	camera_ret_code_type ret_val = CAMERA_SUCCESS;

	MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_antibanding entry (%d)",antibanding,0,0);

	switch(antibanding)
	{
		case CAMERA_ANTIBANDING_50HZ:
		//capture preview daylight 24M 50hz 20-8FPS maxgain  
		  sp0A19_i2c_write(0xfd,0x00);
		  sp0A19_i2c_write(0x03,0x01);
		  sp0A19_i2c_write(0x04,0x32);
		  sp0A19_i2c_write(0x06,0x00);
		  sp0A19_i2c_write(0x09,0x01);
		  sp0A19_i2c_write(0x0a,0x46);
		  sp0A19_i2c_write(0xf0,0x66);
		  sp0A19_i2c_write(0xf1,0x00);
		  sp0A19_i2c_write(0xfd,0x01);
		  sp0A19_i2c_write(0x90,0x0c);
		  sp0A19_i2c_write(0x92,0x01);
		  sp0A19_i2c_write(0x98,0x66);
		  sp0A19_i2c_write(0x99,0x00);
		  sp0A19_i2c_write(0x9a,0x01);
		  sp0A19_i2c_write(0x9b,0x00);
		  
		  sp0A19_i2c_write(0xfd,0x01);
		  sp0A19_i2c_write(0xce,0xc8);
		  sp0A19_i2c_write(0xcf,0x04);
		  sp0A19_i2c_write(0xd0,0xc8);
		  sp0A19_i2c_write(0xd1,0x04);
		  sp0A19_i2c_write(0xfd,0x00); 

			blIs60Hz = FALSE;
			break;
			
		case CAMERA_ANTIBANDING_60HZ:
		//capture preview daylight 24M 60hz 20-8FPS maxgain  
		  sp0A19_i2c_write(0xfd,0x00);
		  sp0A19_i2c_write(0x03,0x00);
		  sp0A19_i2c_write(0x04,0xff);
		  sp0A19_i2c_write(0x06,0x00);
		  sp0A19_i2c_write(0x09,0x01);
		  sp0A19_i2c_write(0x0a,0x46);
		  sp0A19_i2c_write(0xf0,0x55);
		  sp0A19_i2c_write(0xf1,0x00);
		  sp0A19_i2c_write(0xfd,0x01);
		  sp0A19_i2c_write(0x90,0x0f);
		  sp0A19_i2c_write(0x92,0x01);
		  sp0A19_i2c_write(0x98,0x55);
		  sp0A19_i2c_write(0x99,0x00);
		  sp0A19_i2c_write(0x9a,0x01);
		  sp0A19_i2c_write(0x9b,0x00);
		  
		  sp0A19_i2c_write(0xfd,0x01);
		  sp0A19_i2c_write(0xce,0xfb);
		  sp0A19_i2c_write(0xcf,0x04);
		  sp0A19_i2c_write(0xd0,0xfb);
		  sp0A19_i2c_write(0xd1,0x04);
		  sp0A19_i2c_write(0xfd,0x00); 			
					blIs60Hz = TRUE;
			break;

		default:
			ret_val = CAMERA_INVALID_PARM;
			break;			
	}
	return ret_val;
}

/*===========================================================================
Function:	
  camsensor_sp0A19_set_brightness

Describle:
  set brightness

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/
camera_ret_code_type camsensor_sp0A19_set_brightness(int8 value)
{
	camera_ret_code_type ret_val = CAMERA_SUCCESS;

	MSG_HIGH ("[SP0A19] camsensor_sp0A19_set_brightness value = %d",value,0,0);
//#ifdef MMI_PROJECT_E550_2
    switch (value)
    {
            case BRIGHTNESS_NEGTIVE_4:     
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0xc0);
                


			break;		
		case BRIGHTNESS_NEGTIVE_3:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0xd0);
			break;		
		case BRIGHTNESS_NEGTIVE_2:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0xe0);
			break;				
		case BRIGHTNESS_NEGTIVE_1:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0xf0);
			break;				
		case BRIGHTNESS_POSITIVE_0:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0x00);

			break;				
		case BRIGHTNESS_POSITIVE_1:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0x10);
			break;				
		case BRIGHTNESS_POSITIVE_2:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0x20);
			break;				
		case BRIGHTNESS_POSITIVE_3:
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0x30);
			break;				
		case BRIGHTNESS_POSITIVE_4:	
			sp0A19_i2c_write(0xfd, 0x00);
			sp0A19_i2c_write(0xdc, 0x40);
			break;
		default:
			ret_val = CAMERA_INVALID_PARM;
			break;
	}

	return ret_val;
}

/*===========================================================================
Function:	
  camsensor_sp0A19_move_focus

Describle:
  focus

Dependncies 
  Nightmode

Return value:
  CAMERA_SUCCESS or  CAMERA_NOT_SUPPORTED

Side effects:
  None

===========================================================================*/

camera_ret_code_type  camsensor_sp0A19_move_focus(camsensor_move_focus_enum_type direction, int32 num_steps)
{
	return(CAMERA_NOT_SUPPORTED);
}

/*============================================================================
						  INTERNAL API DEFINITIONS
============================================================================*/

/*===========================================================================

FUNCTION   CAMSENSOR_SP0A19_REGISTER 

DESCRIPTION  
  Exports the sensor specific functions to CAMSENSOR

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
LOCAL void camsensor_sp0A19_register (camsensor_function_table_type *camsensor_function_table_ptr)
{
  camsensor_function_table_ptr->camsensor_start           = camsensor_sp0A19_start;
  camsensor_function_table_ptr->camsensor_video_config    = camsensor_sp0A19_video_config;
  camsensor_function_table_ptr->camsensor_power_up        = camsensor_sp0A19_power_up;
  camsensor_function_table_ptr->camsensor_power_down      = camsensor_sp0A19_power_down;
  camsensor_function_table_ptr->camsensor_snapshot_config = camsensor_sp0A19_snapshot_config;
  camsensor_function_table_ptr->camsensor_set_brightness  = camsensor_sp0A19_set_brightness;
  camsensor_function_table_ptr->camsensor_set_contrast    = camsensor_sp0A19_set_contrast;
  camsensor_function_table_ptr->camsensor_set_wb          = camsensor_sp0A19_set_wb;
  camsensor_function_table_ptr->camsensor_set_antibanding = camsensor_sp0A19_set_antibanding;
  camsensor_function_table_ptr->camsensor_raw_snapshot_config = camsensor_sp0A19_raw_snapshot_config;
  camsensor_function_table_ptr->camsensor_set_effect      = camsensor_sp0A19_set_effect;
  camsensor_function_table_ptr->camsensor_write			  = sp0A19_i2c_write;
  camsensor_function_table_ptr->camsensor_read			  =	sp0A19_i2c_read;
} /* camsensor_sp0A19 */

    


#endif /* FEATURE_CAMERA,USE_CAMSENSOR_OMNI_VISION_7675 */


  

