/****************************************************************
*  COPYRIGHT Leadcore Technology Co.,Ltd     *
*****************************************************************/
/*****************************************************************
 * FileName:    sensor_SP0A19.c
 *
 * Version:     
 *
 * Purpose:      
 *
 * Authors:      
 *
 * Notes:       
 ******************************************************************/
 /*****************************************************************
 *                      HISTORY
 *-----------------------------------------------------------------
 *    when        who        where,what,why
 *-----------------------------------------------------------------
 * 2008.02.03    zouqiao     Create.
 * 2008.11.12    zouqiao     Modify for DTT6C03C.   
 * 2009.03.28    zouqiao     Fix bug Arena_Bug00001194.
 * L2000 Enh00000205 by ouyanghua on 09-11-26 : bottom current optimization
 * 2010.06.21    zengjun     L2000 Req00000387:add brightness,contrast support for SP0A19.
 * 2010.07.19    zengjun     L2000 Req00000418: add special effect for SP0A19.
 * 2011.04.21    liuzhen     L1710 Enh00000039  for SP0A19 acclimatize-self
 ******************************************************************/
#include "custom_peripheral.h"

#if defined(DD_SENSOR_IC_SP0A19)

#include "sensor.h"
#include "sensor_SP0A19_c.h"
#include "comip_gpio.h"
#include "comip_gpio_map.h"
#include "comip_ci.h"
#include "i2cpub.h"

#include "dd_dbg.h"

//-------------------------------------------------------------------------------------------------------
//     API Functions
//-------------------------------------------------------------------------------------------------------
/*****************************************************************************
* Function: SP0A19_set_reset_gpio_level
*
* Purpose: Set reset gpio level.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        ucLevel           unsigned char               In             GPIO level value. 1 : high; 0 : low.
*
* Return: None.
*
* Note: None.
*******************************************************************************/
static void SP0A19_set_reset_gpio_level(unsigned char ucLevel)
{
    if(ucLevel)
    {
        comip_gpio_set_level(gPinReset, GPIO_LEVEL_HIGH);
    }
    else
    {
        comip_gpio_set_level(gPinReset, GPIO_LEVEL_LOW);
    }
}

/*****************************************************************************
* Function: SP0A19_set_pwdn_gpio_level
*
* Purpose: Set pwdn gpio level.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        ucLevel           unsigned char               In             GPIO level value. 1 : high; 0 : low.
*
* Return: None.
*
* Note: None.
*******************************************************************************/
static void SP0A19_set_pwdn_gpio_level(unsigned char ucLevel)
{
    if(ucLevel)
    {
        comip_gpio_set_level(gPinPwdn, GPIO_LEVEL_HIGH);
    }
    else
    {
        comip_gpio_set_level(gPinPwdn, GPIO_LEVEL_LOW);
    }
}

/*****************************************************************************
* Function: SP0A19_poweron
*
* Purpose: Power on sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_poweron(void)
{  
    return 0;    
}

/*****************************************************************************
* Function: SP0A19_powerdown
*
* Purpose: Power down sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_powerdown(void)
{
    return 0;
}

/*****************************************************************************
* Function: SP0A19_reset
*
* Purpose: Reset sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_reset(void)
{  
    SP0A19_set_pwdn_gpio_level(0);
    
    SP0A19_set_reset_gpio_level(1);
    sensor_udelay(10);
    SP0A19_set_reset_gpio_level(0);
    sensor_udelay(10);
    SP0A19_set_reset_gpio_level(1);
    sensor_mdelay(10);
    
    return 0;    
}

/*****************************************************************************
* Function: SP0A19_standby
*
* Purpose: Standby sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_standby(void)
{
    SP0A19_set_pwdn_gpio_level(1);

    sensor_mdelay(10);

    return 0;
}

/*****************************************************************************
* Function: SP0A19_resume
*
* Purpose: Resume sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_resume(void)
{
    SP0A19_set_pwdn_gpio_level(0);
    
    return 0;
}

/*****************************************************************************
* Function: SP0A19_i2c_write_table
*
* Purpose: Write table data through I2C
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        pDataBuf        const unsigned char *       In          Data wirte buffer.
*        nDataLen        unsigned int                     In          Data wirte length.
*        nFrameLen      unsigned int                     In          One frame data length.
*
* Return: 
*        0   : Success.
*        -1 : Fail.
*
* Note: None.
*******************************************************************************/
static int SP0A19_i2c_write_table(unsigned char *pDataBuf, unsigned int nDataLen)
{
    unsigned int i;
    SENSOR_I2C_CMD cmd;

    cmd.nRegLen = SENSOR_I2C_CMD_ADDR_8;

    for(i = 0; i < nDataLen; i += SP0A19_ONE_DATA_LENGTH)
    {
        if (pDataBuf[i] == 0xFF)
        {
            tp_os_thread_sleep(pDataBuf[i+1]);
        }else
        {
            cmd.pReg     = &pDataBuf[i];
            cmd.pBuf     = &pDataBuf[i + 1];
            cmd.nBufLen  = SP0A19_ONE_DATA_LENGTH - 1;
            if(sensor_i2c_write(&cmd))
            {   
                return -1;
            }
        }
    }

    return 0;
}

/*****************************************************************************
* Function: SP0A19_sensor_init
*
* Purpose: Send initial code to sensor through I2C.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_sensor_init(void)
{
    return SP0A19_i2c_write_table((unsigned char *)icSP0A19Init, sizeof(icSP0A19Init));
}

/*****************************************************************************
* Function: SP0A19_init
*
* Purpose: Initial sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_init(GPIO_ID pinPwdn, GPIO_ID pinReset)
{
	/*record sensor control pins*/
	gPinPwdn = pinPwdn;
	gPinReset = pinReset;
    // RST --> High
    comip_gpio_set_mode(gPinReset, GPIO_MODE_GPIO);
    comip_gpio_set_direction(gPinReset, GPIO_DIR_OUT);
    comip_gpio_set_level(gPinReset, GPIO_LEVEL_HIGH);

    // PWDN --> High
    comip_gpio_set_mode(gPinPwdn, GPIO_MODE_GPIO);
    comip_gpio_set_direction(gPinPwdn, GPIO_DIR_OUT);
    comip_gpio_set_level(gPinPwdn, GPIO_LEVEL_HIGH);

    return 0;
}

/*****************************************************************************
* Function: SP0A19_deinit
*
* Purpose: Deinitial sensor.
*
* Relation: None.
* 
* Params: None.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_deinit(void)
{
    return 0;
}

/*****************************************************************************
* Function: SP0A19_set_state
*
* Purpose: Set sensor state.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eState            SENSOR_STATE               In          Sensor state.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_state(SENSOR_STATE eState)
{
    SINT32 nRet = 0;
    switch(eState)
    {
        case SENSOR_POWER_ON_INIT:
        case SENSOR_POWER_ON_STANDBY:
        case SENSOR_POWER_ON_INIT_STANDBY:
            /* Reset Sensor. */
            SP0A19_poweron();            
            break;
            
        case SENSOR_POWER_DOWN:
            SP0A19_powerdown();
            break;
            
        case SENSOR_RESUME:
        case SENSOR_RESUME_INIT:     
            SP0A19_resume();
            break;
                
        case SENSOR_STANDBY:
            SP0A19_standby();
            break;

        default:
            break;
    }
    DD_DBG(DD_LOG_CAMERA, "eState=%d\r\n",eState);
    if((eState == SENSOR_POWER_ON_INIT)
        || (eState == SENSOR_POWER_ON_INIT_STANDBY)
        || (eState == SENSOR_RESUME_INIT))
    {
        // Set CamIF
        camif_set_format(CAMIF_CAM_PIX_MOD_YUV422, CAMIF_CAM_ORDER_YUYV);

        // Enable camera clock.
        //_IOW32(CPR_CAMCLKCTL, 0xc);

        /* Reset sensor. */
        SP0A19_reset();
        
        /* Initial I2C. */
        sensor_i2c_init();

        /* Initial sensor. */
        nRet |= SP0A19_sensor_init();        
        
        
        sensor_mdelay(300);// here need to be delay 300 ms, for SP0A19 acclimatize-self
        
    }

    if((eState == SENSOR_POWER_ON_STANDBY)
        || (eState == SENSOR_POWER_ON_INIT_STANDBY))
    {
        nRet |= SP0A19_standby();
    }
    
    return nRet;
}

/*****************************************************************************
* Function: SP0A19_set_format
*
* Purpose: Set format.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*        eFormat         CAM_ENUM_CF                 In          Format.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_format(CAM_ENUM_CST eStreamType, CAM_ENUM_CF eFormat)
{
    return 0;
}

/*****************************************************************************
* Function: SP0A19_set_resolution
*
* Purpose: Set resolution.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*        eResolution     CAM_ENUM_CR                In          Resolution.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_resolution(CAM_ENUM_CST eStreamType, CAM_ENUM_CR eResolution)
{
    int nRet = 0;

    if(eResolution >= CAM_CR_VGA)
    {
        nRet = SP0A19_i2c_write_table((unsigned char *)icSP0A19Vga, sizeof(icSP0A19Vga));
    }
    else if(eResolution == CAM_CR_QVGA)
    {
        nRet = SP0A19_i2c_write_table((unsigned char *)icSP0A19Qvga, sizeof(icSP0A19Qvga));
    }
    else if(eResolution == CAM_CR_QCIF)
    {
        nRet = SP0A19_i2c_write_table((unsigned char *)icSP0A19Qcif, sizeof(icSP0A19Qcif));
    }

    return nRet;
}

/*****************************************************************************
* Function: SP0A19_start_stream
*
* Purpose: Start stream.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_start_stream(CAM_ENUM_CST eStreamType)
{  
    return 0;
}

/*****************************************************************************
* Function: SP0A19_stop_stream
*
* Purpose: Stop stream.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_stop_stream(CAM_ENUM_CST eStreamType)
{
    return 0;
}

/*****************************************************************************
* Function: SP0A19_set_brightness
*
* Purpose: set brightness.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        nValue              int                            In             brightness
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_brightness(int nValue, BOOL bAuto)
{
	SINT32 ret;

	if (bAuto)
		nValue = 2;

	switch (nValue)
	{
		case 0:/*-2*/
			SP0A19_SET_BRIGHTNESS(0x00, 0xe0,0xff);
			break;
		case 1:/*-1*/
			SP0A19_SET_BRIGHTNESS(0x00, 0xf0,0xff);
			break;
		case 2:/*0*/
			SP0A19_SET_BRIGHTNESS(0x00, 0x00,0xff);
			break;
		case 3:/*+1*/
			SP0A19_SET_BRIGHTNESS(0x00, 0x20,0xff);
			break;
		case 4:/*+2*/
			SP0A19_SET_BRIGHTNESS(0x00, 0x30,0xff);
			break;
		default:
			return DD_INVPARAM;
	}

	ret = SP0A19_i2c_write_table((unsigned char *)icSP0A19Brightness, sizeof(icSP0A19Brightness));
	DD_DBG(DD_LOG_CAMERA, "%s ret=%d nValue=%d bAuto=%d\r\n",
		__func__,ret,nValue,bAuto);
	
	return ret;
}

/*****************************************************************************
* Function: SP0A19_set_contrast
*
* Purpose: set contrast.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        nValue              int                            In             contrast
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_contrast(int nValue, BOOL bAuto)
{
	SINT32 ret;
	
	if (bAuto)
		nValue = 2;

	switch (nValue)
	{
		case 0:/*-2*/
			SP0A19_SET_CONTRAST(0x00,SP0A19_P0_0xdd-0x20,SP0A19_P0_0xde-0x20,0xff);
			break;
		case 1:/*-1*/
			SP0A19_SET_CONTRAST(0x00,SP0A19_P0_0xdd-0x10,SP0A19_P0_0xde-0x10,0xff);
			break;
		case 2:/*0*/
			SP0A19_SET_CONTRAST(0x00,SP0A19_P0_0xdd,SP0A19_P0_0xde,0xff);
			break;
		case 3:/*+1*/
			SP0A19_SET_CONTRAST(0x00,SP0A19_P0_0xdd+0x10,SP0A19_P0_0xde+0x10,0xff);
			break;
		case 4:/*+2*/
			SP0A19_SET_CONTRAST(0x00,SP0A19_P0_0xdd+0x20,SP0A19_P0_0xde+0x20,0xff);
			break;
		default:
			return DD_INVPARAM;
	}

	ret = SP0A19_i2c_write_table((unsigned char *)icSP0A19Contrast, sizeof(icSP0A19Contrast));
	DD_DBG(DD_LOG_CAMERA, "%s ret=%d nValue=%d bAuto=%d\r\n",
		__func__,ret,nValue,bAuto);
	return ret;
}

/*****************************************************************************
* Function: SP0A19_set_effect
*
* Purpose: set effect.
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        nValue              int                            In             effect
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_effect(int nValue, BOOL bAuto)
{
	SINT32 ret;

	if (bAuto)
		nValue = CAM_EFFECT_NORMAL;

	switch (nValue)
	{
		case CAM_EFFECT_NORMAL:/*normal*/
			SP0A19_SET_EFFECT(0x00,0x00,0x80,0x80,0xff);
			break;
		case CAM_EFFECT_BW:/*B&W*/
			SP0A19_SET_EFFECT(0x00,0x20,0x80,0x80,0xff);
			break;
		case CAM_EFFECT_REDDISH:/*Redish*/
			SP0A19_SET_EFFECT(0x00,0x10,0xb0,0x90,0xff);
			break;
		case CAM_EFFECT_ANTIQUE:/*Antique*/
			SP0A19_SET_EFFECT(0x00,0x10,0xc0,0x20,0xff);
			break;
		case CAM_EFFECT_GREENISH:/*Greenish*/
			SP0A19_SET_EFFECT(0x00,0x10,0x20,0x20,0xff);
			break;
		case CAM_EFFECT_BLUISH:/*Bluish*/
			SP0A19_SET_EFFECT(0x00,0x10,0x20,0xf0,0xff);
			break;
		case CAM_EFFECT_NEGATIVE:/*Negative*/
			SP0A19_SET_EFFECT(0x00,0x04,0x80,0x80,0xff);
			break;
		default:
			return DD_INVPARAM;
	}

	ret = SP0A19_i2c_write_table((unsigned char *)icSP0A19Effect, sizeof(icSP0A19Effect));
	DD_DBG(DD_LOG_CAMERA, "%s ret=%d nValue=%d bAuto=%d\n",
		__func__,ret,nValue,bAuto);
	
	return ret;
}
/*****************************************************************************
* Function: SP0A19_set_quality
*
* Purpose: set jpeg quality
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        nValue               int                           In             quality
*        nValue;            [IN]Camera quality, 1~100, suggest 75. (0x0080~0x00FF)
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_set_quality(int nValue, BOOL bAuto)
{
	return DD_FAIL;
}

/*****************************************************************************
* Function: SP0A19_get_format_supported
*
* Purpose:  Get Format Supported
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*
* Return: 
*        PCAM_FORMAT_SUPPORTED_DATA 
*
* Note: None.
*******************************************************************************/
static PCAM_FORMAT_SUPPORTED_DATA SP0A19_get_format_supported(CAM_ENUM_CST eStreamType)
{
	return (PCAM_FORMAT_SUPPORTED_DATA)&cCamFormatSupported[eStreamType];

}


/*****************************************************************************
* Function: SP0A19_get_resolution_supported
*
* Purpose:  Get Resolution Supported
*
* Relation: None.
* 
* Params: 
*        Name                Type                         In/Out      Description
*        --------          ----                         ------      -----------
*        eStreamType  CAM_ENUM_CST               In          Camera stream type.
*
* Return: 
*        PCAM_FORMAT_SUPPORTED_DATA 
*
* Note: None.
*******************************************************************************/
static PCAM_RESOLUTION_SUPPORTED_DATA SP0A19_get_resolution_supported(CAM_ENUM_CST eStreamType)
{
	return (PCAM_RESOLUTION_SUPPORTED_DATA)&cCamResolutionSupported[eStreamType];
}
/*****************************************************************************
* Function: SP0A19_i2c_read
*
* Purpose: sensor reg read
*
* Relation: None.
* 
* Params: 
*       Name               Type                   In/Out      	Description
*       --------        ----                  ------        -----------
*       reg            		UINT8             In         		sensor reg address
*
* Return: 
*        -1  : Fail.
*        >0    : sensor reg value.
*
* Note: None.
*******************************************************************************/
static SINT32 SP0A19_i2c_read(UINT8 reg)
{
#if 0
    UINT8 val;
    SENSOR_I2C_CMD cmd;

    cmd.nRegLen  = SENSOR_I2C_CMD_ADDR_8;
    cmd.pReg     = &reg;
    cmd.pBuf     = &val;
    cmd.nBufLen  = 1;
    if (sensor_i2c_read(&cmd) == DD_SUCCESS)
        return val;

    return DD_FAIL;
#endif
   	/*OV SCCB Read operation*/

    UINT8 val;
    SENSOR_I2C_CMD cmd;

    cmd.nRegLen  = SENSOR_I2C_CMD_ADDR_8;
    cmd.pReg     = &reg;
    cmd.pBuf     = NULL;
    cmd.nBufLen  = 0;

    if (sensor_i2c_write(&cmd) == DD_SUCCESS)
    {
        cmd.nRegLen  = SENSOR_I2C_CMD_ADDR_ZERO;
        cmd.pReg     = NULL;
        cmd.pBuf     = &val;
        cmd.nBufLen  = 1;
        if (sensor_i2c_read(&cmd) == DD_SUCCESS)
            return val;
    }

    return DD_FAIL;
}

/*****************************************************************************
* Function: SP0A19_device_id_check
*
* Purpose: sensor device id check
*
* Relation: None.
* 
* Params:  None
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_device_id_check(void)
{
	if(SP0A19_i2c_read(SP0A19_PID_REG) == SP0A19_PID_VAL)	
	{
		return 0;
	}
	else
	{
		return -1;
	}
}


static int  SP0A19_set_nightmode (int nValue, BOOL bAuto)
{
    SINT32 ret = DD_INVPARAM;

    if (bAuto)
        nValue = 0;

    switch (nValue)
    {
        case 0:/*Night Mode off*/
		ret = SP0A19_i2c_write_table((unsigned char *)SP0A19_nightmodedisable , sizeof( SP0A19_nightmodedisable ));
            break;
        case 1:/*Night Mode on*/
           
		ret = SP0A19_i2c_write_table((unsigned char *)SP0A19_nightmodeenable , sizeof(SP0A19_nightmodeenable ));
            break;
        default:
            break;
    }

    DD_DBG(DD_LOG_CAMERA, "%s ret=%d nValue=%d bAuto=%d\r\n",__func__,ret,nValue,bAuto);

    return ret;
}
/*****************************************************************************
* Function: SP0A19_existence_detect
*
* Purpose: sensor existence detect
*
* Relation: None.
* 
* Params: 
*       Name               Type                   In/Out      	Description
*       --------        ----                  ------        -----------
*       pinPwdn            GPIO_ID             In         		power down pin ID
*	    pinReset	       GPIO_ID		       In			    reset pin ID
*
* Return: 
*        -1  : Fail.
*        0    : Success.
*
* Note: None.
*******************************************************************************/
static int SP0A19_existence_detect(GPIO_ID pinPwdn, GPIO_ID pinReset)
{
	int checkResult = 0;
	
	SP0A19_init(pinPwdn, pinReset);

	SP0A19_poweron();			  
	
	/* Reset sensor. */
	SP0A19_reset();
	
	/* Initial I2C. */
	sensor_i2c_init();

	/*check	sensor's device id*/
	checkResult = SP0A19_device_id_check();	 

	SP0A19_standby();

	return checkResult;
}

#endif /* defined(DD_SENSOR_IC_SP0A19) */

