// Here is where all the model specific defines should be


#include "hal_i2c.h"
#include "hal_camera.h"
#include "camera_m.h"

typedef struct
{
    UINT8 Addr;            // Register Address
    UINT8 Data;             // Register Data
    UINT8 Wait;            // Time to wait after write
} CAM_REG_T;

typedef struct
{
    UINT16 Len;             // Size of array
    CAM_REG_T *RegVal;      // Array of registers
} CAM_REG_GROUP_T;

typedef enum
{
    CAM_MODE_IDLE,
    CAM_MODE_VIEWFINDER,
    CAM_MODE_PREP_IMG_CAPTURE,
    CAM_MODE_IMG_CAPTURE,
    CAM_MODE_PREP_VID_CAPTURE,
    CAM_MODE_VID_CAPTURE
} CAM_MODE_T;

// On FPGA platform, Pixel Clock must not be too fast or xfer screws up
#define CAM_CLK_CFG 0x81

#define NORMAL_Y0ffset  0x20
#define LOWLIGHT_Y0ffset  0x25

//AE target
#define  sp0a19_P0_0xf7  0x80
#define  sp0a19_P0_0xf8  0x78
#define  sp0a19_P0_0xf9  0x70
#define  sp0a19_P0_0xfa  0x68
//HEQ
#define  sp0a19_P0_0xdd  0x80
#define  sp0a19_P0_0xde  0xa8
//AWB pre gain
#define  sp0a19_P1_0x28  0xc5
#define  sp0a19_P1_0x29  0x9b
// use this for 320x240 (VGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnQvga[] = 
{//26MHz 16~8fps capture priview
  {0x1C,0x28,1},
  {0x32,0x00,1},
  {0x0f,0x2f,1},
  {0x10,0x2e,1},
  {0x11,0x00,1},
  {0x12,0x18,1},
  {0x13,0x2f,1},
  {0x14,0x00,1},
  {0x15,0x3f,1},
  {0x16,0x00,1},
  {0x17,0x18,1},
  {0x25,0x40,1},
  {0x1a,0x0b,1},
  {0x1b,0xc ,1},
  {0x1e,0xb ,1},
  {0x20,0x3f,1},
  {0x21,0x13,1},
  {0x22,0x19,1},
  {0x26,0x1a,1},
  {0x27,0xab,1},
  {0x28,0xfd,1},
  {0x30,0x00,1},
  {0xfb,0x31,1},
  {0x1f,0x08,1},  
  
//Blacklevel
  {0xfd,0x00,1},
  {0x65,0x00,1},//06
  {0x66,0x00,1},//06
  {0x67,0x00,1},//06
  {0x68,0x00,1},//06
  {0x45,0x00,1},
  {0x46,0x0f,1},
  
 //ae setting //26M 8-16fps  50hz
  {0xfd,0x00,1},
  {0x03,0x00,1},
  {0x04,0xf6,1},
  {0x06,0x00,1},
  {0x09,0x02,1},
  {0x0a,0xdf,1},
  {0xf0,0x52,1},
  {0xf1,0x00,1},
  {0xfd,0x01,1},
  {0x90,0x0c,1},
  {0x92,0x01,1},
  {0x98,0x52,1},
  {0x99,0x00,1},
  {0x9a,0x01,1},
  {0x9b,0x00,1},
  
  //Status
  {0xfd,0x01,1},
  {0xce,0xd8,1},
  {0xcf,0x03,1},
  {0xd0,0xd8,1},
  {0xd1,0x03,1},
   
  {0xfd,0x01,1},
  {0xc4,0x56,1},
  {0xc5,0x8f,1},
  {0xca,0x30,1},
  {0xcb,0x45,1},
  {0xcc,0x70,1},
  {0xcd,0x48,1},
  {0xfd,0x00,1},
  
 //lsc  for st
  {0xfd,0x01,1},
  {0x35,0x15,1},
  {0x36,0x15,1},
  {0x37,0x15,1},
  {0x38,0x15,1},
  {0x39,0x15,1},
  {0x3a,0x15,1},
  {0x3b,0x13,1},
  {0x3c,0x15,1},
  {0x3d,0x15,1},
  {0x3e,0x15,1},
  {0x3f,0x15,1},
  {0x40,0x18,1},
  {0x41,0x00,1},
  {0x42,0x04,1},
  {0x43,0x04,1},
  {0x44,0x00,1},
  {0x45,0x00,1},
  {0x46,0x00,1},
  {0x47,0x00,1},
  {0x48,0x00,1},
  {0x49,0xfd,1},
  {0x4a,0x00,1},
  {0x4b,0x00,1},
  {0x4c,0xfd,1},
  {0xfd,0x00,1},  
//awb 1
  {0xfd,0x01,1},
  {0x28,sp0a19_P1_0x28,1},
  {0x29,sp0a19_P1_0x29,1},
  {0x2e,0x02,1},	
  {0x2f,0x16,1},
  {0x17,0x17,1},
  {0x18,0x19,1},
  {0x19,0x45,1},
  {0x2a,0xef,1},
  {0x2b,0x15,1},  
  //awb2
  {0xfd,0x01,1},
  {0x73,0x80,1},
  {0x1a,0x80,1},
  {0x1b,0x80,1}, 
//d65
  {0x65,0xd5,1},
  {0x66,0xfa,1},
  {0x67,0x72,1},
  {0x68,0x8a,1},
//indoor
  {0x69,0xc6,1},
  {0x6a,0xee,1},
  {0x6b,0x94,1},
  {0x6c,0xab,1},
//f
  {0x61,0x7a,1},
  {0x62,0x98,1},
  {0x63,0xc5,1},
  {0x64,0xe6,1},
//cwf
 {0x6d,0xb9,1},
 {0x6e,0xde,1},
 {0x6f,0xb2,1},
 {0x70,0xd5,1},
 
//////////////////skin detect//////
 {0xfd,0x01,1},
 {0x08,0x15,1},
 {0x09,0x04,1},
 {0x0a,0x20,1},
 {0x0b,0x12,1},
 {0x0c,0x27,1},
 {0x0d,0x06,1},
 {0x0f,0x63,1},  
//BPC_grad
  {0xfd,0x00,1},
  {0x79,0xf0,1},
  {0x7a,0x80,1},  //f0
  {0x7b,0x80,1},   //f0
  {0x7c,0x20,1},    //f0 
  
//smooth
  {0xfd,0x00,1},
	//ͨƽֵ	
  {0x57,0x06,1},	//raw_dif_thr_outdoor
  {0x58,0x0d,1}, //raw_dif_thr_normal
  {0x56,0x10,1}, //raw_dif_thr_dummy
  {0x59,0x10,1}, //raw_dif_thr_lowlight
	//GrGbƽֵ
  {0x89,0x06,1},	//raw_grgb_thr_outdoor 
  {0x8a,0x0d,1}, //raw_grgb_thr_normal  
  {0x9c,0x10,1}, //raw_grgb_thr_dummy   
  {0x9d,0x10,1}, //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  {0x81,0xe0,1},    //raw_gflt_fac_outdoor
  {0x82,0xe0,1}, //80//raw_gflt_fac_normal
  {0x83,0x80,1},    //raw_gflt_fac_dummy
  {0x84,0x40,1},    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  {0x85,0xe0,1}, //raw_gf_fac_outdoor  
  {0x86,0xc0,1}, //raw_gf_fac_normal  
  {0x87,0x80,1}, //raw_gf_fac_dummy   
  {0x88,0x40,1}, //raw_gf_fac_lowlight
	//RBƽǿ  
  {0x5a,0xff,1},		 //raw_rb_fac_outdoor
  {0x5b,0xe0,1}, //40//raw_rb_fac_normal
  {0x5c,0x80,1}, 	 //raw_rb_fac_dummy
  {0x5d,0x00,1}, 	 //raw_rb_fac_lowlight
  
//sharpen 
  {0xfd,0x01,1},
  {0xe2,0x30,1},	//sharpen_y_base
  {0xe4,0xa0,1},	//sharpen_y_max
  
  {0xe5,0x04,1}, //rangek_neg_outdoor
  {0xd3,0x04,1}, //rangek_pos_outdoor   
  {0xd7,0x04,1}, //range_base_outdoor   
  
  {0xe6,0x04,1}, //rangek_neg_normal
  {0xd4,0x04,1}, //rangek_pos_normal 
  {0xd8,0x04,1}, //range_base_normal  
    
  {0xe7,0x08,1}, //rangek_neg_dummy
  {0xd5,0x08,1}, //rangek_pos_dummy
  {0xd9,0x08,1}, //range_base_dummy  	    
  {0xd2,0x10,1}, //rangek_neg_lowlight
  {0xd6,0x10,1}, //rangek_pos_lowlight
  {0xda,0x10,1}, //range_base_lowlight
  {0xe8,0x20,1},//sharp_fac_pos_outdoor
  {0xec,0x35,1},//sharp_fac_neg_outdoor
  {0xe9,0x20,1},//sharp_fac_pos_nr
  {0xed,0x35,1},//sharp_fac_neg_nr
  {0xea,0x20,1},//sharp_fac_pos_dummy
  {0xef,0x30,1},//sharp_fac_neg_dummy
  {0xeb,0x10,1},//sharp_fac_pos_low
  {0xf0,0x20,1},//sharp_fac_neg_low 
  
//CCM
  {0xfd,0x01,1},
  {0xa0,0x80,1},//80(ɫӽɫ)
  {0xa1,0x0 ,1},//0 
  {0xa2,0x0 ,1},//0 
  {0xa3,0xf3,1},//f0
  {0xa4,0x8e,1},//a6
  {0xa5,0x0 ,1},//ea
  {0xa6,0x0 ,1},//0 
  {0xa7,0xe6,1},//e6
  {0xa8,0x9a,1},//9a
  {0xa9,0x0 ,1},//0 
  {0xaa,0x3 ,1},//33
  {0xab,0xc ,1},//c 
  {0xfd,0x00,1},
  
//gamma  
  
 {0xfd,0x00,1},
 {0x8b,0x0 ,1},//0 
 {0x8c,0xC ,1},//11
 {0x8d,0x19,1},//19 
 {0x8e,0x2C,1},//28 
 {0x8f,0x49,1},//46 
 {0x90,0x61,1},//61 
 {0x91,0x77,1},//78 
 {0x92,0x8A,1},//8A 
 {0x93,0x9B,1},//9B 
 {0x94,0xA9,1},//A9 
 {0x95,0xB5,1},//B5 
 {0x96,0xC0,1},//C0 
 {0x97,0xCA,1},//CA 
 {0x98,0xD4,1},//D4 
 {0x99,0xDD,1},//DD 
 {0x9a,0xE6,1},//E6 
 {0x9b,0xEF,1},//EF 
 {0xfd,0x01,1},//01 
 {0x8d,0xF7,1},//F7 
 {0x8e,0xFF,1},//FF 
             
//rpc
  {0xfd,0x00,1},
  {0xe0,0x4c,1},
  {0xe1,0x3c,1},
  {0xe2,0x34,1},
  {0xe3,0x2e,1},
  {0xe4,0x2e,1},
  {0xe5,0x2c,1},
  {0xe6,0x2c,1},
  {0xe8,0x2a,1},
  {0xe9,0x2a,1},
  {0xea,0x2a,1},
  {0xeb,0x28,1},
  {0xf5,0x28,1},
  {0xf6,0x28,1},
//ae min gain  
  {0xfd,0x01,1},
  {0x94,0xa0,1},//rpc_max_indr
  {0x95,0x28,1},//1e//rpc_min_indr 
  {0x9c,0xa0,1},//rpc_max_outdr
  {0x9d,0x28,1},//rpc_min_outdr    

//ae target
  {0xfd,0x00,1}, 
  {0xed, sp0a19_P0_0xf7+0x04,1},
  {0xf7, sp0a19_P0_0xf7,1},
  {0xf8, sp0a19_P0_0xf8,1},
  {0xec, sp0a19_P0_0xf8-0x04,1},
  {0xef, sp0a19_P0_0xf9+0x04,1},
  {0xf9, sp0a19_P0_0xf9,1},
  {0xfa, sp0a19_P0_0xfa,1},
  {0xee, sp0a19_P0_0xfa-0x04,1},

//gray detect
  {0xfd,0x01,1},
  {0x30,0x40,1},
  {0x31,0x70,1},//
  {0x32,0x40,1},//80
  {0x33,0xef,1},//
  {0x34,0x05,1},//04
  {0x4d,0x2f,1},//40
  {0x4e,0x20,1},//
  {0x4f,0x16,1},//13      
  //lowlight lum
  {0xfd,0x00,1}, 
  {0xb2,0x20,1},//lum_limit
  {0xb3,0x1f,1},//lum_set
  {0xb4,0x30,1},//black_vt
  {0xb5,0x45,1},//white_vt
//saturation
  {0xfd,0x00,1},
  {0xbe,0xff,1},
  {0xbf,0x01,1}, 
  {0xc0,0xff,1},
  {0xc1,0xd8,1},
  {0xd3,0x88,1},//0x78
  {0xd4,0x88,1},//0x78
  {0xd6,0x80,1},//0x78      
  {0xd7,0x60,1},//0x78
//HEQ
  {0xfd,0x00,1},
  {0xdc,0x00,1},
  {0xdd,0x80,1},
  {0xde,0xa8,1},//80
  {0xdf,0x80,1},    
//func enable
  {0xfd,0x00,1},
  {0x31,0x00,1},//00
  {0x32,0x15,1},//d
  {0x34,0x76,1},//16
  {0x35,0x40,1},//add by zyy20121011
  {0x33,0xef,1},
  {0x5f,0x51,1},
#if 1
{0xfd , 0x00,1},
{0x30 , 0x10,1},// p1:0x0eʹ1/4sub

{0xfd , 0x01,1},
{0x0e , 0x03,1},//en 1/4sub
#endif
};

// use this for 640x480 (VGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnVga[] =  
{//26MHz 16~8fps capture priview
  {0x1C,0x28,1},
  {0x32,0x00,1},
  {0x0f,0x2f,1},
  {0x10,0x2e,1},
  {0x11,0x00,1},
  {0x12,0x18,1},
  {0x13,0x2f,1},
  {0x14,0x00,1},
  {0x15,0x3f,1},
  {0x16,0x00,1},
  {0x17,0x18,1},
  {0x25,0x40,1},
  {0x1a,0x0b,1},
  {0x1b,0xc ,1},
  {0x1e,0xb ,1},
  {0x20,0x3f,1},
  {0x21,0x13,1},
  {0x22,0x19,1},
  {0x26,0x1a,1},
  {0x27,0xab,1},
  {0x28,0xfd,1},
  {0x30,0x00,1},
  {0xfb,0x31,1},
  {0x1f,0x08,1},  
  
//Blacklevel
  {0xfd,0x00,1},
  {0x65,0x00,1},//06
  {0x66,0x00,1},//06
  {0x67,0x00,1},//06
  {0x68,0x00,1},//06
  {0x45,0x00,1},
  {0x46,0x0f,1},
  
 //ae setting //26M 8-16fps  50hz
  {0xfd,0x00,1},
  {0x03,0x00,1},
  {0x04,0xf6,1},
  {0x06,0x00,1},
  {0x09,0x02,1},
  {0x0a,0xdf,1},
  {0xf0,0x52,1},
  {0xf1,0x00,1},
  {0xfd,0x01,1},
  {0x90,0x0c,1},
  {0x92,0x01,1},
  {0x98,0x52,1},
  {0x99,0x00,1},
  {0x9a,0x01,1},
  {0x9b,0x00,1},
  
  //Status
  {0xfd,0x01,1},
  {0xce,0xd8,1},
  {0xcf,0x03,1},
  {0xd0,0xd8,1},
  {0xd1,0x03,1},
   
  {0xfd,0x01,1},
  {0xc4,0x56,1},
  {0xc5,0x8f,1},
  {0xca,0x30,1},
  {0xcb,0x45,1},
  {0xcc,0x70,1},
  {0xcd,0x48,1},
  {0xfd,0x00,1},
  
 //lsc  for st
  {0xfd,0x01,1},
  {0x35,0x15,1},
  {0x36,0x15,1},
  {0x37,0x15,1},
  {0x38,0x15,1},
  {0x39,0x15,1},
  {0x3a,0x15,1},
  {0x3b,0x13,1},
  {0x3c,0x15,1},
  {0x3d,0x15,1},
  {0x3e,0x15,1},
  {0x3f,0x15,1},
  {0x40,0x18,1},
  {0x41,0x00,1},
  {0x42,0x04,1},
  {0x43,0x04,1},
  {0x44,0x00,1},
  {0x45,0x00,1},
  {0x46,0x00,1},
  {0x47,0x00,1},
  {0x48,0x00,1},
  {0x49,0xfd,1},
  {0x4a,0x00,1},
  {0x4b,0x00,1},
  {0x4c,0xfd,1},
  {0xfd,0x00,1},  
//awb 1
  {0xfd,0x01,1},
  {0x28,sp0a19_P1_0x28,1},
  {0x29,sp0a19_P1_0x29,1},
  {0x2e,0x02,1},	
  {0x2f,0x16,1},
  {0x17,0x17,1},
  {0x18,0x19,1},
  {0x19,0x45,1},
  {0x2a,0xef,1},
  {0x2b,0x15,1},  
  //awb2
  {0xfd,0x01,1},
  {0x73,0x80,1},
  {0x1a,0x80,1},
  {0x1b,0x80,1}, 
//d65
  {0x65,0xd5,1},
  {0x66,0xfa,1},
  {0x67,0x72,1},
  {0x68,0x8a,1},
//indoor
  {0x69,0xc6,1},
  {0x6a,0xee,1},
  {0x6b,0x94,1},
  {0x6c,0xab,1},
//f
  {0x61,0x7a,1},
  {0x62,0x98,1},
  {0x63,0xc5,1},
  {0x64,0xe6,1},
//cwf
 {0x6d,0xb9,1},
 {0x6e,0xde,1},
 {0x6f,0xb2,1},
 {0x70,0xd5,1},
 
//////////////////skin detect//////
 {0xfd,0x01,1},
 {0x08,0x15,1},
 {0x09,0x04,1},
 {0x0a,0x20,1},
 {0x0b,0x12,1},
 {0x0c,0x27,1},
 {0x0d,0x06,1},
 {0x0f,0x63,1},  
//BPC_grad
  {0xfd,0x00,1},
  {0x79,0xf0,1},
  {0x7a,0x80,1},  //f0
  {0x7b,0x80,1},  //f0
  {0x7c,0x20,1},   //f0 
  
//smooth
  {0xfd,0x00,1},
	//ͨƽֵ	
  {0x57,0x06,1},	//raw_dif_thr_outdoor
  {0x58,0x0d,1}, //raw_dif_thr_normal
  {0x56,0x10,1}, //raw_dif_thr_dummy
  {0x59,0x10,1}, //raw_dif_thr_lowlight
	//GrGbƽֵ
  {0x89,0x06,1},	//raw_grgb_thr_outdoor 
  {0x8a,0x0d,1}, //raw_grgb_thr_normal  
  {0x9c,0x10,1}, //raw_grgb_thr_dummy   
  {0x9d,0x10,1}, //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  {0x81,0xe0,1},    //raw_gflt_fac_outdoor
  {0x82,0xe0,1}, //80//raw_gflt_fac_normal
  {0x83,0x80,1},    //raw_gflt_fac_dummy
  {0x84,0x40,1},    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  {0x85,0xe0,1}, //raw_gf_fac_outdoor  
  {0x86,0xc0,1}, //raw_gf_fac_normal  
  {0x87,0x80,1}, //raw_gf_fac_dummy   
  {0x88,0x40,1}, //raw_gf_fac_lowlight
	//RBƽǿ  
  {0x5a,0xff,1},		 //raw_rb_fac_outdoor
  {0x5b,0xe0,1}, //40//raw_rb_fac_normal
  {0x5c,0x80,1}, 	 //raw_rb_fac_dummy
  {0x5d,0x00,1}, 	 //raw_rb_fac_lowlight
  
//sharpen 
  {0xfd,0x01,1},
  {0xe2,0x30,1},	//sharpen_y_base
  {0xe4,0xa0,1},	//sharpen_y_max
  
  {0xe5,0x04,1}, //rangek_neg_outdoor
  {0xd3,0x04,1}, //rangek_pos_outdoor   
  {0xd7,0x04,1}, //range_base_outdoor   
  
  {0xe6,0x04,1}, //rangek_neg_normal
  {0xd4,0x04,1}, //rangek_pos_normal 
  {0xd8,0x04,1}, //range_base_normal  
    
  {0xe7,0x08,1}, //rangek_neg_dummy
  {0xd5,0x08,1}, //rangek_pos_dummy
  {0xd9,0x08,1}, //range_base_dummy  	    
  {0xd2,0x10,1}, //rangek_neg_lowlight
  {0xd6,0x10,1}, //rangek_pos_lowlight
  {0xda,0x10,1}, //range_base_lowlight
  {0xe8,0x20,1},//sharp_fac_pos_outdoor
  {0xec,0x35,1},//sharp_fac_neg_outdoor
  {0xe9,0x20,1},//sharp_fac_pos_nr
  {0xed,0x35,1},//sharp_fac_neg_nr
  {0xea,0x20,1},//sharp_fac_pos_dummy
  {0xef,0x30,1},//sharp_fac_neg_dummy
  {0xeb,0x10,1},//sharp_fac_pos_low
  {0xf0,0x20,1},//sharp_fac_neg_low 
  
//CCM
  {0xfd,0x01,1},
  {0xa0,0x80,1},//80(ɫӽɫ)
  {0xa1,0x0 ,1},//0 
  {0xa2,0x0 ,1},//0 
  {0xa3,0xf3,1},//f0
  {0xa4,0x8e,1},//a6
  {0xa5,0x0 ,1},//ea
  {0xa6,0x0 ,1},//0 
  {0xa7,0xe6,1},//e6
  {0xa8,0x9a,1},//9a
  {0xa9,0x0 ,1},//0 
  {0xaa,0x3 ,1},//33
  {0xab,0xc ,1},//c 
  {0xfd,0x00,1},
  
//gamma  
  
 {0xfd,0x00,1},
 {0x8b,0x0 ,1},//0 
 {0x8c,0xC ,1},//11
 {0x8d,0x19,1},//19 
 {0x8e,0x2C,1},//28 
 {0x8f,0x49,1},//46 
 {0x90,0x61,1},//61 
 {0x91,0x77,1},//78 
 {0x92,0x8A,1},//8A 
 {0x93,0x9B,1},//9B 
 {0x94,0xA9,1},//A9 
 {0x95,0xB5,1},//B5 
 {0x96,0xC0,1},//C0 
 {0x97,0xCA,1},//CA 
 {0x98,0xD4,1},//D4 
 {0x99,0xDD,1},//DD 
 {0x9a,0xE6,1},//E6 
 {0x9b,0xEF,1},//EF 
 {0xfd,0x01,1},//01 
 {0x8d,0xF7,1},//F7 
 {0x8e,0xFF,1},//FF 
             
//rpc
  {0xfd,0x00,1},
  {0xe0,0x4c,1},
  {0xe1,0x3c,1},
  {0xe2,0x34,1},
  {0xe3,0x2e,1},
  {0xe4,0x2e,1},
  {0xe5,0x2c,1},
  {0xe6,0x2c,1},
  {0xe8,0x2a,1},
  {0xe9,0x2a,1},
  {0xea,0x2a,1},
  {0xeb,0x28,1},
  {0xf5,0x28,1},
  {0xf6,0x28,1},
//ae min gain  
  {0xfd,0x01,1},
  {0x94,0xa0,1},//rpc_max_indr
  {0x95,0x28,1},//1e//rpc_min_indr 
  {0x9c,0xa0,1},//rpc_max_outdr
  {0x9d,0x28,1},//rpc_min_outdr    

//ae target
  {0xfd,0x00,1}, 
  {0xed, sp0a19_P0_0xf7+0x04,1},
  {0xf7, sp0a19_P0_0xf7,1},
  {0xf8, sp0a19_P0_0xf8,1},
  {0xec, sp0a19_P0_0xf8-0x04,1},
  {0xef, sp0a19_P0_0xf9+0x04,1},
  {0xf9, sp0a19_P0_0xf9,1},
  {0xfa, sp0a19_P0_0xfa,1},
  {0xee, sp0a19_P0_0xfa-0x04,1},

//gray detect
  {0xfd,0x01,1},
  {0x30,0x40,1},
  {0x31,0x70,1},//
  {0x32,0x40,1},//80
  {0x33,0xef,1},//
  {0x34,0x05,1},//04
  {0x4d,0x2f,1},//40
  {0x4e,0x20,1},//
  {0x4f,0x16,1},//13      
  //lowlight lum
  {0xfd,0x00,1}, 
  {0xb2,0x20,1},//lum_limit
  {0xb3,0x1f,1},//lum_set
  {0xb4,0x30,1},//black_vt
  {0xb5,0x45,1},//white_vt
//saturation
  {0xfd,0x00,1},
  {0xbe,0xff,1},
  {0xbf,0x01,1}, 
  {0xc0,0xff,1},
  {0xc1,0xd8,1},
  {0xd3,0x88,1},//0x78
  {0xd4,0x88,1},//0x78
  {0xd6,0x80,1},//0x78      
  {0xd7,0x60,1},//0x78
//HEQ
  {0xfd,0x00,1},
  {0xdc,0x00,1},
  {0xdd,0x80,1},
  {0xde,0xa8,1},//80
  {0xdf,0x80,1},    
//func enable
  {0xfd,0x00,1},
  {0x31,0x00,1},//00
  {0x32,0x15,1},//d
  {0x34,0x76,1},//16
  {0x35,0x40,1},//add by zyy20121011
  {0x33,0xef,1},
  {0x5f,0x51,1},
};

// use this for 160*120 (QQVGA) capture
PRIVATE const CAM_REG_T RG_InitPowerOnQqvga[] = 
//26MHz fix 10fps video priview
{
  {0x1C,0x28,1},
  {0x32,0x00,1},
  {0x0f,0x2f,1},
  {0x10,0x2e,1},
  {0x11,0x00,1},
  {0x12,0x18,1},
  {0x13,0x2f,1},
  {0x14,0x00,1},
  {0x15,0x3f,1},
  {0x16,0x00,1},
  {0x17,0x18,1},
  {0x25,0x40,1},
  {0x1a,0x0b,1},
  {0x1b,0xc ,1},
  {0x1e,0xb ,1},
  {0x20,0x3f,1},
  {0x21,0x13,1},
  {0x22,0x19,1},
  {0x26,0x1a,1},
  {0x27,0xab,1},
  {0x28,0xfd,1},
  {0x30,0x00,1},
  {0xfb,0x31,1},
  {0x1f,0x08,1},  
  
//Blacklevel
  {0xfd,0x00,1},
  {0x65,0x00,1},//06
  {0x66,0x00,1},//06
  {0x67,0x00,1},//06
  {0x68,0x00,1},//06
  {0x45,0x00,1},
  {0x46,0x0f,1},
  
 //ae setting //26M 10fps  50hz
  {0xfd,0x00,1},
  {0x03,0x00,1},
  {0x04,0xe4,1},
  {0x06,0xfa,1},
  {0x09,0x03,1},
  {0x0a,0x5d,1},
  {0xf0,0x4c,1},
  {0xf1,0x00,1},
  {0xfd,0x01,1},
  {0x90,0x0a,1},
  {0x92,0x01,1},
  {0x98,0x4c,1},
  {0x99,0x00,1},
  {0x9a,0x01,1},
  {0x9b,0x00,1},
  
  //Status
  {0xfd,0x01,1},
  {0xce,0xf8,1},
  {0xcf,0x02,1},
  {0xd0,0xf8,1},
  {0xd1,0x02,1},
   
  {0xfd,0x01,1},
  {0xc4,0x56,1},
  {0xc5,0x8f,1},
  {0xca,0x30,1},
  {0xcb,0x45,1},
  {0xcc,0x70,1},
  {0xcd,0x48,1},
  {0xfd,0x00,1},
  
 //lsc  for st
  {0xfd,0x01,1},
  {0x35,0x15,1},
  {0x36,0x15,1},
  {0x37,0x15,1},
  {0x38,0x15,1},
  {0x39,0x15,1},
  {0x3a,0x15,1},
  {0x3b,0x13,1},
  {0x3c,0x15,1},
  {0x3d,0x15,1},
  {0x3e,0x15,1},
  {0x3f,0x15,1},
  {0x40,0x18,1},
  {0x41,0x00,1},
  {0x42,0x04,1},
  {0x43,0x04,1},
  {0x44,0x00,1},
  {0x45,0x00,1},
  {0x46,0x00,1},
  {0x47,0x00,1},
  {0x48,0x00,1},
  {0x49,0xfd,1},
  {0x4a,0x00,1},
  {0x4b,0x00,1},
  {0x4c,0xfd,1},
  {0xfd,0x00,1},  
//awb 1
  {0xfd,0x01,1},
  {0x28,sp0a19_P1_0x28,1},
  {0x29,sp0a19_P1_0x29,1},
  {0x2e,0x02,1},	
  {0x2f,0x16,1},
  {0x17,0x17,1},
  {0x18,0x19,1},
  {0x19,0x45,1},
  {0x2a,0xef,1},
  {0x2b,0x15,1},  
  //awb2
  {0xfd,0x01,1},
  {0x73,0x80,1},
  {0x1a,0x80,1},
  {0x1b,0x80,1}, 
//d65
  {0x65,0xd5,1},
  {0x66,0xfa,1},
  {0x67,0x72,1},
  {0x68,0x8a,1},
//indoor
  {0x69,0xc6,1},
  {0x6a,0xee,1},
  {0x6b,0x94,1},
  {0x6c,0xab,1},
//f
  {0x61,0x7a,1},
  {0x62,0x98,1},
  {0x63,0xc5,1},
  {0x64,0xe6,1},
//cwf
 {0x6d,0xb9,1},
 {0x6e,0xde,1},
 {0x6f,0xb2,1},
 {0x70,0xd5,1},
 
//////////////////skin detect//////
 {0xfd,0x01,1},
 {0x08,0x15,1},
 {0x09,0x04,1},
 {0x0a,0x20,1},
 {0x0b,0x12,1},
 {0x0c,0x27,1},
 {0x0d,0x06,1},
 {0x0f,0x63,1},  
//BPC_grad
  {0xfd,0x00,1},
  {0x79,0xf0,1},
  {0x7a,0x80,1},  //f0
  {0x7b,0x80,1},  //f0
  {0x7c,0x20,1},   //f0 
  
//smooth
  {0xfd,0x00,1},
	//ͨƽֵ	
  {0x57,0x06,1},	//raw_dif_thr_outdoor
  {0x58,0x0d,1}, //raw_dif_thr_normal
  {0x56,0x10,1}, //raw_dif_thr_dummy
  {0x59,0x10,1}, //raw_dif_thr_lowlight
	//GrGbƽֵ
  {0x89,0x06,1},	//raw_grgb_thr_outdoor 
  {0x8a,0x0d,1}, //raw_grgb_thr_normal  
  {0x9c,0x10,1}, //raw_grgb_thr_dummy   
  {0x9d,0x10,1}, //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  {0x81,0xe0,1},    //raw_gflt_fac_outdoor
  {0x82,0xe0,1}, //80//raw_gflt_fac_normal
  {0x83,0x80,1},    //raw_gflt_fac_dummy
  {0x84,0x40,1},    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  {0x85,0xe0,1}, //raw_gf_fac_outdoor  
  {0x86,0xc0,1}, //raw_gf_fac_normal  
  {0x87,0x80,1}, //raw_gf_fac_dummy   
  {0x88,0x40,1}, //raw_gf_fac_lowlight
	//RBƽǿ  
  {0x5a,0xff,1},		 //raw_rb_fac_outdoor
  {0x5b,0xe0,1}, //40//raw_rb_fac_normal
  {0x5c,0x80,1}, 	 //raw_rb_fac_dummy
  {0x5d,0x00,1}, 	 //raw_rb_fac_lowlight
  
//sharpen 
  {0xfd,0x01,1},
  {0xe2,0x30,1},	//sharpen_y_base
  {0xe4,0xa0,1},	//sharpen_y_max
  
  {0xe5,0x04,1}, //rangek_neg_outdoor
  {0xd3,0x04,1}, //rangek_pos_outdoor   
  {0xd7,0x04,1}, //range_base_outdoor   
  
  {0xe6,0x04,1}, //rangek_neg_normal
  {0xd4,0x04,1}, //rangek_pos_normal 
  {0xd8,0x04,1}, //range_base_normal  
    
  {0xe7,0x08,1}, //rangek_neg_dummy
  {0xd5,0x08,1}, //rangek_pos_dummy
  {0xd9,0x08,1}, //range_base_dummy  	    
  {0xd2,0x10,1}, //rangek_neg_lowlight
  {0xd6,0x10,1}, //rangek_pos_lowlight
  {0xda,0x10,1}, //range_base_lowlight
  {0xe8,0x20,1},//sharp_fac_pos_outdoor
  {0xec,0x35,1},//sharp_fac_neg_outdoor
  {0xe9,0x20,1},//sharp_fac_pos_nr
  {0xed,0x35,1},//sharp_fac_neg_nr
  {0xea,0x20,1},//sharp_fac_pos_dummy
  {0xef,0x30,1},//sharp_fac_neg_dummy
  {0xeb,0x10,1},//sharp_fac_pos_low
  {0xf0,0x20,1},//sharp_fac_neg_low 
  
//CCM
  {0xfd,0x01,1},
  {0xa0,0x80,1},//80(ɫӽɫ)
  {0xa1,0x0 ,1},//0 
  {0xa2,0x0 ,1},//0 
  {0xa3,0xf3,1},//f0
  {0xa4,0x8e,1},//a6
  {0xa5,0x0 ,1},//ea
  {0xa6,0x0 ,1},//0 
  {0xa7,0xe6,1},//e6
  {0xa8,0x9a,1},//9a
  {0xa9,0x0 ,1},//0 
  {0xaa,0x3 ,1},//33
  {0xab,0xc ,1},//c 
  {0xfd,0x00,1},
  
//gamma  
  
 {0xfd,0x00,1},
 {0x8b,0x0 ,1},//0 
 {0x8c,0xC ,1},//11
 {0x8d,0x19,1},//19 
 {0x8e,0x2C,1},//28 
 {0x8f,0x49,1},//46 
 {0x90,0x61,1},//61 
 {0x91,0x77,1},//78 
 {0x92,0x8A,1},//8A 
 {0x93,0x9B,1},//9B 
 {0x94,0xA9,1},//A9 
 {0x95,0xB5,1},//B5 
 {0x96,0xC0,1},//C0 
 {0x97,0xCA,1},//CA 
 {0x98,0xD4,1},//D4 
 {0x99,0xDD,1},//DD 
 {0x9a,0xE6,1},//E6 
 {0x9b,0xEF,1},//EF 
 {0xfd,0x01,1},//01 
 {0x8d,0xF7,1},//F7 
 {0x8e,0xFF,1},//FF 
             
//rpc
  {0xfd,0x00,1},
  {0xe0,0x4c,1},
  {0xe1,0x3c,1},
  {0xe2,0x34,1},
  {0xe3,0x2e,1},
  {0xe4,0x2e,1},
  {0xe5,0x2c,1},
  {0xe6,0x2c,1},
  {0xe8,0x2a,1},
  {0xe9,0x2a,1},
  {0xea,0x2a,1},
  {0xeb,0x28,1},
  {0xf5,0x28,1},
  {0xf6,0x28,1},
//ae min gain  
  {0xfd,0x01,1},
  {0x94,0xa0,1},//rpc_max_indr
  {0x95,0x28,1},//1e//rpc_min_indr 
  {0x9c,0xa0,1},//rpc_max_outdr
  {0x9d,0x28,1},//rpc_min_outdr    

//ae target
  {0xfd,0x00,1}, 
  {0xed, sp0a19_P0_0xf7+0x04,1},
  {0xf7, sp0a19_P0_0xf7,1},
  {0xf8, sp0a19_P0_0xf8,1},
  {0xec, sp0a19_P0_0xf8-0x04,1},
  {0xef, sp0a19_P0_0xf9+0x04,1},
  {0xf9, sp0a19_P0_0xf9,1},
  {0xfa, sp0a19_P0_0xfa,1},
  {0xee, sp0a19_P0_0xfa-0x04,1},

//gray detect
  {0xfd,0x01,1},
  {0x30,0x40,1},
  {0x31,0x70,1},//
  {0x32,0x40,1},//80
  {0x33,0xef,1},//
  {0x34,0x05,1},//04
  {0x4d,0x2f,1},//40
  {0x4e,0x20,1},//
  {0x4f,0x16,1},//13      
  //lowlight lum
  {0xfd,0x00,1}, 
  {0xb2,0x20,1},//lum_limit
  {0xb3,0x1f,1},//lum_set
  {0xb4,0x30,1},//black_vt
  {0xb5,0x45,1},//white_vt
//saturation
  {0xfd,0x00,1},
  {0xbe,0xff,1},
  {0xbf,0x01,1}, 
  {0xc0,0xff,1},
  {0xc1,0xd8,1},
  {0xd3,0x88,1},//0x78
  {0xd4,0x88,1},//0x78
  {0xd6,0x80,1},//0x78      
  {0xd7,0x60,1},//0x78
//HEQ
  {0xfd,0x00,1},
  {0xdc,0x00,1},
  {0xdd,0x80,1},
  {0xde,0xa8,1},//80
  {0xdf,0x80,1},    
//func enable
  {0xfd,0x00,1},
  {0x31,0x00,1},//00
  {0x32,0x15,1},//d
  {0x34,0x76,1},// 16
  {0x35,0x40,1},//add by zyy20121011
  {0x33,0xef,1},
  {0x5f,0x51,1},
//window 480*360
  {0x4c,0x50,1},
  {0x48,0x3c,1},
  {0x4d,0x01,1},
  {0x4e,0xe0,1},
  {0x49,0x01,1},
  {0x4a,0x68,1},
};

PRIVATE const CAM_REG_T RG_VidFormatVga[] = 
{
};

PRIVATE const CAM_REG_T RG_VidFormatQvga[] = 
{
};

PRIVATE const CAM_REG_T RG_VidFormatQqvga[] = 
{
};

typedef struct
{
    // Camera information structure
    char                desc[40];	

    UINT8               snrtype;            //sensor type(RGB or YUV)
    UINT8               pclk;               //use PCLK of sensor
    UINT8               clkcfg;             //[0]: VSync delay enable; [1]: Fifo VSync delay enable; [2]: Use PCLK Negtive Edge.
    HAL_I2C_BPS_T       i2crate;            //I2C rate : KHz				
    UINT8               i2caddress;         //I2C address
    UINT8               i2cispaddress;      //ISP I2C address
    UINT8               pwrcfg;             //sensor power initial configure(SIF REG801 BIT[0]~BIT[2])
    BOOL                rstactiveh;         //sensor reset active hi/low polarity (TRUE = active hi)
    BOOL                snrrst;             //Reset sensor enable

    CAM_MODE_T          cammode;


    BOOL                capturevideo;
    BOOL                captureframe;
    UINT8               numactivebuffers;
    UINT8               currentbuf;

    UINT8               brightness;         //brightness
    UINT8               contrast;           //contrast

    CAM_FORMAT_T        format;
    UINT32              npixels;
    CAM_IMAGE_T         vidbuffer;

    CAM_REG_GROUP_T     standby;            //sensor standby register
    CAM_REG_GROUP_T     Initpoweron;        //SENSOR ISP initial configure

    CAM_REG_GROUP_T     sifpwronseq;        //Sif config sequence(Reg.800 bit0~2) when sensor power on
    CAM_REG_GROUP_T     sifstdbyseq;        //Sif config sequence(Reg.800 bit0~2) when sensor standby
    CAM_REG_GROUP_T     dsif;               //SIF initial configure
    //    TSnrSizeCfgGroup    snrSizeCfg;         //sensor size configure information

    // This 'callback' mechanism is not necessary because the encapsulation is taken
    // care of by the environment
    //    PSensorSetReg       snrSetRegCall;      //set reg callback
    //    PSensorGetReg       snrGetRegCall;
    //    PSnrIdChk           snrIdChkCall;
    //    PSnrMirrorFlip      snrMirrorFlipCall;
    //    PSnrContrast        snrContrastCall;
    //    PSnrBrightness      snrBrightnessCall;
    //    PSnrMode            snrModeCall;
    //    PSnrGetEt           snrGetEt;
    //    PSnrSetEt           snrSetEt;
    //    TFlashParm          flashparm;
    CAM_IRQ_HANDLER_T   camirqhandler;
} CAM_INFO_T;

PRIVATE volatile CAM_INFO_T gSensorInfo =
{
    "SP0A19",      // char desc[40];	

    0,                      // UINT8 snrtype; (rgb)
    1,                      // UINT8 pclk; use or don't use PCLK of sensor???
    0,                      // UINT8 clkcfg; ???
    HAL_I2C_BPS_100K,       // UINT16 i2crate;
    0x21,                   // UINT8 i2caddress; this is the _REAL_ slave address.  0x40 ronlus
    //  The value written in phase 1 of a write cycle will be 0x42
    //  while the value just before the read will be 0x43
    0x21,                   // UINT8 i2cispaddress;
    0x02,                   // UINT8 pwrcfg;???
    TRUE,                   // BOOL rstactiveh;  // active high
    FALSE,                  // BOOL snrrst;

    CAM_MODE_IDLE,          // cammode

    FALSE,                  // Capture video mode
    FALSE,                  // Capturing single frame
    0,                      // Number of active buffers reserved by the upper layer
    0,                      // current frame buffer

    3,                      // UINT8 brightness;
    4,                      // UINT8 contrast;
    CAM_FORMAT_RGB565,      // Format
    CAM_NPIX_VGA,           // npixels(VGA, QVGA, QQVGA...)
    NULL,                   // Video Buffer.  First frame buffer for video or image buffer

    {0, NULL},              // CAM_REG_GROUP_T standby;
    // Should start with QQVGA, but for testing, start with VGA
    //    {sizeof(RG_InitPowerOnQqvga) / sizeof(CAM_REG_T), (CAM_REG_T*)&RG_InitPowerOnQqvga}, // CAM_REG_GROUP_T Initpoweron;
    {sizeof(RG_InitPowerOnVga) / sizeof(CAM_REG_T), (CAM_REG_T*)&RG_InitPowerOnVga}, // CAM_REG_GROUP_T Initpoweron;

    {0, NULL},              // CAM_REG_GROUP_T sifpwronseq;
    {0, NULL},              // CAM_REG_GROUP_T sifstdbyseq;
    {0, NULL},              // CAM_REG_GROUP_T dsif; initial value
    NULL                    // IRQ Callback
};

// ############
// Private
// ############

// These are the commands specific to the SCCB
#define PHASE3_WR_PH1   0x11000     // Start & Write    Slave ID
#define PHASE3_WR_PH2   0x1000      // Write            Slave Register Address
#define PHASE3_WR_PH3   0x1100      // Stop & Write     Slave Register Data

#define PHASE2_WR_PH1   0x11000     // Start & Write    Slave ID
#define PHASE2_WR_PH2   0x1100      // Stop & Write     Slave Register Address (to read)

#define PHASE2_RD_PH1   0x11000     // Start & Write    Slave ID
#define PHASE2_RD_PH2   0x111       // Stop & Read & NACK   Slave Register Read Data

PRIVATE VOID camerap_WriteOneReg( UINT8 Addr, UINT8 Data);
PRIVATE VOID camerap_Delay(UINT16 Wait_mS);
PRIVATE VOID camerap_WriteReg(CAM_REG_T RegWrite);
PRIVATE UINT8 camerap_ReadReg(UINT8 Addr);
PRIVATE VOID camerap_SendRgSeq(const CAM_REG_T *RegList, UINT16 Len);
PRIVATE VOID camerap_InteruptHandler(HAL_CAMERA_IRQ_CAUSE_T cause);
PRIVATE UINT8 camerap_GetId(VOID);

