
//2012-1113 QVGA subsample  ջ

/*------------------------------------------------------------------------------
 * (c) Copyright, Augusta Technology, Inc., 2006-present.
 * (c) Copyright, Augusta Technology USA, Inc., 2006-present.
 *
 * This software, document, web pages, or material (the "Work") is copyrighted
 * by its respective copyright owners.  The Work may be confidential and
 * proprietary.  The Work may be further protected by one or more patents and
 * be protected as a part of a trade secret package.
 *
 * No part of the Work may be copied, photocopied, reproduced, translated, or
 * reduced to any electronic medium or machine-readable form, in whole or in
 * part, without prior written consent of the copyright owner. Any other
 * reproduction in any form without the permission of the copyright owner is
 * prohibited.
 *
 * All Work are protected by the copyright laws of all relevant jurisdictions,
 * including protection under the United States copyright laws, and may not be
 * reproduced, distributed, transmitted, displayed, published, or broadcast
 * without the prior written permission of the copyright owner.
 *
 ------------------------------------------------------------------------------*/


#include <linux/module.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/time.h>
#include <linux/atxx/camera.h>
#include <mach/gpio.h>
#include <mach/pmu.h>
#include <linux/clk.h>
#include <linux/cam.h>
#include <linux/cam_core.h>

#define ATXX_CAM_ID     0xa6
#define ATXX_CAM_BUS    3
#define ATXX_CAM_ADDR   0x21
#define ATXX_CAM_NAME   "sp0a19"
#define ATXX_CAM_SUBDEV ATXX_SUBDEV_FORE_CAM

static int flip_val = 0;

/***********************************************************************
 * Camera Common Function                                              *
 ***********************************************************************/

struct cam_reg {
	unsigned char reg;
	unsigned char val;
};

static int write_regs(struct cam_info *info, const struct cam_reg reglist[])
{
	const struct cam_reg *next = reglist;

	while (!((next->reg == REG_TERM) && (next->val == VAL_TERM)))
	{
		cam_write_byte(info->i2c_dev, next->reg, next->val);
		next++;
	}

	return 0;
}

/***********************************************************************
 * Camera Flip Function                                                *
 ***********************************************************************/

static struct cam_reg cam_reg_flipnone[] = {
	{0xfd, 0x00},
	{0x31, 0x10},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_hflip[] = {
	{0xfd, 0x00},
	{0x31, 0x30},


	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_vflip[] = {
	{0xfd, 0x00},
	{0x31, 0x50},


	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_reg_hvflip[] = {
	{0xfd, 0x00},
	{0x31, 0x70},


	{REG_TERM, VAL_TERM}
};


struct cam_reg* cam_reg_flip[4] =
{
	cam_reg_flipnone,
	cam_reg_hflip,
	cam_reg_vflip,
	cam_reg_hvflip
};

static int cam_flip(struct cam_info *info, int val)
{
	return write_regs(info, cam_reg_flip[val]);
}

/***********************************************************************
 * Camera Initialize Function                                          *
 ***********************************************************************/

static struct cam_reg cam_set_initialize[] = 
{
    {0xfd,0x00},//
	{0x1C,0x28},//
	{0x32,0x00},//
	{0x0f,0x2f},//
	{0x10,0x2e},//
	{0x11,0x00},//
	{0x12,0x18},//
	{0x13,0x2f},//
	{0x14,0x00},//
	{0x15,0x3f},//
	{0x16,0x00},//
	{0x17,0x18},//
	{0x25,0x40},//
	{0x1a,0x0b},//
	{0x1b,0xc },//
	{0x1e,0xb },//
	{0x20,0x3f},//
	{0x21,0x13},//
	{0x22,0x19},//
	{0x26,0x1a},//
	{0x27,0xab},//
	{0x28,0xfd},//
    {0x30,0x00},//
  {0xfb,0x31},
  {0x1f,0x08},  
  
//Blacklevel
  {0xfd,0x00},
  {0x65,0x00},//06
  {0x66,0x00},//06
  {0x67,0x00},//06
  {0x68,0x00},//06
  {0x45,0x00},
  {0x46,0x0f},
  
  //ae setting 
  {0xfd,0x00},
  {0x03,0x01},
  {0x04,0x32},
  {0x06,0x00},
  {0x09,0x01},
  {0x0a,0x46},
  {0xf0,0x66},
  {0xf1,0x00},
  {0xfd,0x01},
  {0x90,0x0c},
  {0x92,0x01},
  {0x98,0x66},
  {0x99,0x00},
  {0x9a,0x01},
  {0x9b,0x00},
  
//Status
  {0xfd,0x01},
  {0xce,0xc8},
  {0xcf,0x04},
  {0xd0,0xc8},
  {0xd1,0x04}, 
	{0xfd,0x01},//
	{0xc4,0x56},//
	{0xc5,0x8f},//
	{0xca,0x30},//
	{0xcb,0x45},//
	{0xcc,0x70},//
	{0xcd,0x48},//
	{0xfd,0x00},//
	{0xfd,0x01},//
	{0x35,0x15},//
	{0x36,0x15},//
	{0x37,0x15},//
	{0x38,0x15},//
	{0x39,0x15},//
	{0x3a,0x15},//
	{0x3b,0x13},//
	{0x3c,0x15},//
	{0x3d,0x15},//
	{0x3e,0x15},//
	{0x3f,0x15},//
	{0x40,0x18},//
	{0x41,0x00},//
	{0x42,0x04},//
	{0x43,0x04},//
	{0x44,0x00},//
	{0x45,0x00},//
	{0x46,0x00},//
	{0x47,0x00},//
	{0x48,0x00},//
	{0x49,0xfd},//
	{0x4a,0x00},//
	{0x4b,0x00},//
	{0x4c,0xfd},//
	{0xfd,0x00},//	
	{0xfd,0x01},//
	{0x28,0xc5},//
	{0x29,0x9b},//
	{0x2e,0x02},//	
	{0x2f,0x16},//
	{0x17,0x17},//
	{0x18,0x19},//
	{0x19,0x45},//
	{0x2a,0xef},//
	{0x2b,0x15},//	
	{0xfd,0x01},//
	{0x73,0x80},//
	{0x1a,0x80},//
	{0x1b,0x80},// 
	{0x65,0xd5},//
	{0x66,0xfa},//
	{0x67,0x72},//
	{0x68,0x8a},//
	{0x69,0xc6},//
	{0x6a,0xee},//
	{0x6b,0x94},//
	{0x6c,0xab},//
	{0x61,0x7a},//
	{0x62,0x98},//
	{0x63,0xc5},//
	{0x64,0xe6},//
	{0x6d,0xb9},//
	{0x6e,0xde},//
	{0x6f,0xb2},//
	{0x70,0xd5},//
	{0xfd,0x01},//
	{0x08,0x15},//
	{0x09,0x04},//
	{0x0a,0x20},//
	{0x0b,0x12},//
	{0x0c,0x27},//
	{0x0d,0x06},//
	{0x0f,0x63},//	
	{0xfd,0x00},//
	{0x79,0xf0},//
	{0x7a,0x80},//f0	
	{0x7b,0x80},//f0	
	{0x7c,0x20},//f0	  
	{0xfd,0x00},//
	{0x57,0x06},//	//raw_dif_thr_outdoor
	{0x58,0x0d},// //raw_dif_thr_normal
	{0x56,0x10},// //raw_dif_thr_dummy
	{0x59,0x10},// //raw_dif_thr_lowlight
	{0x89,0x06},//	//raw_grgb_thr_outdoor 
	{0x8a,0x0d},// //raw_grgb_thr_normal  
	{0x9c,0x10},// //raw_grgb_thr_dummy   
	{0x9d,0x10},// //raw_grgb_thr_lowlight
	{0x81,0xe0},//	  //raw_gflt_fac_outdoor
	{0x82,0xe0},// //80//raw_gflt_fac_normal
	{0x83,0x80},//	  //raw_gflt_fac_dummy
	{0x84,0x40},//	  //raw_gflt_fac_lowlight
	{0x85,0xe0},// //raw_gf_fac_outdoor  
	{0x86,0xc0},// //raw_gf_fac_normal	
	{0x87,0x80},// //raw_gf_fac_dummy	
	{0x88,0x40},// //raw_gf_fac_lowlight 
	{0x5a,0xff},//		 //raw_rb_fac_outdoor
	{0x5b,0xe0},// //40//raw_rb_fac_normal
	{0x5c,0x80},//	 //raw_rb_fac_dummy
	{0x5d,0x00},//	 //raw_rb_fac_lowlight
	{0xfd,0x01},//
	{0xe2,0x30},//	//sharpen_y_base
	{0xe4,0xa0},//	//sharpen_y_max
	{0xe5,0x04},// //rangek_neg_outdoor
	{0xd3,0x04},// //rangek_pos_outdoor   
	{0xd7,0x04},// //range_base_outdoor   
	{0xe6,0x04},// //rangek_neg_normal
	{0xd4,0x04},// //rangek_pos_normal 
	{0xd8,0x04},// //range_base_normal	
	{0xe7,0x08},// //rangek_neg_dummy
	{0xd5,0x08},// //rangek_pos_dummy
	{0xd9,0x08},// //range_base_dummy		
	{0xd2,0x10},// //rangek_neg_lowlight
	{0xd6,0x10},// //rangek_pos_lowlight
	{0xda,0x10},// //range_base_lowlight
	{0xe8,0x20},////sharp_fac_pos_outdoor
	{0xec,0x35},////sharp_fac_neg_outdoor
	{0xe9,0x20},////sharp_fac_pos_nr
	{0xed,0x35},////sharp_fac_neg_nr
	{0xea,0x20},////sharp_fac_pos_dummy
	{0xef,0x30},////sharp_fac_neg_dummy
	{0xeb,0x10},////sharp_fac_pos_low
	{0xf0,0x20},////sharp_fac_neg_low 
	{0xfd,0x01},//
	{0xa0,0x80},////80(ɫӽɫ)
	{0xa1,0x0 },////0 
	{0xa2,0x0 },////0 
	{0xa3,0xf3},////f0
	{0xa4,0x8e},////a6
	{0xa5,0x0 },////ea
	{0xa6,0x0 },////0 
	{0xa7,0xe6},////e6
	{0xa8,0x9a},////9a
	{0xa9,0x0 },////0 
	{0xaa,0x3 },////33
	{0xab,0xc },////c 
	{0xfd,0x00},//
	{0xfd,0x00},//
	{0x8b,0x0 },////0 
	{0x8c,0xC },////11
	{0x8d,0x19},////19 
	{0x8e,0x2C},////28 
	{0x8f,0x49},////46 
	{0x90,0x61},////61 
	{0x91,0x77},////78 
	{0x92,0x8A},////8A 
	{0x93,0x9B},////9B 
	{0x94,0xA9},////A9 
	{0x95,0xB5},////B5 
	{0x96,0xC0},////C0 
	{0x97,0xCA},////CA 
	{0x98,0xD4},////D4 
	{0x99,0xDD},////DD 
	{0x9a,0xE6},////E6 
	{0x9b,0xEF},////EF 
	{0xfd,0x01},////01 
	{0x8d,0xF7},////F7 
	{0x8e,0xFF},////FF 
	{0xfd,0x00},//
	{0xe0,0x4c},//
	{0xe1,0x3c},//
	{0xe2,0x34},//
	{0xe3,0x2e},//
	{0xe4,0x2e},//
	{0xe5,0x2c},//
	{0xe6,0x2c},//
	{0xe8,0x2a},//
	{0xe9,0x2a},//
	{0xea,0x2a},//
	{0xeb,0x28},//
	{0xf5,0x28},//
	{0xf6,0x28},//
	{0xfd,0x01},//
       {0x94,0xa0},//rpc_max_indr
	{0x95,0x28},////1e//rpc_min_indr 
       {0x9c,0xa0},//rpc_max_outdr
	{0x9d,0x28},////rpc_min_outdr	 
	{0xfd,0x00},// 
	{0xed,0x8c},////80 
	{0xf7,0x88},////7c 
	{0xf8,0x80},////70 
	{0xec,0x7c},////6c	
	{0xef,0x74},////99
	{0xf9,0x70},////90
	{0xfa,0x68},////80
	{0xee,0x64},////78	  
	{0xfd,0x01},//
	{0x30,0x40},//
	{0x31,0x70},////
	{0x32,0x40},////80
	{0x33,0xef},////
	{0x34,0x05},////04
	{0x4d,0x2f},////40
	{0x4e,0x20},////
	{0x4f,0x16},////13		
	{0xfd,0x00},// 
	{0xb2,0x20},////lum_limit
	{0xb3,0x1f},////lum_set
	{0xb4,0x30},////black_vt
	{0xb5,0x45},////white_vt
	{0xfd,0x00},//
	{0xbe,0xff},//
	{0xbf,0x01},// 
	{0xc0,0xff},//
	{0xc1,0xd8},//
	{0xd3,0x88},////0x78
	{0xd4,0x88},////0x78
      {0xd6,0x80},//0x78      
	{0xd7,0x60},////0x78
	{0xfd,0x00},//
	{0xdc,0x00},//
	{0xdd,0x80},//
	{0xde,0xa8},////80
	{0xdf,0x80},//	  
	{0xfd,0x00},//
	{0x32,0x15},////d
	{0x34,0x76},//16
	{0x35,0x40},//
	{0x33,0xef},//
	{0x5f,0x51},//
    
    {REG_TERM, VAL_TERM}
};

static struct cam_reg cam_set_preview[] = {

	#if 1	
	{0xfd , 0x00},   	 
	{0x4c , 0x00},	  
	{0x48 , 0x00},	 
	{0x4d , 0x02},	 
	{0x4e , 0x80},	 
	{0x49 , 0x01},	 
	{0x4a , 0xe0},	 
	{0xfd , 0x00}, 	
	{0xfd , 0x00},
	{0x30 , 0x00},	
	{0xfd , 0x01},
	{0x0e , 0x00},
	{0xfd , 0x00},
	{0xe7 , 0x03}, 
	{0xe7 , 0x00},


	#endif
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_set_capture[] = {

	{0xfd , 0x00},// //P0
	{0x31 , 0x00},//00-------fae
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_init[3] =
{
	cam_set_initialize,
	cam_set_preview,
	cam_set_capture
};

static int cam_initialize(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[0]);
}

static int cam_preview(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[1]);
}

static int cam_capture(struct cam_info *info)
{
	return write_regs(info, cam_reg_init[2]);
}

/***********************************************************************
 * Camera Resolution Function                                          *
 ***********************************************************************/

static struct cam_reg cam_resolution_qcif[] = {
	
	

        // sub for qcif	
       {0xfd, 0x00},

	{0x48, 0x90},
	{0x49, 0x01},
	{0x4a, 0x60},

	{0x4c, 0x60},
	{0x4d, 0x01},
	{0x4e, 0x20},

	{0xfd, 0x00},
	{0x30, 0x10},
	
	{0xfd, 0x01},
	{0x0e, 0x03},
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_cif[] = {
	{0xfd, 0x00},

	{0x48, 0x90},
	{0x49, 0x01},
	{0x4a, 0x60},

	{0x4c, 0x60},
	{0x4d, 0x01},
	{0x4e, 0x20},

	{0xfd, 0x00},
	{0x30, 0x00},
	
	{0xfd, 0x01},
	{0x0e, 0x00},
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_qvga[] = {
#if 1
	{0xfd, 0x00},

	{0x48, 0x00},
	{0x49, 0x01},
	{0x4a, 0xe0},

	{0x4c, 0x00},
	{0x4d, 0x02},
	{0x4e, 0x80},

	{0xfd, 0x00},
	{0x30, 0x10},

	{0xfd, 0x01},
	{0x0e, 0x03},
{0xfd , 0x00},
{0xe7 , 0x03}, 
{0xe7 , 0x00},
#endif

#if 0

{0xfd , 0x00}, 
{0x4c , 0x00}, 
{0x48 , 0x00}, 
{0x4d , 0x01}, 
{0x4e , 0x40}, 
{0x49 , 0x00}, 
{0x4a , 0xf0}, 
{0xfd , 0x00}, 


#endif
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_vga[] = {

	{0xfd, 0x00},

	{0x48, 0x00},
	{0x49, 0x01},
	{0x4a, 0xe0},

	{0x4c, 0x00},
	{0x4d, 0x02},
	{0x4e, 0x80},

	{0xfd, 0x00},
	{0x30, 0x00},

	{0xfd, 0x01},
	{0x0e, 0x00},
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_svga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_xga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_sxga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_uxga[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_resolution_qxga[] = {
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_resolution[RES_FLAG_MAX] =
{
	cam_resolution_qcif,
	cam_resolution_cif,
	cam_resolution_qvga,
	cam_resolution_vga,
	cam_resolution_svga,
	cam_resolution_xga,
	cam_resolution_sxga,
	cam_resolution_uxga,
	cam_resolution_qxga
};

static int cam_resolution(struct cam_info *info, int val)
{
	switch (val) {
	case RES_FLAG_QCIF:
		write_regs(info, cam_reg_resolution[RES_FLAG_QCIF]);
		info->res_flags = RES_FLAG_QCIF;
		break;

	case RES_FLAG_CIF:
		write_regs(info, cam_reg_resolution[RES_FLAG_CIF]);
		info->res_flags = RES_FLAG_CIF;
		break;

	case RES_FLAG_QVGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_QVGA]);
		info->res_flags = RES_FLAG_QVGA;
		break;

	case RES_FLAG_VGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_VGA]);
		info->res_flags = RES_FLAG_VGA;
		break;

	case RES_FLAG_SVGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_SVGA]);
		info->res_flags = RES_FLAG_SVGA;
		break;

	case RES_FLAG_XGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_XGA]);
		info->res_flags = RES_FLAG_XGA;
		break;

	case RES_FLAG_SXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_SXGA]);
		info->res_flags = RES_FLAG_SXGA;
		break;

	case RES_FLAG_UXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_UXGA]);
		info->res_flags = RES_FLAG_UXGA;
		break;

	case RES_FLAG_QXGA:
		write_regs(info, cam_reg_resolution[RES_FLAG_QXGA]);
		info->res_flags = RES_FLAG_QXGA;
		break;

	default:
		return -EINVAL;
	}

	return 0;
}

/***********************************************************************
 * Camera Autofocus Function                                           *
 ***********************************************************************/

static struct cam_reg cam_af_single[] = {
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_af_manual[] = {
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_af[2] =
{
	cam_af_single,
	cam_af_manual
};

static int cam_autofocus(struct cam_info *info, int val)
{
	return write_regs(info, cam_reg_af[val]);
}

/***********************************************************************
 * Camera WhiteBalance Function                                        *
 ***********************************************************************/

static struct cam_reg cam_wb_auto[] = {
	//SP0A18_reg_WB_auto
	{0xfd, 0x01},                                                          
	{0x28, 0xc5},		                                                       
	{0x29, 0x9b},
	{0xfd, 0x00},  // AUTO 3000K~7000K   	  
	{0x32, 0x15}, 

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_sunshine[] = {
	// SP0A18_reg_WB_auto   
	{0xfd, 0x00},  //6500K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0xbc},		                                                       
	{0x29, 0x5d},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_cloudy[] = {
	// SP0A18_reg_WB_auto   
	{0xfd, 0x00},   //7000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0xbf},		                                                       
	{0x29, 0x89},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_fluorescence[] = {
	//SP0A18_reg_WB_auto  ӫ 
	{0xfd, 0x00},  //4200K~5000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0xb5},		                                                       
	{0x29, 0xa5},		                                                       
	{0xfd, 0x00},

	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_incandescence[] = {
	// SP0A18_reg_WB_auto ׳ 
	{0xfd, 0x00},  //2800K~3000K                                     
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0x89},		                                                       
	{0x29, 0xb8},		                                                       
	{0xfd, 0x00}, 
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_warm_fluorescence[] = {
	// SP0A18_reg_WB_auto ȹ
	{0xfd, 0x00},  //4000K                                   
	{0x32, 0x05},                                                          
	{0xfd, 0x01},                                                          
	{0x28, 0xaf},		                                                       
	{0x29, 0x99},		                                                       
	{0xfd, 0x00},
	
	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_twilight[] = {


	{REG_TERM, VAL_TERM}
};

static struct cam_reg cam_wb_shade[] = {
	{REG_TERM, VAL_TERM}
};

struct cam_reg* cam_reg_wb[WB_MAX] =
{
	cam_wb_auto,
	cam_wb_sunshine,
	cam_wb_cloudy,
	cam_wb_fluorescence,
	cam_wb_incandescence,
	cam_wb_warm_fluorescence,
	cam_wb_twilight,
	cam_wb_shade
};

static int cam_wb(struct cam_info *info, int val)
{
	switch (val) {
	case WB_AUTO:
		write_regs(info, cam_reg_wb[WB_AUTO]);
		break;

	case WB_SUNSHINE:
		write_regs(info, cam_reg_wb[WB_SUNSHINE]);
		break;

	case WB_CLOUDY:
		write_regs(info, cam_reg_wb[WB_CLOUDY]);
		break;

	case WB_FLUORESCENCE:
		write_regs(info, cam_reg_wb[WB_FLUORESCENCE]);
		break;

	case WB_INCANDESCENCE:
		write_regs(info, cam_reg_wb[WB_INCANDESCENCE]);
		break;

	case WB_WARM_FLUORESCENCE:
		write_regs(info, cam_reg_wb[WB_WARM_FLUORESCENCE]);
		break;

	case WB_TWILIGHT:
		write_regs(info, cam_reg_wb[WB_TWILIGHT]);
		break;

	case WB_SHADE:
		write_regs(info, cam_reg_wb[WB_SHADE]);
		break;

	default:
		return -EINVAL;
	}

	return 0;
}
#if 1
/***********************************************************************
 * Camera Power Control Function                                       *
 ***********************************************************************/

static int cam_gpio_request(void)
{
	gpio_request(GPIO_CAM_PWDN_B, "cam_pwdn_b");
	gpio_request(GPIO_CAM_PWDN_F, "cam_pwdn_f");

	return 0;
}

static int cam_gpio_free(void)
{
	gpio_free(GPIO_CAM_PWDN_B);
	gpio_free(GPIO_CAM_PWDN_F);

	return 0;
}

static void cam_poweron(void)
{
	pmu_power_set(PPS_CAMERA_IO, 2800);
	msleep(10);
	cam_notice("[1103-------------------sp0a19----cam_poweron]");
}

static void cam_poweroff(void)
{
//	pmu_power_set(PPS_CAMERA_IO,0);
//	msleep(5);
 


cam_notice("[1108--------zz---------sp0a19----cam_poweroff]");
}

static void cam_gpio_reset(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	msleep(5);
	gpio_direction_output(GPIO_CAM_PWDN_F, 0);
	msleep(5);
	cam_notice("[1103-------------------sp0a19----cam_gpio_reset]");
}

static void cam_standby_mode(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	gpio_direction_output(GPIO_CAM_PWDN_F, 1);
	msleep(5);
	gpio_direction_input(GPIO_CAM_PWDN_B);
	gpio_direction_input(GPIO_CAM_PWDN_F);
	msleep(5);
	cam_notice("[1103-------------------sp0a19----cam_standby_mode]");
}

static void cam_gpio_preset(void)
{
	gpio_direction_output(GPIO_CAM_PWDN_B, 1);
	msleep(5);
	gpio_direction_output(GPIO_CAM_PWDN_F, 1);
	msleep(5);

	cam_notice("[1103-------------------sp0a19----cam_gpio_preset]");
}

static int detect_poweron(struct cam_info *info)
{
	cam_gpio_preset();
	cam_poweron();
	cam_gpio_reset();
	cam_notice("[1103-------------------sp0a19----detect_poweron]");

	return 0;
}

static int detect_poweroff(struct cam_info *info)
{
	/* after detection finished, poweroff */
	cam_standby_mode();
	cam_poweroff();
	cam_notice("[1103-------------------sp0a19----detect_poweroff]");

	return 0;
}

static int detect_readid(struct cam_info *info)
{
	unsigned int cam_id;

	cam_id = cam_read_byte(info->i2c_dev, 0x02);
	cam_notice("read %s id = 0x%x\n", ATXX_CAM_NAME, cam_id);

	if (cam_id == ATXX_CAM_ID) {
		cam_notice("camera %s detected...\n", ATXX_CAM_NAME);
		return 0;
	}

	return -ENODEV;
}

static int cam_config(struct cam_info *info)
{
	info->minwidth	= QCIF_WIDTH;
	info->minheight	= QCIF_HEIGHT;
	info->maxwidth	= VGA_WIDTH;
	info->maxheight	= VGA_HEIGHT;

	info->bl  = BAND_50;
	info->fps = 25;
	info->night_mode = NIGHT_MODE_OFF;

	return 0;
}

static int cam_open(struct cam_info *info)
{
	clk_enable(info->camera);
	msleep(10);

	cam_gpio_preset();
	cam_poweron();
	cam_gpio_reset();

	cam_initialize(info);
	cam_flip(info, flip_val); 

	info->width  = VGA_WIDTH;
	info->height = VGA_HEIGHT;
	info->res_flags = RES_FLAG_MAX;	
	cam_notice("[1103-------------------sp0a19----cam_open]");

	return 0;
}

static int cam_close(struct cam_info *info)
{
	cam_standby_mode();
	cam_poweroff();

	clk_disable(info->camera);
	cam_notice("[1103-------------------sp0a19----cam_close]");

	return 0;
}
#endif
/***********************************************************************
 * Camera Function List                                                *
 ***********************************************************************/

static struct camera_fmt cam_fmt_list[] = {
	{
		.name	= "4:2:2 planar, Y-Cb-Cr",
		.fourcc	= V4L2_PIX_FMT_YUV422P,
		.depth	= 16,
	}
};

#define N_FORMATS ARRAY_SIZE(cam_fmt_list)

static struct camera_res cam_res_list[] = {
	{
		.name   = "176x144, QCIF",
		.width  = QCIF_WIDTH,
		.height = QCIF_HEIGHT
	}, {
		.name   = "352x288, QVGA",
		.width  = CIF_WIDTH,
		.height = CIF_HEIGHT
	}, {
		.name   = "320x240, QVGA",
		.width  = QVGA_WIDTH,
		.height = QVGA_HEIGHT
	}, {
		.name   = "640x480, VGA",
		.width  = VGA_WIDTH,
		.height = VGA_HEIGHT
	}
};

#define N_RESOLUTIONS ARRAY_SIZE(cam_res_list)

static struct camera_control cam_ctl_list[] = {
	{
		.qc = {
			.id      = V4L2_CID_DO_WHITE_BALANCE,
			.type    = V4L2_CTRL_TYPE_INTEGER,
			.name    = "WhiteBalance",
			.minimum = WB_AUTO,
			.maximum = WB_INCANDESCENCE,
			.step    = 1,
			.default_value = WB_AUTO,
		},
		.cur_value   = WB_AUTO,
		.tweak       = cam_wb,
	}
};

#define N_CONTROLS ARRAY_SIZE(cam_ctl_list)

static const struct i2c_device_id cam_id_table[] = {
	{ ATXX_CAM_NAME,  0 },
	{ }
};

/***********************************************************************
 * Camera Priv Init                                                    *
 ***********************************************************************/

static struct cam_priv priv = {
	.name       = ATXX_CAM_NAME,
	.addr       = ATXX_CAM_ADDR,
	.i2c_bus    = ATXX_CAM_BUS,
	.subdev_id  = ATXX_CAM_SUBDEV,

	.fmt_num    = N_FORMATS,
	.res_num    = N_RESOLUTIONS,
	.ctl_num    = N_CONTROLS,

	.fmt_list   = cam_fmt_list,
	.res_list   = cam_res_list,
	.ctl_list   = cam_ctl_list,
	.id_table   = cam_id_table,

	.Open       = cam_open,
	.Close      = cam_close,

	.Set_Preview     = cam_preview,
	.Set_Capture     = cam_capture,
	.Set_Config      = cam_config,
	.Set_Resolution  = cam_resolution,

	.Request_Gpio    = cam_gpio_request,
	.Free_Gpio       = cam_gpio_free,

	.Detect_PowerON  = detect_poweron,
	.Detect_PowerOFF = detect_poweroff,
	.Detect_ReadId   = detect_readid,
};

static int __init cam_init(void)
{
	init_atxx_cam(&priv);
	cam_notice("[1103--------i2c addr = 0x21--------sp0a19----cam_init]");

	return 0;
}

static void __exit cam_exit(void)
{
	cam_notice("[1103-------------------sp0a19----cam_exit]");
	exit_atxx_cam(&priv);
}

module_param(flip_val, int , 0);
MODULE_PARM_DESC(flip_val, "sp0a19 flip value.");

module_init(cam_init);
module_exit(cam_exit);

MODULE_DESCRIPTION("sp0a19 camera Module");
MODULE_AUTHOR("weiyu Wei");
MODULE_LICENSE("Dual BSD/GPL");


#if 0	//for effect & brightness param

//Effect
switch (para)
    {
        case MEFFECT_OFF:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x00);
		SP0A19_write_cmos_sensor(0x63, 0x80);
		SP0A19_write_cmos_sensor(0x64, 0x80);
            break;

        case MEFFECT_SEPIA:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x10);
		SP0A19_write_cmos_sensor(0x63, 0xc0);
		SP0A19_write_cmos_sensor(0x64, 0x20);

            break;

        case MEFFECT_NEGATIVE:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x04);
		SP0A19_write_cmos_sensor(0x63, 0x80);
		SP0A19_write_cmos_sensor(0x64, 0x80);
            break;

        case MEFFECT_SEPIAGREEN:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x10);
		SP0A19_write_cmos_sensor(0x63, 0x20);
		SP0A19_write_cmos_sensor(0x64, 0x20);
            break;

        case MEFFECT_SEPIABLUE:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x10);
		SP0A19_write_cmos_sensor(0x63, 0x20);
		SP0A19_write_cmos_sensor(0x64, 0xf0);

            break;
			
		case MEFFECT_MONO: //B&W
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0x62, 0x20);
		SP0A19_write_cmos_sensor(0x63, 0x80);
		SP0A19_write_cmos_sensor(0x64, 0x80);

			break;
        default:
            return FALSE;
    }

//Brightness
switch (para)
    {
        case AE_EV_COMP_n13:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0xc0);
            break;

        case AE_EV_COMP_n10:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0xd0);
            break;

        case AE_EV_COMP_n07:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0xe0);
            break;

        case AE_EV_COMP_n03:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0xf0);
            break;

        case AE_EV_COMP_00:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0x00);//0xfa before
            break;

        case AE_EV_COMP_03:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0x10);
            break;

        case AE_EV_COMP_07:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0x20);
            break;

        case AE_EV_COMP_10:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0x30);
            break;

        case AE_EV_COMP_13:
		SP0A19_write_cmos_sensor(0xfd, 0x00);
		SP0A19_write_cmos_sensor(0xdc, 0x40);
            break;

        default:
            return FALSE;
    }

#endif
