/******************************************************************************
 ** File Name:    Sensor_SP0A19.c                                         *
 ** Author:         Mormo                                            	  *
 ** Date:           2011/06/15                                           *
 ** Copyright:    Spreadtrum All Rights Reserved.        *
 ** Description:  Implementation of digital camera register interface       *
 ** Edition: 	    V1.0.0
 ******************************************************************************

 ******************************************************************************
 **                        Edit History                                       *
 ** ------------------------------------------------------------------------- *
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 * $Log$
 * 2011/06/23 Firstly Released By Mormo.
 ******************************************************************************/
#ifndef _SP0A19_C_

#define _SP0A19_C_
#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "i2c_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0A19_I2C_ADDR_W    			0x42
#define SP0A19_I2C_ADDR_R    			0x43
#define SP0A19_I2C_ACK					0x0
LOCAL BOOLEAN  	bl_SP_50Hz_SP0A19		= FALSE;


//AE target
#define  SP0A19_P0_0xf7  0x88
#define  SP0A19_P0_0xf8  0x80
#define  SP0A19_P0_0xf9  0x70
#define  SP0A19_P0_0xfa  0x68
//HEQ
#define  SP0A19_P0_0xdd  0x80
#define  SP0A19_P0_0xde  0xa8

/**---------------------------------------------------------------------------*
 **                         Macro define                                      *
 **---------------------------------------------------------------------------*/
#define    SP0A19_DELAY_MS(ms)            {\
                                              uint32 end = SCI_GetTickCount() + ms;\
                                              while (SCI_GetTickCount() < end)\
                                              ;\
                                          }
                                           

/**---------------------------------------------------------------------------*
 **                     Local Function Prototypes                             *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0A19_Power_On(uint32 power_on);
LOCAL uint32 SP0A19_Identify(void);
LOCAL void    SP0A19_WriteReg( uint8  subaddr, uint8 data );
LOCAL uint8 	SP0A19_ReadReg( uint8  subaddr);
LOCAL void 	SP0A19_Delayms (uint32 ms);
LOCAL uint32 Set_SP0A19_Brightness(uint32 level);
LOCAL uint32 Set_SP0A19_Contrast(uint32 level);
LOCAL uint32 Set_SP0A19_Image_Effect(uint32 effect_type);
LOCAL uint32 Set_SP0A19_Ev(uint32 level);
LOCAL uint32 Set_SP0A19_Anti_Flicker(uint32 mode);
LOCAL uint32 Set_SP0A19_Preview_Mode(uint32 preview_mode);
LOCAL uint32 Set_SP0A19_AWB(uint32 mode);
LOCAL uint32 SP0A19_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable);
LOCAL uint32 SP0A19_Before_Snapshot(uint32 para);
LOCAL uint32 SP0A19_After_Snapshot(uint32 para);
LOCAL uint32 Set_SP0A19_Video_Mode(uint32 mode);
LOCAL void    SP0A19_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr );
/**---------------------------------------------------------------------------*
 ** 						Local Variables 								 *
 **---------------------------------------------------------------------------*/
__align(4) const SENSOR_REG_T SP0A19_YUV_640X480[] = 
{
 //init
		  {0xfd,0x00},
  {0x1C,0x28},
  {0x32,0x00},
  {0x0f,0x2f},
  {0x10,0x2e},
  {0x11,0x00},
  {0x12,0x18},
  {0x13,0x2f},
  {0x14,0x00},
  {0x15,0x3f},
  {0x16,0x00},
  {0x17,0x18},
  {0x25,0x40},
  {0x1a,0x0b},
  {0x1b,0xc },
  {0x1e,0xb },
  {0x20,0x3f},
  {0x21,0x13},
  {0x22,0x19},
  {0x26,0x1a},
  {0x27,0xab},
  {0x28,0xfd},
  {0x30,0x00},
  {0x31,0x10},
  {0xfb,0x31},
  {0x1f,0x08},  
  
//Blacklevel
  {0xfd,0x00},
  {0x65,0x00},//06
  {0x66,0x00},//06
  {0x67,0x00},//06
  {0x68,0x00},//06
               {0x45,0x00},
		  {0x46,0x0f},
  
  //ae setting 
  {0xfd,0x00},
  {0x03,0x01},
  {0x04,0x32},
  {0x06,0x00},
  {0x09,0x01},
  {0x0a,0x46},
  {0xf0,0x66},
  {0xf1,0x00},
  {0xfd,0x01},
  {0x90,0x0c},
  {0x92,0x01},
  {0x98,0x66},
  {0x99,0x00},
  {0x9a,0x01},
  {0x9b,0x00},
  
//Status
  {0xfd,0x01},
  {0xce,0xc8},
  {0xcf,0x04},
  {0xd0,0xc8},
  {0xd1,0x04}, 
   
  {0xfd,0x01},
  {0xc4,0x56},
  {0xc5,0x8f},
  {0xca,0x30},
  {0xcb,0x45},
  {0xcc,0x70},
  {0xcd,0x48},
  {0xfd,0x00},
  
 //lsc  for st
  {0xfd,0x01},
  {0x35,0x15},
  {0x36,0x15},
  {0x37,0x15},
  {0x38,0x15},
  {0x39,0x15},
  {0x3a,0x15},
  {0x3b,0x13},
  {0x3c,0x15},
  {0x3d,0x15},
  {0x3e,0x15},
  {0x3f,0x15},
  {0x40,0x18},
  {0x41,0x00},
  {0x42,0x04},
  {0x43,0x04},
  {0x44,0x00},
  {0x45,0x00},
  {0x46,0x00},
  {0x47,0x00},
  {0x48,0x00},
  {0x49,0xfd},
  {0x4a,0x00},
  {0x4b,0x00},
  {0x4c,0xfd},
  {0xfd,0x00},  
//awb 1
  {0xfd,0x01},
  {0x28,0xc5},
  {0x29,0x9b},
  {0x2e,0x02},	
  {0x2f,0x16},
  {0x17,0x17},
  {0x18,0x19},
  {0x19,0x45},
  {0x2a,0xef},
  {0x2b,0x15},  
  //awb2
  {0xfd,0x01},
  {0x73,0x80},
  {0x1a,0x80},
  {0x1b,0x80}, 
//d65
  {0x65,0xd5},
  {0x66,0xfa},
  {0x67,0x72},
  {0x68,0x8a},
//indoor
  {0x69,0xc6},
  {0x6a,0xee},
  {0x6b,0x94},
  {0x6c,0xab},
//f
  {0x61,0x7a},
  {0x62,0x98},
  {0x63,0xc5},
  {0x64,0xe6},
//cwf
 {0x6d,0xb9},
 {0x6e,0xde},
 {0x6f,0xb2},
 {0x70,0xd5},
 
//////////////////skin detect//////
 {0xfd,0x01},
 {0x08,0x15},
 {0x09,0x04},
 {0x0a,0x20},
 {0x0b,0x12},
 {0x0c,0x27},
 {0x0d,0x06},
 {0x0f,0x63},  
//BPC_grad
  {0xfd,0x00},
  {0x79,0xf0},
		  {0x7a,0x80},  //f0
		  {0x7b,0x80},  //f0
		  {0x7c,0x20},  //f0  
  
//smooth
  {0xfd,0x00},
	//ͨƽֵ	
  {0x57,0x06},	//raw_dif_thr_outdoor
  {0x58,0x0d}, //raw_dif_thr_normal
  {0x56,0x10}, //raw_dif_thr_dummy
  {0x59,0x10}, //raw_dif_thr_lowlight
	//GrGbƽֵ
  {0x89,0x06},	//raw_grgb_thr_outdoor 
  {0x8a,0x0d}, //raw_grgb_thr_normal  
  {0x9c,0x10}, //raw_grgb_thr_dummy   
  {0x9d,0x10}, //raw_grgb_thr_lowlight

	//Gr\Gb֮ƽǿ
  {0x81,0xe0},    //raw_gflt_fac_outdoor
  {0x82,0xe0}, //80//raw_gflt_fac_normal
  {0x83,0x80},    //raw_gflt_fac_dummy
  {0x84,0x40},    //raw_gflt_fac_lowlight
	//GrGbͨƽǿ  
  {0x85,0xe0}, //raw_gf_fac_outdoor  
  {0x86,0xc0}, //raw_gf_fac_normal  
  {0x87,0x80}, //raw_gf_fac_dummy   
  {0x88,0x40}, //raw_gf_fac_lowlight
	//RBƽǿ  
  {0x5a,0xff},		 //raw_rb_fac_outdoor
  {0x5b,0xe0}, //40//raw_rb_fac_normal
  {0x5c,0x80}, 	 //raw_rb_fac_dummy
  {0x5d,0x00}, 	 //raw_rb_fac_lowlight
  
//sharpen 
  {0xfd,0x01},
  {0xe2,0x30},	//sharpen_y_base
  {0xe4,0xa0},	//sharpen_y_max
  
  {0xe5,0x04}, //rangek_neg_outdoor
  {0xd3,0x04}, //rangek_pos_outdoor   
  {0xd7,0x04}, //range_base_outdoor   
  
  {0xe6,0x04}, //rangek_neg_normal
  {0xd4,0x04}, //rangek_pos_normal 
  {0xd8,0x04}, //range_base_normal  
    
  {0xe7,0x08}, //rangek_neg_dummy
  {0xd5,0x08}, //rangek_pos_dummy
  {0xd9,0x08}, //range_base_dummy  	    
  {0xd2,0x10}, //rangek_neg_lowlight
  {0xd6,0x10}, //rangek_pos_lowlight
  {0xda,0x10}, //range_base_lowlight
  {0xe8,0x20},//sharp_fac_pos_outdoor
  {0xec,0x35},//sharp_fac_neg_outdoor
  {0xe9,0x20},//sharp_fac_pos_nr
  {0xed,0x35},//sharp_fac_neg_nr
  {0xea,0x20},//sharp_fac_pos_dummy
  {0xef,0x30},//sharp_fac_neg_dummy
  {0xeb,0x10},//sharp_fac_pos_low
  {0xf0,0x20},//sharp_fac_neg_low 
  
//CCM
  {0xfd,0x01},
  {0xa0,0x80},//80(ɫӽɫ)
  {0xa1,0x0 },//0 
  {0xa2,0x0 },//0 
  {0xa3,0xf3},//f0
  {0xa4,0x8e},//a6
  {0xa5,0x0 },//ea
  {0xa6,0x0 },//0 
  {0xa7,0xe6},//e6
  {0xa8,0x9a},//9a
  {0xa9,0x0 },//0 
  {0xaa,0x3 },//33
  {0xab,0xc },//c 
  {0xfd,0x00},
  
//gamma  
  
 {0xfd,0x00},
 {0x8b,0x0 },//0 
 {0x8c,0xC },//11
 {0x8d,0x19},//19 
 {0x8e,0x2C},//28 
 {0x8f,0x49},//46 
 {0x90,0x61},//61 
 {0x91,0x77},//78 
 {0x92,0x8A},//8A 
 {0x93,0x9B},//9B 
 {0x94,0xA9},//A9 
 {0x95,0xB5},//B5 
 {0x96,0xC0},//C0 
 {0x97,0xCA},//CA 
 {0x98,0xD4},//D4 
 {0x99,0xDD},//DD 
 {0x9a,0xE6},//E6 
 {0x9b,0xEF},//EF 
 {0xfd,0x01},//01 
 {0x8d,0xF7},//F7 
 {0x8e,0xFF},//FF 
             
//rpc
  {0xfd,0x00},
  {0xe0,0x4c},
  {0xe1,0x3c},
  {0xe2,0x34},
  {0xe3,0x2e},
  {0xe4,0x2e},
  {0xe5,0x2c},
  {0xe6,0x2c},
  {0xe8,0x2a},
  {0xe9,0x2a},
  {0xea,0x2a},
  {0xeb,0x28},
  {0xf5,0x28},
  {0xf6,0x28},
//ae min gain  
  {0xfd,0x01},
  {0x94,0xa0},//rpc_max_indr
  {0x95,0x28},//1e//rpc_min_indr 
  {0x9c,0xa0},//rpc_max_outdr
  {0x9d,0x28},//rpc_min_outdr    

//ae target
  {0xfd,0x00}, 
  {0xed,0x8c},//80 
  {0xf7,0x88},//7c 
  {0xf8,0x80},//70 
  {0xec,0x7c},//6c  
  {0xef,0x74},//99
  {0xf9,0x70},//90
  {0xfa,0x68},//80
  {0xee,0x64},//78    
//gray detect
  {0xfd,0x01},
  {0x30,0x40},
  {0x31,0x70},//
  {0x32,0x40},//80
  {0x33,0xef},//
  {0x34,0x05},//04
  {0x4d,0x2f},//40
  {0x4e,0x20},//
  {0x4f,0x16},//13      
  //lowlight lum
  {0xfd,0x00}, 
  {0xb2,0x20},//lum_limit
  {0xb3,0x1f},//lum_set
  {0xb4,0x30},//black_vt
  {0xb5,0x45},//white_vt
//saturation
  {0xfd,0x00},
  {0xbe,0xff},
  {0xbf,0x01}, 
  {0xc0,0xff},
  {0xc1,0xd8},
  {0xd3,0x88},//0x78
  {0xd4,0x88},//0x78
  {0xd6,0x80},//0x78      
  {0xd7,0x60},//0x78
//HEQ
  {0xfd,0x00},
  {0xdc,0x00},
  {0xdd,0x80},
  {0xde,0xa8},//80
  {0xdf,0x80},    
//func enable
  {0xfd,0x00},
  {0x31,0x10},//00
  {0x32,0x15},//d
		  {0x34,0x76},  //16
  {0x33,0xef},
  {0x5f,0x51},

 {SENSOR_WRITE_DELAY, 200},//delay 200ms
  {0xff , 0xff},	
};



LOCAL SENSOR_REG_TAB_INFO_T s_SP0A19_resolution_Tab_YUV[]=
{   
    { ADDR_AND_LEN_OF_ARRAY(SP0A19_YUV_640X480),    640,    480, 24,  SENSOR_IMAGE_FORMAT_YUV422 },
    // YUV422 PREVIEW 1
    { PNULL,                    0,   640,    480,    24   ,        SENSOR_IMAGE_FORMAT_YUV422},
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    { PNULL,                    0,      0,  0  ,        0,        0      },   
    

    // YUV422 PREVIEW 2 
     { PNULL,                   0,      0,  0 ,        0,         0     },  //For BCR Preview 
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      },   
     { PNULL,                   0,      0,  0  ,        0,        0      }

};

 LOCAL uint32 hmirror_enable(uint32 enable)
{
 }

LOCAL uint32 vmirror_enable(uint32 enable)
{
 }



LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0A19_ioctl_func_tab = 
{
// Internal 
    PNULL,
    SP0A19_Power_On, 
    PNULL,
    SP0A19_Identify,
    
    PNULL,          // write register
    PNULL,          // read  register   
    PNULL,
    PNULL,

// External
    PNULL, //set_ae_enable,
    hmirror_enable, //PNULL  pzt 2012-7-26
    vmirror_enable, //PNULL
    
    Set_SP0A19_Brightness,
    Set_SP0A19_Contrast, //write_ev_value,
    PNULL, //read_gain_value,
    PNULL, //write_gain_value,
    Set_SP0A19_Preview_Mode, //read_gain_scale,
    Set_SP0A19_Image_Effect, //set_frame_rate,
    PNULL, //SP0A19_Before_Snapshot,
    PNULL, //SP0A19_After_Snapshot,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    Set_SP0A19_AWB,    
    PNULL, //read_ev_value,
    PNULL, //write_ev_value,
    Set_SP0A19_Ev, //read_gain_value,
    PNULL, //write_gain_value,
    PNULL, //read_gain_scale,
    PNULL, //set_frame_rate,
    
    PNULL,
    PNULL,
    Set_SP0A19_Anti_Flicker,
    Set_SP0A19_Video_Mode,   
    //PNULL   //pzt 2012-7-26
};

 

/**---------------------------------------------------------------------------*
 ** 						Global Variables								  *
 **---------------------------------------------------------------------------*/
PUBLIC SENSOR_INFO_T g_SP0A19_yuv_info =
{
	SP0A19_I2C_ADDR_W,				// salve i2c write address
	SP0A19_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_P|\
	SENSOR_HW_SIGNAL_HSYNC_P,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											

	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		
	
	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,		// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	
	2,								// count of identify code 
	0x02, 0xa6,						// supply two code to identify sensor.
	0x02, 0xa6,						// for Example: index = 0-> Device id, index = 1 -> version id											
											
	SENSOR_AVDD_2800MV,				// voltage of avdd	
	
	640,							// max width of source image
	480,							// max height of source image
	"SP0A19",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0A19_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0A19_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
	SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	3,                     // skip frame num before preview 
	3,                      // skip frame num before capture
	0,                      // deci frame num during preview	
	1                       // deci frame num during video preview
};



/**---------------------------------------------------------------------------*
 **                         Function Definitions                              *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0A19_Power_On(uint32 power_on)
{
    SENSOR_AVDD_VAL_E		dvdd_val=g_SP0A19_yuv_info.dvdd_val;
    SENSOR_AVDD_VAL_E		avdd_val=g_SP0A19_yuv_info.avdd_val;
    SENSOR_AVDD_VAL_E		iovdd_val=g_SP0A19_yuv_info.iovdd_val;  
    BOOLEAN 				power_down=g_SP0A19_yuv_info.power_down_level;	    
    BOOLEAN 				reset_level=g_SP0A19_yuv_info.reset_pulse_level;
    uint32 				reset_width=g_SP0A19_yuv_info.reset_pulse_width;	    
    
    if(SCI_TRUE==power_on)
    {
        Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
        
        GPIO_SetSensorPwdn(!power_down);

        // Open Mclk in default frequency
        Sensor_SetMCLK(24);   //SENSOR_DEFALUT_MCLK

	 Sensor_SetResetLevel(!reset_level);
        SP0A19_Delayms(20);
        Sensor_SetResetLevel(reset_level);
        SCI_Sleep(reset_width);
        Sensor_SetResetLevel(!reset_level);
        SP0A19_Delayms(20);
    }
    else
    {
        GPIO_SetSensorPwdn(power_down);

        Sensor_SetMCLK(SENSOR_DISABLE_MCLK);           

        Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);    
	 SP0A19_Delayms(20);
    }
    
    SCI_TRACE_LOW("set_SP0A19_Power_On");
    return SCI_SUCCESS;
}


/******************************************************************************/
// Description: sensor probe function
// Author:     benny.zou
// Input:      none
// Output:     result
// Return:     0           successful
//             others      failed
// Note:       this function only to check whether sensor is work, not identify 
//              whitch it is!!
/******************************************************************************/
LOCAL uint32 SP0A19_Identify(void)
{
    uint8 reg[2]    = {0x02, 0x02};
    uint8 value[2]  = {0xa6, 0xa6};
    uint8 ret       = 0;
    uint32 i;
    uint8   err_cnt = 0;
    uint32 nLoop = 1000;

    SP0A19_DELAY_MS(10);
	
    for(i = 0; i<2; )
    {
        nLoop = 1000;
        ret = SP0A19_ReadReg(reg[i]);    
		
	SCI_TRACE_LOW( "SENSOR: SP0A19 Read reg0x00 = %x",ret );
		
        if( ret != value[i])
        {
            err_cnt++;
            if(err_cnt>3)
            {
	            	SCI_TRACE_LOW( "SP0A19 Fail" );
	             return 0xFF;
            }
            else
            {
                //Masked by frank.yang,SCI_Sleep() will cause a  Assert when called in boot precedure
                //SCI_Sleep(10);
                while(nLoop--);
                continue;
            }
        }
        err_cnt = 0;
        i++;
    }

	SCI_TRACE_LOW( "SP0A19 Identify" );

	return (uint32)SCI_SUCCESS;

}



LOCAL void SP0A19_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		uint8 cmd[2];
		cmd[0]	=	subaddr;
		cmd[1]	=	data;
		
		I2C_WriteCmdArr(SP0A19_I2C_ADDR_W, cmd, 2, SCI_TRUE);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	SCI_TRACE_LOW("SP0A19 WriteReg reg/value(%x,%x) ", subaddr, data);

}



LOCAL uint8 SP0A19_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
		I2C_WriteCmdArr(SP0A19_I2C_ADDR_W, &subaddr, 1, SCI_TRUE);
		I2C_ReadCmd(SP0A19_I2C_ADDR_R, &value, SCI_TRUE);
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif

	SCI_TRACE_LOW( "SP0A19 ReadReg reg/value(%x,%x) ",subaddr,value );
	return value;
}



LOCAL void SP0A19_Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}



LOCAL void SP0A19_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr )
{
    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

}



__align(4) const SENSOR_REG_T SP0A19_brightness_tab[][2]=
{
       {		
		{0xfd, 0x00},	{0xdc, 0xd0},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0xe0},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0xf0},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0x00},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0x20},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0x30},	{0xff,0xff},
	},

	{
		{0xfd, 0x00},	{0xdc, 0x40},	{0xff,0xff},
	},
};

LOCAL uint32 Set_SP0A19_Brightness(uint32 level)
{
 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_brightness_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    SP0A19_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0A19_brightness: level = %d", level);
    
    return 0;
}



__align(4) const SENSOR_REG_T SP0A19_contrast_tab[][2]=
{
	{
		{0xfd, 0x00},
		{0xdd,  SP0A19_P0_0xdd-0x30},	//level -3
		{0xde,  SP0A19_P0_0xde-0x30},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0A19_P0_0xdd-0x20},	//level -2
		{0xde,  SP0A19_P0_0xde-0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, SP0A19_P0_0xdd-0x10},	//level -1
		{0xde, SP0A19_P0_0xde-0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, SP0A19_P0_0xdd},	//level 0
		{0xde, SP0A19_P0_0xde},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0A19_P0_0xdd+0x10},	//level +1
		{0xde,  SP0A19_P0_0xde+0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0A19_P0_0xdd+0x20},	//level +2
		{0xde,  SP0A19_P0_0xde+0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  SP0A19_P0_0xdd+0x30},	//level +3
		{0xde,  SP0A19_P0_0xde+0x30},
		{0xff, 0xff}
	}, 
};

LOCAL uint32 Set_SP0A19_Contrast(uint32 level)
{
 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_contrast_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    SP0A19_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0A19_contrast: level = %d", level);
    
    return 0;
}



__align(4) const SENSOR_REG_T SP0A19_image_effect_tab[][11]=	
{
    // effect normal
    {
		{0xfd, 0x00},
		{0x62, 0x00},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    //effect BLACKWHITE//GRAYSCALE
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    // effect RED pink
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xb0},
		{0x64, 0x90},
		{0xff, 0xff}
    },
    // effect GREEN//SEPIAGREEN
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x20},
		{0x64, 0x20},
		{0xff, 0xff}
    },
    // effect  BLUE//SEPIABLUE
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x20},
		{0x64, 0xf0},
		{0xff, 0xff}
    },
    // effect  YELLOW//SEPIAYELLOW
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xb0},
		{0x64, 0x00},
		{0xff, 0xff}

    },  
    // effect NEGATIVE//COLORINV
    {	     
		{0xfd, 0x00},
		{0x62, 0x04},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },    
    //effect ANTIQUE//SEPIA
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xc0},
		{0x64, 0x20},
		{0xff, 0xff}
    },
};

LOCAL uint32 Set_SP0A19_Image_Effect(uint32 effect_type)
{
   
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_image_effect_tab[effect_type];

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    SP0A19_Write_Group_Regs(sensor_reg_ptr);
	
    SCI_TRACE_LOW("set_SP0A19_image_effect: effect_type = %d", effect_type);
    
    return 0;
}



__align(4) const SENSOR_REG_T SP0A19_ev_tab[][2]=
{   
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7-0x18+0x04},			
		{0xf7,SP0A19_P0_0xf7-0x18},//level -3
		{0xf8,SP0A19_P0_0xf8-0x18},	
		{0xec,SP0A19_P0_0xf8-0x18-0x04},		
		{0xef,SP0A19_P0_0xf9-0x18+0x04},	
		{0xf9,SP0A19_P0_0xf9-0x18},
		{0xfa,SP0A19_P0_0xfa-0x18},		
		{0xee,SP0A19_P0_0xfa-0x18-0x04},			
		{0xff,0xff},        
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7-0x10+0x04},			
		{0xf7,SP0A19_P0_0xf7-0x10},//level -2
		{0xf8,SP0A19_P0_0xf8-0x10},	
		{0xec,SP0A19_P0_0xf8-0x10-0x04},		
		{0xef,SP0A19_P0_0xf9-0x10+0x04},	
		{0xf9,SP0A19_P0_0xf9-0x10},
		{0xfa,SP0A19_P0_0xfa-0x10},		
		{0xee,SP0A19_P0_0xfa-0x10-0x04},			
		{0xff,0xff},      
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7-0x08+0x04},			
		{0xf7,SP0A19_P0_0xf7-0x08},//level -1
		{0xf8,SP0A19_P0_0xf8-0x08},	
		{0xec,SP0A19_P0_0xf8-0x08-0x04},		
		{0xef,SP0A19_P0_0xf9-0x08+0x04},	
		{0xf9,SP0A19_P0_0xf9-0x08},
		{0xfa,SP0A19_P0_0xfa-0x08},		
		{0xee,SP0A19_P0_0xfa-0x08-0x04},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7+0x04},			
		{0xf7,SP0A19_P0_0xf7},//level 0
		{0xf8,SP0A19_P0_0xf8},	
		{0xec,SP0A19_P0_0xf8-0x04},		
		{0xef,SP0A19_P0_0xf9+0x04},	
		{0xf9,SP0A19_P0_0xf9},
		{0xfa,SP0A19_P0_0xfa},		
		{0xee,SP0A19_P0_0xfa-0x04},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7+0x08+0x04},			
		{0xf7,SP0A19_P0_0xf7+0x08},//level -1
		{0xf8,SP0A19_P0_0xf8+0x08},	
		{0xec,SP0A19_P0_0xf8+0x08-0x04},		
		{0xef,SP0A19_P0_0xf9+0x08+0x04},	
		{0xf9,SP0A19_P0_0xf9+0x08},
		{0xfa,SP0A19_P0_0xfa+0x08},		
		{0xee,SP0A19_P0_0xfa+0x08-0x04},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7+0x10+0x04},			
		{0xf7,SP0A19_P0_0xf7+0x10},//level -2
		{0xf8,SP0A19_P0_0xf8+0x10},	
		{0xec,SP0A19_P0_0xf8+0x10-0x04},		
		{0xef,SP0A19_P0_0xf9+0x10+0x04},	
		{0xf9,SP0A19_P0_0xf9+0x10},
		{0xfa,SP0A19_P0_0xfa+0x10},		
		{0xee,SP0A19_P0_0xfa+0x10-0x04},			
		{0xff,0xff},       
	},
    {
		{0xfd,0x00},   
		{0xed,SP0A19_P0_0xf7+0x18+0x04},			
		{0xf7,SP0A19_P0_0xf7+0x18},//level -3
		{0xf8,SP0A19_P0_0xf8+0x18},	
		{0xec,SP0A19_P0_0xf8+0x18-0x04},		
		{0xef,SP0A19_P0_0xf9+0x18+0x04},	
		{0xf9,SP0A19_P0_0xf9+0x18},
		{0xfa,SP0A19_P0_0xfa+0x18},		
		{0xee,SP0A19_P0_0xfa+0x18-0x04},			
		{0xff,0xff},    
	},
};

LOCAL uint32 Set_SP0A19_Ev(uint32 level)
{
  
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    SCI_ASSERT(level < 7);
 
    SP0A19_Write_Group_Regs(sensor_reg_ptr );

    SCI_TRACE_LOW("set_SP0A19_ev: level = %d", level);

    return 0;
}



/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/
LOCAL uint32 Set_SP0A19_Anti_Flicker(uint32 mode)
{ 
     switch(mode)
    {
        case DCAMERA_FLICKER_50HZ:

		bl_SP_50Hz_SP0A19 = TRUE;
                // SP0A19  24M  50HZ  8-20FPS MAXGAIN
		  SP0A19_WriteReg(0xfd,0x00);
		  SP0A19_WriteReg(0x03,0x01);
		  SP0A19_WriteReg(0x04,0x32);
		  SP0A19_WriteReg(0x06,0x00);
		  SP0A19_WriteReg(0x09,0x01);
		  SP0A19_WriteReg(0x0a,0x46);
		  SP0A19_WriteReg(0xf0,0x66);
		  SP0A19_WriteReg(0xf1,0x00);
		  SP0A19_WriteReg(0xfd,0x01);
		  SP0A19_WriteReg(0x90,0x0c);
		  SP0A19_WriteReg(0x92,0x01);
		  SP0A19_WriteReg(0x98,0x66);
		  SP0A19_WriteReg(0x99,0x00);
		  SP0A19_WriteReg(0x9a,0x01);
		  SP0A19_WriteReg(0x9b,0x00);
		  
		  SP0A19_WriteReg(0xfd,0x01);
		  SP0A19_WriteReg(0xce,0xc8);
		  SP0A19_WriteReg(0xcf,0x04);
		  SP0A19_WriteReg(0xd0,0xc8);
		  SP0A19_WriteReg(0xd1,0x04);
		  SP0A19_WriteReg(0xfd,0x00);   
            break;
            
        case DCAMERA_FLICKER_60HZ:

		bl_SP_50Hz_SP0A19 = FALSE;
              // SP0A19  24M  60HZ  8-20FPS MAXGAIN
		  SP0A19_WriteReg(0xfd,0x00);
		  SP0A19_WriteReg(0x03,0x00);
		  SP0A19_WriteReg(0x04,0xff);
		  SP0A19_WriteReg(0x06,0x00);
		  SP0A19_WriteReg(0x09,0x01);
		  SP0A19_WriteReg(0x0a,0x46);
		  SP0A19_WriteReg(0xf0,0x55);
		  SP0A19_WriteReg(0xf1,0x00);
		  SP0A19_WriteReg(0xfd,0x01);
		  SP0A19_WriteReg(0x90,0x0f);
		  SP0A19_WriteReg(0x92,0x01);
		  SP0A19_WriteReg(0x98,0x55);
		  SP0A19_WriteReg(0x99,0x00);
		  SP0A19_WriteReg(0x9a,0x01);
		  SP0A19_WriteReg(0x9b,0x00);
		  
		  SP0A19_WriteReg(0xfd,0x01);
		  SP0A19_WriteReg(0xce,0xfb);
		  SP0A19_WriteReg(0xcf,0x04);
		  SP0A19_WriteReg(0xd0,0xfb);
		  SP0A19_WriteReg(0xd1,0x04);
		  SP0A19_WriteReg(0xfd,0x00);  			
		//  TODO:
            break;
            
        default:
            break;
    }
      
    SCI_TRACE_LOW("set_SP0A19_anti_flicker-mode=%d",mode);
    OS_TickDelay(100); 

    return 0;
}



LOCAL uint32 Set_SP0A19_Preview_Mode(uint32 preview_mode)
{
	switch (preview_mode)
	{
		case DCAMERA_ENVIRONMENT_NORMAL: 
              {
			SP0A19_WriteReg(0xfd,0x0 );
			SP0A19_WriteReg(0xb2,0x20);
			SP0A19_WriteReg(0xb3,0x1f);
			if(bl_SP_50Hz_SP0A19 == TRUE)
			{
			SCI_TRACE_LOW("normal mode 50hz\r\n");
			//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			  SP0A19_WriteReg(0xfd,0x00);
			  SP0A19_WriteReg(0x03,0x01);
			  SP0A19_WriteReg(0x04,0x32);
			  SP0A19_WriteReg(0x06,0x00);
			  SP0A19_WriteReg(0x09,0x01);
			  SP0A19_WriteReg(0x0a,0x46);
			  SP0A19_WriteReg(0xf0,0x66);
			  SP0A19_WriteReg(0xf1,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0x90,0x0c);
			  SP0A19_WriteReg(0x92,0x01);
			  SP0A19_WriteReg(0x98,0x66);
			  SP0A19_WriteReg(0x99,0x00);
			  SP0A19_WriteReg(0x9a,0x01);
			  SP0A19_WriteReg(0x9b,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0xce,0xc8);
			  SP0A19_WriteReg(0xcf,0x04);
			  SP0A19_WriteReg(0xd0,0xc8);
			  SP0A19_WriteReg(0xd1,0x04);
			  SP0A19_WriteReg(0xfd,0x00);		
			 }
			else	
			{
			SCI_TRACE_LOW("normal mode 60hz\r\n");
			//capture preview daylight 24M 60hz 20-8FPS maxgain:0x70   
			  SP0A19_WriteReg(0xfd,0x00);
			  SP0A19_WriteReg(0x03,0x00);
			  SP0A19_WriteReg(0x04,0xff);
			  SP0A19_WriteReg(0x06,0x00);
			  SP0A19_WriteReg(0x09,0x01);
			  SP0A19_WriteReg(0x0a,0x46);
			  SP0A19_WriteReg(0xf0,0x55);
			  SP0A19_WriteReg(0xf1,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0x90,0x0f);
			  SP0A19_WriteReg(0x92,0x01);
			  SP0A19_WriteReg(0x98,0x55);
			  SP0A19_WriteReg(0x99,0x00);
			  SP0A19_WriteReg(0x9a,0x01);
			  SP0A19_WriteReg(0x9b,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0xce,0xfb);
			  SP0A19_WriteReg(0xcf,0x04);
			  SP0A19_WriteReg(0xd0,0xfb);
			  SP0A19_WriteReg(0xd1,0x04);
			  SP0A19_WriteReg(0xfd,0x00);			
			 }	
			break;
		}	

		case DCAMERA_ENVIRONMENT_NIGHT:
              {
			SP0A19_WriteReg(0xfd,0x0 );
			SP0A19_WriteReg(0xb2,0x25);
			SP0A19_WriteReg(0xb3,0x1f);
			if(bl_SP_50Hz_SP0A19 = FALSE)
			{
			SCI_TRACE_LOW("night mode 50hz\r\n");
			//capture preview night 24M 50hz 20-6FPS maxgain:0x78	 
			  SP0A19_WriteReg(0xfd,0x00);
			  SP0A19_WriteReg(0x03,0x01);
			  SP0A19_WriteReg(0x04,0x32);
			  SP0A19_WriteReg(0x06,0x00);
			  SP0A19_WriteReg(0x09,0x01);
			  SP0A19_WriteReg(0x0a,0x46);
			  SP0A19_WriteReg(0xf0,0x66);
			  SP0A19_WriteReg(0xf1,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0x90,0x10);
			  SP0A19_WriteReg(0x92,0x01);
			  SP0A19_WriteReg(0x98,0x66);
			  SP0A19_WriteReg(0x99,0x00);
			  SP0A19_WriteReg(0x9a,0x01);
			  SP0A19_WriteReg(0x9b,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0xce,0x60);
			  SP0A19_WriteReg(0xcf,0x06);
			  SP0A19_WriteReg(0xd0,0x60);
			  SP0A19_WriteReg(0xd1,0x06);
			  SP0A19_WriteReg(0xfd,0x00);		
			 }
			else
			{
			SCI_TRACE_LOW("night mode 60hz\r\n");
			//capture preview night 24M 60hz 20-6FPS maxgain:0x78
			  SP0A19_WriteReg(0xfd,0x00);
			  SP0A19_WriteReg(0x03,0x00);
			  SP0A19_WriteReg(0x04,0xff);
			  SP0A19_WriteReg(0x06,0x00);
			  SP0A19_WriteReg(0x09,0x01);
			  SP0A19_WriteReg(0x0a,0x46);
			  SP0A19_WriteReg(0xf0,0x55);
			  SP0A19_WriteReg(0xf1,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0x90,0x14);
			  SP0A19_WriteReg(0x92,0x01);
			  SP0A19_WriteReg(0x98,0x55);
			  SP0A19_WriteReg(0x99,0x00);
			  SP0A19_WriteReg(0x9a,0x01);
			  SP0A19_WriteReg(0x9b,0x00);
			  SP0A19_WriteReg(0xfd,0x01);
			  SP0A19_WriteReg(0xce,0xa4);
			  SP0A19_WriteReg(0xcf,0x06);
			  SP0A19_WriteReg(0xd0,0xa4);
			  SP0A19_WriteReg(0xd1,0x06);
			  SP0A19_WriteReg(0xfd,0x00);		
			 }
			break;
		}		
		default:
		{
			break;
		}
      SCI_TRACE_LOW("set_SP0A19_preview_mode: level = %d", preview_mode);
      SP0A19_Delayms(100);
      return 0;
	} 
}



__align(4) const SENSOR_REG_T SP0A19_awb_tab[][8]=
{
	   	//AUTO
	      {
			{0xfd,0x01},                      
			{0x28,0xc5},		                  
			{0x29,0x9b},                      
			{0xfd,0x00}, 		
			{0x32,0x15},   //awb & ae  opened
			{0xfd,0x00},  
		       {0xff,0xff} 
		},    
		//INCANDESCENCE: f
		{
			{0xfd,0x00}, 
			{0x32,0x05},      //awb closed, ae opened           
			{0xfd,0x01},                 
			{0x28,0x89},		             
			{0x29,0xb8},		             
			{0xfd,0x00}, 
			{0xff,0xff} 
		},
		//U30 
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x90},	     
			{0x29,0xc7},	     
			{0xfd,0x00},   	
			{0xff,0xff} 
		},  
		//CWF  
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x9f},	     
			{0x29,0xb8},	     
			{0xfd,0x00}, 		
			{0xff,0xff} 
		},    
		//FLUORESCENT: tl84
		{
			{0xfd,0x00},  
			{0x32,0x05},                  
			{0xfd,0x01},                  
			{0x28,0xaf},		              
			{0x29,0x99},		              
			{0xfd,0x00},		
			{0xff,0xff} 
		},
		//SUN:
		{
			{0xfd,0x00}, 
			{0x32,0x05},           
			{0xfd,0x01},           
			{0x28,0xbc},	         
			{0x29,0x5d},	         
			{0xfd,0x00}, 		
			{0xff,0xff} 
		},
                //CLOUD:d65
		{
			{0xfd,0x00}, 
			{0x32,0x05},          
			{0xfd,0x01},          
			{0x28,0xbf},	        
			{0x29,0x89},	        
			{0xfd,0x00},		
			{0xff,0xff} 
		},	
};

LOCAL uint32 Set_SP0A19_AWB(uint32 mode)
{
	
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_awb_tab[mode];

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	SP0A19_Write_Group_Regs(sensor_reg_ptr);
	
	OS_TickDelay(100); 
	SCI_TRACE_LOW("set_SP0A19_awb_mode: mode = %d", mode);
	
	return 0;
}

LOCAL uint32 SP0A19_AE_AWB_Enable(uint32 ae_enable, uint32 awb_enable)
{
	uint16 ae_value = 0 , awb_value=0;

	SP0A19_WriteReg(0xfd,0x00);
	ae_value = SP0A19_ReadReg(0x32);
	awb_value = SP0A19_ReadReg(0x32);

	if(0x01==ae_enable)
	{
		ae_value |= 0x05;       
	}
	else if(0x00==ae_enable)
	{
		ae_value &= 0xf8;
	}

	if(0x01==awb_enable)
	{
		awb_value |= 0x10;  //0x08 pzt 2012-7-20
	}
	else if(0x00==awb_enable)
	{
		awb_value &= 0xef;  //0xf7;
	}	
	
	SP0A19_WriteReg(0x32, ae_value);
	SP0A19_WriteReg(0x32, awb_value);
	return 0;
}



LOCAL uint32 SP0A19_Before_Snapshot(uint32 para)
{
       #if 0
	uint8 reg_val = 0;
	uint16 exposal_time=0x00;

	SP0A19_AE_AWB_Enable(0x00,0x00);   // close aec awb

	reg_val = SP0A19_ReadReg(0x04);
	exposal_time=reg_val&0x00ff;
	reg_val = SP0A19_ReadReg(0x03);
	exposal_time|=((reg_val&0x00ff)<<0x08);

	/*SP0A19_WriteReg(0x28,0x11);
	exposal_time=exposal_time/2;*/

	if(exposal_time<1)
	{
		exposal_time=1;
	}

	reg_val=exposal_time&0x00ff;
	SP0A19_WriteReg(0x04, reg_val);
	reg_val=(exposal_time&0xff00)>>0x08;
	SP0A19_WriteReg(0x03, reg_val);

	SP0A19_Delayms(400);	
	
	SCI_TRACE_LOW("SENSOR_SP0A19: Before Snapshot");

	return 0;
	#endif
}



LOCAL uint32 SP0A19_After_Snapshot(uint32 para)
{
       #if 0
	//SP0A19_WriteReg(0x28,0x00);

	SP0A19_AE_AWB_Enable(0x01,0x01);   // Open aec awb
	
	SP0A19_Delayms(400);
	
	SCI_TRACE_LOW("SENSOR_SP0A19: After Snapshot");

	return 0;
	#endif
}


 
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0A19_video_mode_nor_tab_50hz[][20]=
{
          //Video preview
           {
			//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			  {0xfd,0x00},
			  {0x03,0x01},
			  {0x04,0x32},
			  {0x06,0x00},
			  {0x09,0x01},
			  {0x0a,0x46},
			  {0xf0,0x66},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x0c},
			  {0x92,0x01},
			  {0x98,0x66},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xc8},
			  {0xcf,0x04},
			  {0xd0,0xc8},
			  {0xd1,0x04},
			  {0xfd,0x00},                   
			  {0xFF,0xFF},	//־	
               },    
//Video record
{
			//Video record daylight 24M 50hz 14-14FPS maxgain:0x80
			  {0xfd,0x00},
			  {0x03,0x00},
			  {0x04,0xd8},
			  {0x06,0x00},
			  {0x09,0x03},
			  {0x0a,0x31},
			  {0xf0,0x48},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x07},
			  {0x92,0x01},
			  {0x98,0x48},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xf8},
			  {0xcf,0x01},
			  {0xd0,0xf8},
			  {0xd1,0x01},
			  {0xfd,0x00},             
			  {0xFF,0xFF},	//־		
},
// UPCC  mode
{
			//Video record daylight 24M 50hz 14-14FPS maxgain:0x80
			  {0xfd,0x00},
			  {0x03,0x00},
			  {0x04,0xd8},
			  {0x06,0x00},
			  {0x09,0x03},
			  {0x0a,0x31},
			  {0xf0,0x48},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x07},
			  {0x92,0x01},
			  {0x98,0x48},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xf8},
			  {0xcf,0x01},
			  {0xd0,0xf8},
			  {0xd1,0x01},
			  {0xfd,0x00},             
                       {0xFF,0xFF},	//־	
}  
};    

__align(4) const SENSOR_REG_T SP0A19_video_mode_nor_tab_60hz[][20]=
{
//Video preview
{
			//capture preview daylight 24M 60Hz 20-8FPS maxgain:0x70   
			  {0xfd,0x00},
			  {0x03,0x00},
			  {0x04,0xff},
			  {0x06,0x00},
			  {0x09,0x01},
			  {0x0a,0x46},
			  {0xf0,0x55},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x0f},
			  {0x92,0x01},
			  {0x98,0x55},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xfb},
			  {0xcf,0x04},
			  {0xd0,0xfb},
			  {0xd1,0x04},
			  {0xfd,0x00},
                       {0xFF,0xFF},	//־
},    
//Video record
{
			//Video record daylight 24M 60Hz 14-14FPS maxgain:0x80
			  {0xfd,0x00},
			  {0x03,0x00},
			  {0x04,0xb4},
			  {0x06,0x00},
			  {0x09,0x03},
			  {0x0a,0x31},
			  {0xf0,0x3c},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x08},
			  {0x92,0x01},
			  {0x98,0x3c},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xe0},
			  {0xcf,0x01},
			  {0xd0,0xe0},
			  {0xd1,0x01},
			  {0xfd,0x00},
                       {0xFF,0xFF},	//־	       
},
// UPCC  mode
{	
			//Video record daylight 24M 60Hz 14-14FPS maxgain:0x80
			  {0xfd,0x00},
			  {0x03,0x00},
			  {0x04,0xb4},
			  {0x06,0x00},
			  {0x09,0x03},
			  {0x0a,0x31},
			  {0xf0,0x3c},
			  {0xf1,0x00},
			  {0xfd,0x01},
			  {0x90,0x08},
			  {0x92,0x01},
			  {0x98,0x3c},
			  {0x99,0x00},
			  {0x9a,0x01},
			  {0x9b,0x00},
			  {0xfd,0x01},
			  {0xce,0xe0},
			  {0xcf,0x01},
			  {0xd0,0xe0},
			  {0xd1,0x01},
			  {0xfd,0x00},
                       {0xFF,0xFF},	//־	     
}  
};

LOCAL uint32 Set_SP0A19_Video_Mode(uint32 mode)
{
    uint8 data=0x00;
    uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = PNULL;
    uint8 tempregval = 0;
   // SCI_TRACE_LOW(" xg:set_SP0A19_video_mode ,%d,%d\r\n",mode,Antiflicker);
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

    if(1==mode ||2==mode )
    {
    	if(bl_SP_50Hz_SP0A19)  // is 50hz in video
    	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_video_mode_nor_tab_50hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_video_mode_nor_tab_50hz[mode];
	}
	else  // is 60hz in video mode
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_video_mode_nor_tab_60hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A19_video_mode_nor_tab_60hz[mode];
	}
	
         SCI_TRACE_LOW("set_SP0A19_video_mode=%d",mode);
    }	

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
    {
    	SP0A19_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;

}


#endif

